/**
 * This module provides methods for Addresses, setting / updating addresses on specific
 * objects like customer  and order are done directly on those objects
 *
 * @preferred
 */
import { FetchableItem,
         FetchablePagedList             } from "./base";
import { EVAService,
         IEVAServiceResponse            } from "../service";

export class AddressBook extends FetchablePagedList<EVA.Core.Services.AddressBookDto, EVA.Core.Services.ListAddressBook, EVA.Core.Services.ListAddressBookResponse>
{
    private userId: number;

    public static setDefaultBillingAddress( id: number ): Promise<IEVAServiceResponse<EVA.Core.Services.SetDefaultBillingAddress, EVA.Core.RequestMessageWithEmptyResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            var service = new EVAService<EVA.Core.Services.SetDefaultBillingAddress, EVA.Core.RequestMessageWithEmptyResponse>( "SetDefaultBillingAddress" );

            // Build the request data
            // The base class will add things like SessionID so keep typescript
            // happy we're using an 'any' type object here
            //
            var requestData: any = {
                AddressBookID: id
            };

            service.call( requestData )
            .then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.SetDefaultBillingAddress, EVA.Core.RequestMessageWithEmptyResponse> ) =>
            {
                resolve( serviceResult.response );
            } )
            .catch( ( error: any ) =>
            {
                reject( error );
            } );
        } );
    }

    public static setDefaultShippingAddress( id: number ): Promise<IEVAServiceResponse<EVA.Core.Services.SetDefaultShippingAddress, EVA.Core.RequestMessageWithEmptyResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            var service = new EVAService<EVA.Core.Services.SetDefaultShippingAddress, EVA.Core.RequestMessageWithEmptyResponse>( "SetDefaultShippingAddress" );

            // Build the request data
            // The base class will add things like SessionID so keep typescript
            // happy we're using an 'any' type object here
            //
            var requestData: any = {
                AddressBookID: id
            };

            service.call( requestData )
            .then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.SetDefaultShippingAddress, EVA.Core.RequestMessageWithEmptyResponse> ) =>
            {
                resolve( serviceResult.response );
            } )
            .catch( ( error: any ) =>
            {
                reject( error );
            } );
        } );
    }

    public static createItem( address: EVA.Core.AddressDto, description?: string ): Promise<IEVAServiceResponse<EVA.Core.Services.CreateAddressBookItem, EVA.Core.CreateResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create our service instance
            //
            var service = new EVAService<EVA.Core.Services.CreateAddressBookItem, EVA.Core.CreateResponse>( "CreateAddressBookItem" );

            // Build the request data
            // The base class will add things like SessionID so to keep typescript
            // happy we're using an 'any' type object here
            //
            var requestData: any = {
                Description: description || "",
                Address: address
            };

            // Perform the call and capture the resulting promise
            //
            service.call( requestData )
            .then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.CreateAddressBookItem, EVA.Core.CreateResponse> ) =>
            {
                resolve( serviceResult.response );
            } )
            .catch( reject );
        } );
    }

    public static updateItem( id: number, address: EVA.Core.AddressDto, description?: string ): Promise<IEVAServiceResponse<EVA.Core.Services.UpdateAddressBookItem, EVA.Core.EmptyResponseMessage>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create our service instance
            //
            var service = new EVAService<EVA.Core.Services.UpdateAddressBookItem, EVA.Core.EmptyResponseMessage>( "UpdateAddressBookItem" );

            // Build the request data
            // The base class will add things like SessionID so to keep typescript
            // happy we're using an 'any' type object here
            //
            var requestData: any = {
                ID: id,
                Description: description || "",
                Address: address
            };

            // Perform the call and capture the resulting promise
            //
            service.call( requestData )
            .then( () =>
            {
                resolve();
            } )
            .catch( reject );
        } );
    }

    public static deleteItem( id: number ): Promise<IEVAServiceResponse<EVA.Core.Services.DeleteAddressBookItem, EVA.Core.EmptyResponseMessage>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create our service instance
            //
            var service = new EVAService<EVA.Core.Services.DeleteAddressBookItem, EVA.Core.EmptyResponseMessage>( "DeleteAddressBookItem" );

            // Build the request data
            // The base class will add things like SessionID so to keep typescript
            // happy we're using an 'any' type object here
            //
            var requestData: any = {
                ID: id
            };

            // Perform the call and capture the resulting promise
            //
            service.call( requestData )
            .then( () =>
            {
                resolve();
            } )
            .catch( reject );
        } );
    }

    constructor( limit: number = 15, start: number = 0, sortDirection: EVA.Core.SortDirection = 0, sortProperty: string = null ) {
        super( limit, start, sortDirection, sortProperty );
    }

    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.ListAddressBook, EVA.Core.Services.ListAddressBookResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Core.Services.ListAddressBook, EVA.Core.Services.ListAddressBookResponse>>
    {
        // Build the request data
        //
        var requestData: any = {
            UserID: this.userId
        };

        return super.fetch( "ListAddressBook", requestData );
    }
}

export namespace Addresses
{
    "use strict";

    /**
     * Wrapper class for Address responses, since this is not a fetchable item
     * it's just a plain object
     */
    export class Address {

        public data: EVA.Core.AddressDto;

        public setData( response: any ): EVA.Core.AddressDto {
            return this.data = response.Result;
        }
    }

    /**
     * Perform a lookup using zipcode and housenumber, resolve with an Address object
     *
     * @param  {string}  zipCode     zipcode to lookup
     * @param  {string}  houseNumber housenumber to lookup
     * @return {Promise}             [description]
     */
    export function lookup( zipCode: string, houseNumber: string ): Promise<Address> {

        var service = new EVAService<EVA.Core.Services.GetAddressForZipCode, EVA.Core.Services.GetAddressForZipCodeResponse>( "GetAddressForZipCode" );

        // Build the request data
        // The base class will add things like SessionID so keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: any = {
            ZipCode:        zipCode,
            HouseNumber:    houseNumber
        };

        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Perform the call and capture the resulting promise
            //
            service.call( requestData ).then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.GetAddressForZipCode, EVA.Core.Services.GetAddressForZipCodeResponse> ) =>
            {
                let address = new Address();
                address.setData( serviceResult.response );

                resolve( address );
            } )
            .catch( ( error ) =>
            {
                reject( error );
            });
        } );
    }

    /**
     * Request a list of available countries for a user to create addresses with
     * @return {Promise}
     */
    export function getAvailableCountries(): Promise<EVA.Core.Services.CountryDto[]> {
        var service = new EVAService<EVA.Core.Services.ListCountries, EVA.Core.Services.ListCountriesResponse>( "ListCountries" );

        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Perform the call and capture the resulting promise
            //
            service.call().then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.ListCountries, EVA.Core.Services.ListCountriesResponse> ) =>
            {
                resolve( serviceResult.response );
            } )
            .catch( ( error ) =>
            {
                reject( error );
            });
        } );
    }

    /**
     * Perform an address search using a single text string, responds with textual suggestions.
     * To get the details of the suggestion (city/street etc) a additional call
     * needs to be made to getAddressSuggestionDetails()
     *
     * @param {string}      query       The search query
     * @return {Promise}    array of found suggestions
     */
    export function getAddressSuggestions( query: string ): Promise<EVA.Core.AddressSuggestion[]> {
        var service = new EVAService<EVA.Core.Services.AutocompleteAddress, EVA.Core.Services.AutocompleteAddressResponse>( "AutocompleteAddress" );

        // Build the request data
        // The base class will add things like SessionID so keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: any = {
            Query:        query
        };

        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Perform the call and capture the resulting promise
            //
            service.call( requestData ).then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.AutocompleteAddress, EVA.Core.Services.AutocompleteAddressResponse> ) =>
            {
                resolve( serviceResult.response );
            } )
            .catch( ( error ) =>
            {
                reject( error );
            } );
        } );
    }

    /**
     * Retrieve the details of a suggestion
     *
     * @param {EVA.Core.AddressSuggestion}      suggestion      the address suggestion, can be retrieved using the getAddressSuggestions() function
     * @return {Promise}    the details of the address suggestion
     */
    export function getAddressSuggestionDetails( suggestion: EVA.Core.AddressSuggestion ): Promise<EVA.Core.AddressSuggestionDetails> {
        var service = new EVAService<EVA.Core.Services.GetAutocompleteAddressByReference, EVA.Core.Services.GetAutocompleteAddressByReferenceResponse>( "GetAutocompleteAddressByReference" );

        // Build the request data
        // The base class will add things like SessionID so keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: any = {
            Suggestion:        suggestion
        };

        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Perform the call and capture the resulting promise
            //
            service.call( requestData ).then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.GetAutocompleteAddressByReference, EVA.Core.Services.GetAutocompleteAddressByReferenceResponse> ) =>
            {
                resolve( serviceResult.response );
            } )
            .catch( ( error ) =>
            {
                reject( error );
            });
        } );
    }
}
