/**
 * The base module declares reusable base classes for items, lists, etc.
 * @preferred
 */

import { EVAService,
         IEVAServiceResponse    } from "../service";
import { EXMLHttpRequestMethod  } from "../xhr";

export interface IPageInfo {
    currentPageNumber:  number;
    totalNumberOfPages: number;
    totalNumberOfItems: number;
}

/**
 * Provides access to an item available on the EVA back-end
 */
export class FetchableItem<T, REQ, RES> {
    /**
     * The item detail data. This is available after a fetch from the back-end.
     * It is exposed globally so clients can bind and watch it directly for updates
     * @type {T}
     */
    public data:          T;

    /**
     * The unique identifier of the item. Used when retrieving an exsting item from the back-end.
     * Defaults to a number but some items use string based identifiers
     * @type {number | string}
     */
    protected _id:         number | string;

    /**
     * This class wraps an item from EVA and the operations you can perform on it.
     * You should supply the item identifier when creating an instance of this class
     *
     * @param  {number | string}    id      The identifier of the item to retrieve
     * @return {void}                       This method doesn't return anything
     */
    constructor( id?: number | string ) {
        this.id = id;
    }

    /**
     * Getter for the internal id property of this item so it is possible
     * to override the behaviour of determining the value of the id
     *
     * @return {number | string} the identifiable value of this object
     */
    get id(): number | string {
        return this._id;
    }

    /**
     * Set for the id of the fetchable item. This can be overriden in a subclass so
     * additional behaviour can be implemented when setting the id
     *
     * @param  {number | string}      id An item identifier
     * @return {void}                    This item doesn't return anything
     */
    set id( id: number | string ) {
        this._id = id;
    }

    /**
     * Returns the current item identifier
     * @return {number | string} An item identifier
     */
    public getId(): number | string {
        return this._id;
    }

    /**
     * Returns the current item details. Only available after a successful fetch
     * The details are also exposed direct as the .data attribute for easy binding
     * @return {T} The item details
     */
    public getData(): T {
        return this.data;
    }

    /**
     * Sets the current item details using the fetched service response.
     * Extending classes should override this method to pull the details from response
     * @param  {RES} response The response from the service call
     * @return {T}            The item details
     */
    public setData( response: any ): T {
        // The default response is usually EVA.Core.GetResponse
        // An extending class will know exactly what is returned and where
        // the data for <T> can be found
        //
        return this.data = response.Result;
    }

    /**
     * Fetches the item details from the EVA back-end
     * @param  {string}  serviceName The name of the EVA Service to call
     * @param  {REQ}     requestData The request data for service call
     * @param  {string}  path        The request path (rest-like) if not using post message
     * @return {Promise<IEVAServiceResponse<REQ, RES>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch( serviceName: string, requestData: REQ, path?: string, method?: EXMLHttpRequestMethod ): Promise<IEVAServiceResponse<REQ, RES>> {

        // Create our service instance
        //
        var service = new EVAService<REQ, RES>( serviceName, path, method );

        // Perform the call and capture the resulting promise
        //
        return new Promise( ( resolve, reject ) => {
            service.call( <REQ>requestData ).then( ( serviceResult: IEVAServiceResponse<REQ, RES> ) =>
            {
                this.setData( serviceResult.response );
                resolve( serviceResult );
            } )
            .catch( reject );
        } );
    }
}

/**
 * Provides access to a pageable list available on the EVA back-end
 */
export class FetchablePagedList<T, REQ, RES> {
    /**
     * The page item data. This is available after a fetch from the back-end.
     * It is exposed globally so clients can bind and watch it directly for updates
     * @type {Arra<T>}
     */
    public data:                  Array<T>;

    /**
     * The page configuration instance used to instruct the EVA back-end which
     * data to fetch
     * @type {EVA.Core.PageConfig}
     */
    protected pageConfig:         EVA.Core.PageConfig;

    /**
     * The current page for which the data is available
     * Only available after fetch has been called
     * @type {number}
     */
    protected currentPageNumber:  number;

    /**
     * The total number of pages in the list
     * Only available after fetch has been called
     * @type {number}
     */
    protected totalNumberOfPages: number;

    /**
     * The total number of items in the list
     * Only available after fetch has been called
     * @type {number}
     */
    protected totalNumberOfItems: number;

    /**
     * The list filter applied to the list
     * This may differ from the list of requested filter in the page configuration
     * Only available after fetch has been called
     * @type {any}
     */
    protected appliedFilter: any;

    /**
     * This class wraps a paged result list from EVA and the operations you can perform on it.
     * You can optionally supply page configuration values to start somewhere other then
     * the start of the list
     *
     * @param  {number}                         start           The item to start the list at. Defaults to 0 and is a 0-based index.
     * @param  {number}                         limit           The page size limit. Defaults to 15
     * @param  {EVA.Core.SortDirection}  sortDirection   The sorting direction for the list. Defaults to ascending
     * @return {void}                                           This method doesn't return anything
     */
    constructor( limit: number = 15, start: number = 0, sortDirection: EVA.Core.SortDirection = 0, sortProperty: string = "product_id" ) {

        // Initialise the page configuration
        //
        this.pageConfig = {
            Start:          start,
            Limit:          limit,
            SortDirection:  sortDirection,
            SortProperty:   sortProperty,
            Filter:         {}
        };
    }

    /**
     * Returns the current page size limit
     * @return {number}       The configured page size limit
     */
    public getLimit(): number {
        return this.pageConfig.Limit;
    }

    /**
     * Set the page size limit
     * @param  {number} limit The new page size limit
     * @return {number}       The configured page size limit
     */
    public setLimit( limit: number ): number {
        return this.pageConfig.Limit = limit;
    }

    /**
     * Return the current starting item for the list
     * @return {number}       The configured starting point for the list
     */
    public getStart(): number {
        return this.pageConfig.Start;
    }

    /**
     * Set the starting point for the list
     * @param  {number} start The new starting point for the list
     * @return {number}       The configured starting point for the list
     */
    public setStart( start: number ): number {
        return this.pageConfig.Start = start;
    }

    /**
     * Return the current property being sorted on
     * Can be undefined which means the server decides
     * @return {string}      The configured sorting property
     */
    public getSortProperty(): string {
        return this.pageConfig.SortProperty;
    }

    /**
     * Set the property to sort on
     * @param  {string} name The new sorting property
     * @return {string}      The configured sorting property
     */
    public setSortProperty( name: string ): string {
        return this.pageConfig.SortProperty = name;
    }

    /**
     * Returns the current sort direction
     * @return {string}      The configured sorting property
     */
    public getSortDirection(): EVA.Core.SortDirection {
        return this.pageConfig.SortDirection;
    }

    /**
     * Sets the direction to sort the list on
     * @param  {EVA.Core.SortDirection} direction The new sort direction
     * @return {string}      The configured sorting property
     */
    public setSortDirection( direction: EVA.Core.SortDirection ): EVA.Core.SortDirection {
        return this.pageConfig.SortDirection = direction;
    }

    /**
     * Set the filter that is to be applied to the list
     * @param  {any} filter The new filter structure
     * @return {any}        The configured filter
     */
    public setFilter( filter: any ): any {
        return this.pageConfig.Filter = filter;
    }

    /**
     * Returns the current filter for the list
     * @return {any}        The configured filter
     */
    public getFilter(): any {
        return this.pageConfig.Filter;
    }

    /**
     * Returns the current filter applied for the list
     * Only after available after fetch has been called
     * The applied filter can differ from the requested filter depending on how
     * the EVA back-end and its products are configured
     * @return {any}        The applied filter
     */
    public getAppliedFilter(): any {
        return this.appliedFilter;
    }

    /**
     * Returns all the current page configuration parameters
     * These determine which part of the list data is returned
     * @return {EVA.Core.PageConfig} The current page configuration
     */
    public getPageConfiguration(): EVA.Core.PageConfig {
        return this.pageConfig;
    }

    /**
     * Sets all the current page configuration parameters at once
     * These determine which part of the list data is returned
     * @param  {EVA.Core.PageConfig} pageConfig The page configuration details
     * @return {EVA.Core.PageConfig}            The new page configuration details
     */
    public setPageConfiguration( pageConfig: EVA.Core.PageConfig ): EVA.Core.PageConfig {
        return this.pageConfig = pageConfig;
    }

    /**
     * Returns the current page information which entails the current page, total number of pages and total number of items
     * @return {IPageInfo}  The current page information
     */
    public getPageInfo(): IPageInfo {
        return {
            currentPageNumber:  this.currentPageNumber,
            totalNumberOfPages: this.totalNumberOfPages,
            totalNumberOfItems: this.totalNumberOfItems
        };
    }

    /**
     * Returns the current page item details. Only available after a successful fetch
     * The details are also exposed direct as the .data attribute for easy binding
     * @return {Array<T>} An array of item details
     */
    public getData(): T[] {
        return this.data;
    }

    /**
     * Sets the current item details using the fetched service response.
     * Extending classes should override this method to pull the details from response
     * @param  {RES}        response    The response from the service call
     * @return {Array<T>}               The item details
     */
    public setData( response: any ): T[] {
        // The default response is usually EVA.Core.PagedResultGeneric
        // An extending class will know exactly what is returned and where
        // the data for <T> can be found
        //
        // Store the paging response information
        //
        this.currentPageNumber  = response.Result.CurrentPage;
        this.totalNumberOfPages = response.Result.NumberOfPages;
        this.totalNumberOfItems = response.Result.Total;
        this.appliedFilter      = response.Result.Filters;

        // Set and return the page items
        //
        return this.data = response.Result.Page;
    }

     /**
      * Fetches the list of items from the EVA back-end
      * @param  {string}  serviceName The name of the EVA Service to call
      * @param  {REQ}     requestData The request data for service call
      * @return {Promise<IEVAServiceResponse<REQ, RES>>} A promise resolving when the fetch from the back-end has completed
      */
    public fetch( serviceName: string, requestData: REQ, path?: string, method?: EXMLHttpRequestMethod ): Promise<IEVAServiceResponse<REQ, RES>> {

        // Create our service instance
        //
        var service = new EVAService<REQ, RES>( serviceName, path, method );

        // Perform the call and capture the resulting promise
        //
        return new Promise( ( resolve, reject ) => {
            service.call( <REQ>requestData ).then( ( serviceResult: IEVAServiceResponse<REQ, RES> ) =>
            {
                this.setData( serviceResult.response );
                resolve( serviceResult );
            } )
            .catch( reject );
        } );
    }
}
