/**
 * The applications module provides access to the applications data from the EVA back-end
 * @preferred
 */

import { FetchableItem                     } from "./base";
import { SettingsManager                   } from "../settings";
import { EVAService, IEVAServiceResponse   } from "../service";
import { EXMLHttpRequestMethod             } from "../xhr";

/**
 * Provides access to a babylon binary blobs in EVA
 */
export class Blob extends FetchableItem<EVA.Blobs.BlobDto, EVA.Blobs.GetBlobInfo, EVA.Blobs.GetBlobInfoResponse> {
    /**
     * The optional category for the blob
     * @type {string}
     */
    private category: string;

    /**
     * The optional original name for the blob
     * @type {string}
     */
    private originalName: string;

    /**
     * Convenience method to build a blob image URL
     * @param  {string} guid        The blob guid
     * @param  {number} width       The output image width
     * @param  {number} height      The output image height
     * @param  {string} type        The image type (ie. png, jpg, etc)
     *
     * @return {string}             An image URL
     */
    public static getImageUrl( guid: string, width = 100, height = 100, format = "png" )
    {
        return SettingsManager.getEndPointURL() + `/image/${Math.round(width)}/${Math.round(height)}/${guid}.${format}`;
    }

    /**
     * Sets the category to retrieve the blob info for
     * @param  {string} category    The blob category
     * @return {void}               This method doesn't return anything
     */
    public setCategory( category: string )
    {
        this.category = category;
    }

    /**
     * Sets the original name to retrieve the blob info for
     * @param  {string} original name    The blob original name
     * @return {void}               This method doesn't return anything
     */
    public setOriginalName( originalName: string )
    {
        this.originalName = originalName;
    }

    public fetch(): Promise<IEVAServiceResponse<EVA.Blobs.GetBlobInfo, EVA.Blobs.GetBlobInfoResponse>> {

        // Build the request data
        //
        var requestData: EVA.Blobs.GetBlobInfo = <EVA.Blobs.GetBlobInfo>{
            Guid:           this.getId()
        ,   Category:       this.category
        ,   OriginalName:   this.originalName
        };

        if ( this.getId() === undefined )
        {
            return Promise.reject( {
                status:     404,
                response:   "Not found"
            } );
        }
        else
        {
            return super.fetch( "GetBlobInfo", requestData, "/api/v1/blob/" + this.getId() + "/info", EXMLHttpRequestMethod.GET );
        }
    }

    /**
     * Sets the current item details using the fetched service response.
     * Extending classes should override this method to pull the details from response
     * @param  {RES} response The response from the service call
     * @return {T}            The item details
     */
    public setData( response: any ): EVA.Blobs.BlobDto
    {
        // The default response is usually EVA.Core.GetResponse
        // An extending class will know exactly what is returned and where
        // the data for <T> can be found
        //
        return this.data = response;
    }

    public create( newBlobInfo: EVA.Blobs.StoreBlob ): Promise<IEVAServiceResponse<EVA.Blobs.GetBlobInfo, EVA.Blobs.GetBlobInfoResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create the service instance
            //
            var service = new EVAService<EVA.Blobs.StoreBlob, EVA.Blobs.StoreBlobResponse>( "StoreBlob", "/api/v1/blob", EXMLHttpRequestMethod.POST );

            // Perform the call and capture the resulting promise
            //
            service.call( newBlobInfo )
            .then( ( createResult: IEVAServiceResponse<EVA.Blobs.StoreBlob, EVA.Blobs.StoreBlobResponse> ) =>
            {
                // Collect the ID from the response
                //
                this.id = createResult.response.Guid;

                // Return the fetch of the newly created blob
                //
                this.fetch().then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }

    public createEmpty( newBlobInfo: EVA.Blobs.CreateBlob ): Promise<IEVAServiceResponse<EVA.Blobs.CreateBlob, string>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create the service instance
            //
            var service = new EVAService<EVA.Blobs.CreateBlob, EVA.Blobs.CreateBlobResponse>( "CreateBlob", "/api/v1/blob/empty", EXMLHttpRequestMethod.POST );

            // Perform the call and capture the resulting promise
            //
            service.call( newBlobInfo )
            .then( ( createResult: IEVAServiceResponse<EVA.Blobs.CreateBlob, EVA.Blobs.CreateBlobResponse> ) =>
            {
                // Collect the ID from the response
                //
                this.id = createResult.response.Guid;

                // Resolve with the upload URL
                //
                resolve( createResult.response.Url );
            } )
            .catch( reject );
        } );
    }

    public createEmptyBlob( newBlobInfo: EVA.Blobs.CreateBlob ): Promise<IEVAServiceResponse<EVA.Blobs.CreateBlob, EVA.Blobs.CreateBlobResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create the service instance
            //
            var service = new EVAService<EVA.Blobs.CreateBlob, EVA.Blobs.CreateBlobResponse>( "CreateBlob", "/api/v1/blob/empty", EXMLHttpRequestMethod.POST );

            // Perform the call and capture the resulting promise
            //
            service.call( newBlobInfo )
            .then( ( createResult: IEVAServiceResponse<EVA.Blobs.CreateBlob, EVA.Blobs.CreateBlobResponse> ) =>
            {
                // Collect the ID from the response
                //
                this.id = createResult.response.Guid;

                // Resolve with the upload URL
                //
                resolve( createResult.response );
            } )
            .catch( reject );
        } );
    }

    public getUploadUrl( guid: string )
    {
        return SettingsManager.getEndPointURL() + `/blob/${guid}`;
    }

    public process( imageDataBase64: string, type?: string ): Promise<IEVAServiceResponse<EVA.Blobs.ProcessDocument, EVA.Blobs.ProcessDocumentResponse>>
    {
        // Create the service instance
        //
        var service = new EVAService<EVA.Blobs.ProcessDocument, EVA.Blobs.ProcessDocumentResponse>( "ProcessDocument", "/api/v1/blob/process", EXMLHttpRequestMethod.POST );

        // Build the request data
        // The base class will add things like SessionID so keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: any = {
            Data: imageDataBase64
        ,   Type: type
        };

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData );
    }

    public destroy(): Promise<IEVAServiceResponse<EVA.Blobs.DeleteBlob, EVA.Core.EmptyResponseMessage>>
    {
        // Create the service instance
        //
        var service = new EVAService<EVA.Blobs.DeleteBlob, EVA.Core.EmptyResponseMessage>( "DeleteBlob", `/api/v1/blob/${this.getId()}`, EXMLHttpRequestMethod.DELETE );

        // Build the request data
        // The base class will add things like SessionID so keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: any = {
        };

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData );
    }

    public getImageCutoutSuggestions(): Promise<IEVAServiceResponse<EVA.Babylon.GetImageCutoutSuggestions, EVA.Babylon.GetImageCutoutSuggestionsResponse>>
    {
        // Create the service instance
        //
        var service = new EVAService<EVA.Babylon.GetImageCutoutSuggestions, EVA.Babylon.GetImageCutoutSuggestionsResponse>( "GetImageCutoutSuggestions" );

        // Build the request data
        // The base class will add things like SessionID so keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: any = {
            BlobID: this.getId()
        };

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData );
    }
}
