/**
 * The applications module provides access to the applications data from the EVA back-end
 * @preferred
 */

import { FetchableItem, FetchablePagedList } from "./base";
import { EVAService, IEVAServiceResponse   } from "../service";
import { EXMLHttpRequestMethod             } from "../xhr";

/**
 * Provides access to a babylon Brands (fashion) in EVA
 */
export class Brand extends FetchableItem<EVA.Core.Services.BrandDto, EVA.PIM.Core.GetBrand, EVA.PIM.Core.GetBrandResponse> {

    public fetch(): Promise<IEVAServiceResponse<EVA.PIM.Core.GetBrand, EVA.PIM.Core.GetBrandResponse>> {

        // Build the request data
        //
        var requestData: EVA.PIM.Core.GetBrand = <EVA.PIM.Core.GetBrand>{
            ID: this.getId()
        };

        return super.fetch( "GetBrand", requestData );
    }

    public create( name: string ): Promise<IEVAServiceResponse<EVA.PIM.Core.GetBrand, EVA.PIM.Core.GetBrandResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create the service instance
            //
            var service = new EVAService<EVA.Babylon.CreateFashionBrand, EVA.Babylon.CreateFashionBrandResponse>( "CreateFashionBrand", "/api/v1/fashion/brand", EXMLHttpRequestMethod.POST );

            // Build the request data
            // The base class will add things like SessionID so keep typescript
            // happy we're using an 'any' type object here
            //
            var requestData: any = {
                Name: name
            };

            // Perform the call and capture the resulting promise
            //
            service.call( requestData )
            .then( ( createResult: IEVAServiceResponse<EVA.Babylon.CreateFashionBrand, EVA.Babylon.CreateFashionBrandResponse> ) =>
            {
                // Collect the ID from the response
                //
                this.id = createResult.response.BrandID;

                // Return the fetch of the newly created collection
                //
                this.fetch().then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }
}

export class Brands extends FetchablePagedList<EVA.Core.Services.BrandDto, EVA.Core.Services.ListBrands, EVA.Core.Services.ListBrandsResponse>
{
    /**
     * Sets the current item details using the fetched service response.
     * Extending classes should override this method to pull the details from response
     * @param  {RES}        response    The response from the service call
     * @return {Array<T>}               The item details
     */
    public setData( response: any ): EVA.Core.Services.BrandDto[]
    {
        // The default response is usually EVA.Core.PagedResultGeneric
        // An extending class will know exactly what is returned and where
        // the data for <T> can be found
        //
        // Store the paging response information
        //
        this.currentPageNumber  = 1;
        this.totalNumberOfPages = 1;
        this.totalNumberOfItems = response.Result.length;
        this.appliedFilter      = {};

        // Set and return the page items
        //
        return this.data = response.Result;
    }

    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.ListBrands, EVA.Core.Services.ListBrandsResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Core.Services.ListBrands, EVA.Core.Services.ListBrandsResponse>>
    {
        // Build the request data
        //
        var requestData: any = {};

        return super.fetch( "ListBrands", requestData );
    }
}

export function searchBrands( query: string, pageSize?: number )
{
    "use strict";

    // Create the service instance
    //
    var callURL: string = `/api/v1/brand/search`;
    var service = new EVAService<EVA.Core.Services.SearchBrands, EVA.Core.Services.SearchBrandsResponse>( "SearchBrands", callURL, EXMLHttpRequestMethod.POST );

    // PageSize does not appear to function
    //
    var requestData: EVA.Core.Services.SearchBrands = <EVA.Core.Services.SearchBrands>{
        PageSize:               pageSize,
        Query:                  query
    };

    // Perform the call and capture the resulting promise
    //
    return service.call( requestData );
}
