/**
 * The CMS Page module provides access to content management pages from the EVA back-end * @preferred
 */

import { FetchableItem,
         FetchablePagedList     } from "./base";
import { EVAService,
         IEVAServiceResponse    } from "../service";

/**
 * Provides access to a content page available in EVA
 */
export class ContentPage extends FetchableItem<EVA.CMS.CmsPageDto, EVA.CMS.GetCmsPage, EVA.CMS.GetCmsPageResponse> {
    /**
     * The request path to retrieve the page for
     * @type {string}
     */
    private requestPath: string;

    /**
     * Returns the current request path
     * @return {string}       The configured request path
     */
    public getRequestPath(): string {
        return this.requestPath;
    }

    /**
     * Set the request path to retrieve the page for
     * @param  {string} query The new request path
     * @return {string}       The configured request path
     */
    public setRequestPath( requestPath: string ): string {
        return this.requestPath = requestPath;
    }

    /**
     * Fetches the page content
     * @return {Promise<IEVAServiceResponse<EVA.CMS.GetCmsPage, EVA.CMS.GetCmsPageResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.CMS.GetCmsPage, EVA.CMS.GetCmsPageResponse>> {

        if ( this.getId() )
        {
            // Build the request data
            //
            var requestData: EVA.CMS.GetCmsPage = <EVA.CMS.GetCmsPage>{
                ID: this.getId()
            };

            return super.fetch( "GetCmsPage", requestData );
        }
        else
        {
            // Build the request data
            //
            var requestByPathData: EVA.CMS.GetCmsPageByRequestPath = <EVA.CMS.GetCmsPageByRequestPath>{
                Path: this.requestPath
            };

            return super.fetch( "GetCmsPageByRequestPath", <any>requestByPathData );
        }
    }
}

/**
 * Provides access to the generic content pages by path request available in EVA
 */
export function getCmsPagesByRequestPath( path: string )
{
    // Create our service instance
    //
    var service = new EVAService<EVA.CMS.GetCmsPagesByRequestPath, EVA.CMS.GetCmsPagesByRequestPathResponse>( "GetCmsPagesByRequestPath" );

    // Build the request data
    //
    var requestData: EVA.CMS.GetCmsPagesByRequestPath = {
        Path:       path,
        __response: undefined
    };

    // Perform the call and capture the resulting promise
    //
    return service.call( requestData );
}

/**
 * Provides access to a list of content pages available in EVA
 */
export class ContentPageList extends FetchablePagedList<EVA.CMS.CmsPageDto, EVA.CMS.ListCmsPages, EVA.CMS.ListCmsPagesResponse> {
    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.CMS.CmsPageDto, EVA.CMS.ListCmsPages, EVA.CMS.ListCmsPagesResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.CMS.ListCmsPages, EVA.CMS.ListCmsPagesResponse>> {

        // Build the request data
        //
        var requestData: EVA.CMS.ListCmsPages = <EVA.CMS.ListCmsPages>{
            PageConfig: this.getPageConfiguration()
        };

        return super.fetch( "ListCmsPages", requestData );
    }
}
