/**
 * The applications module provides access to the applications data from the EVA back-end
 * @preferred
 */

import { FetchableItem, FetchablePagedList } from "./base";
import { EVAService, IEVAServiceResponse   } from "../service";
import { EXMLHttpRequestMethod             } from "../xhr";
import * as _                                from "lodash";

/**
 * Provides access to a babylon collections (fashion) in EVA
 */
export class Collection extends FetchableItem<EVA.Babylon.FashionCollectionDetail, EVA.Babylon.GetFashionCollectionDetailsByID, EVA.Babylon.GetFashionCollectionDetailsByIdResponse> {

    public fetch(): Promise<IEVAServiceResponse<EVA.Babylon.GetFashionCollectionDetailsByID, EVA.Babylon.GetFashionCollectionDetailsByIdResponse>>
    {
        if ( this.getId() === undefined )
        {
            return Promise.reject( {
                status:     404,
                response:   "Not found"
            } );
        }
        else
        {
            // Build the request data
            //
            var requestData: EVA.Babylon.GetFashionCollectionDetailsByID = <EVA.Babylon.GetFashionCollectionDetailsByID>{
                CollectionID: this.getId()
            };

            return super.fetch( "GetFashionCollectionDetailsByID", requestData, "/api/v1/fashion/collection/" + this.getId() + "/detail", EXMLHttpRequestMethod.GET );
        }
    }

    /**
     * Sets the current item details using the fetched service response.
     * Extending classes should override this method to pull the details from response
     * @param  {RES} response The response from the service call
     * @return {T}            The item details
     */
    public setData( response: any ): EVA.Babylon.FashionCollectionDetail
    {
        // The default response is usually EVA.Core.GetResponse
        // An extending class will know exactly what is returned and where
        // the data for <T> can be found
        //
        return this.data = response.Collection;
    }

    public create( collection: EVA.Babylon.CreateFashionCollectionRequestModel ): Promise<IEVAServiceResponse<EVA.Babylon.GetFashionCollectionDetailsByID, EVA.Babylon.GetFashionCollectionDetailsByIdResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create the service instance
            //
            var service = new EVAService<EVA.Babylon.CreateFashionCollection, EVA.Babylon.CreateFashionCollectionResponse>( "CreateFashionCollection", "/api/v1/fashion/collection", EXMLHttpRequestMethod.POST );

            // Build the request data
            // The base class will add things like SessionID so keep typescript
            // happy we're using an 'any' type object here
            //
            var requestData: any = {
                Collection: collection
            };

            // Perform the call and capture the resulting promise
            //
            service.call( requestData )
            .then( ( createResult: IEVAServiceResponse<EVA.Babylon.CreateFashionCollection, EVA.Babylon.CreateFashionCollectionResponse> ) =>
            {
                // Collect the ID from the response
                //
                this.id = createResult.response.ID;

                // Return the fetch of the newly created collection
                //
                this.fetch().then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }

    public update( collectionDetailsToUpdate: EVA.Babylon.UpdateFashionCollection ): Promise<IEVAServiceResponse<EVA.Babylon.GetFashionCollectionDetailsByID, EVA.Babylon.GetFashionCollectionDetailsByIdResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            if ( this.getId() === undefined )
            {
                reject( {
                    status:     404,
                    response:   "Not found"
                } );
            }
            else
            {
                // Update the fashion collection
                //
                var service = new EVAService<EVA.Babylon.UpdateFashionCollection, EVA.Core.EmptyResponseMessage>( "UpdateFashionCollection", "/api/v1/fashion/collection/" + this.getId(), EXMLHttpRequestMethod.PATCH );

                // Ensure collectionID is set
                //
                collectionDetailsToUpdate.CollectionID = <number>this.getId();

                // Perform the call and capture the resulting promise
                //
                service.call( collectionDetailsToUpdate )
                .then( ( updateResult: IEVAServiceResponse<EVA.Babylon.UpdateFashionCollection, EVA.Core.EmptyResponseMessage> ) =>
                {
                    // Return the fetch of the updated user profile
                    //
                    this.fetch().then( resolve ).catch( reject );
                } )
                .catch( reject );
            }
        } );
    }

    public delete( ): Promise<IEVAServiceResponse<EVA.Babylon.DeleteFashionCollection, EVA.Core.EmptyResponseMessage>>
    {
        if ( this.getId() === undefined )
        {
            return Promise.reject( {
                status:     404,
                response:   "Not found"
            } );
        }
        else
        {
            // Create the service instance
            //
            var callURL: string = "/api/v1/fashion/collection/" + this.getId();
            var service = new EVAService<EVA.Babylon.DeleteFashionCollection, EVA.Core.EmptyResponseMessage>( "DeleteFashionCollection", callURL, EXMLHttpRequestMethod.DELETE );

            // Build the request data
            // The base class will add things like SessionID so keep typescript
            // happy we're using an 'any' type object here
            //
            var requestData: any = {
            };

            // Perform the call and capture the resulting promise
            //
            return service.call( requestData );
        }
    }

    public addProducts( productIDs: Array<number> ): Promise<IEVAServiceResponse<EVA.Babylon.GetFashionCollectionDetailsByID, EVA.Babylon.GetFashionCollectionDetailsByIdResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            if ( this.getId() === undefined )
            {
                reject( {
                    status:     404,
                    response:   "Not found"
                } );
            }
            else
            {
                // Create the service instance
                //
                var callURL: string = "/api/v1/fashion/collection/" + this.getId() + "/product";
                var service = new EVAService<EVA.Babylon.AddProductsToFashionCollection, EVA.Core.EmptyResponseMessage>( "AddProductsToFashionCollection", callURL, EXMLHttpRequestMethod.POST );

                // Build the request data
                // The base class will add things like SessionID so keep typescript
                // happy we're using an 'any' type object here
                //
                var requestData: any = {
                    ProductIDs: productIDs
                };

                // Perform the call and capture the resulting promise
                //
                service.call( requestData )
                .then( () =>
                {
                    // Return the fetch of the updated collection
                    //
                    this.fetch().then( resolve ).catch( reject );
                } )
                .catch( reject );
            }
        } );
    }

    public removeProduct( productID: number ): Promise<IEVAServiceResponse<EVA.Babylon.GetFashionCollectionDetailsByID, EVA.Babylon.GetFashionCollectionDetailsByIdResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            if ( this.getId() === undefined )
            {
                reject( {
                    status:     404,
                    response:   "Not found"
                } );
            }
            else
            {
                // Create the service instance
                //
                var callURL: string = "/api/v1/fashion/collection/" + this.getId() + "/product/" + productID;
                var service = new EVAService<EVA.Babylon.DeleteProductFromFashionCollection, EVA.Core.EmptyResponseMessage>( "DeleteProductFromFashionCollection", callURL, EXMLHttpRequestMethod.DELETE );

                // Build the request data
                // The base class will add things like SessionID so keep typescript
                // happy we're using an 'any' type object here
                //
                var requestData: any = {
                };

                // Perform the call and capture the resulting promise
                //
                service.call( requestData )
                .then( () =>
                {
                    // Return the fetch of the updated collection
                    //
                    this.fetch().then( resolve ).catch( reject );
                } )
                .catch( reject );
            }
        } );
    }

    public getProductDetails( productID: number ): Promise<IEVAServiceResponse<EVA.Babylon.GetFashionCollectionProductDetails, EVA.Babylon.GetFashionCollectionProductDetailsResponse>>
    {
        if ( this.getId() === undefined )
        {
            return Promise.reject( {
                status:     404,
                response:   "Not found"
            } );
        }
        else
        {
            // Create the service instance
            //
            var callURL: string = "/api/v1/fashion/collection/" + this.getId() + "/product/" + productID;
            var service = new EVAService<EVA.Babylon.GetFashionCollectionProductDetails, EVA.Babylon.GetFashionCollectionProductDetailsResponse>( "GetFashionCollectionProductDetails", callURL, EXMLHttpRequestMethod.GET );

            // Build the request data
            // The base class will add things like SessionID so keep typescript
            // happy we're using an 'any' type object here
            //
            var requestData: any = {
            };

            // Perform the call and capture the resulting promise
            //
            return service.call( requestData );
        }
    }

    public updateProductSequence( productID: number, sequence: number ): Promise<IEVAServiceResponse<EVA.Babylon.GetFashionCollectionProductDetails, EVA.Babylon.GetFashionCollectionProductDetailsResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            if ( this.getId() === undefined )
            {
                reject( {
                    status:     404,
                    response:   "Not found"
                } );
            }
            else
            {
                // Create the service instance
                //
                var callURL: string = "/api/v1/fashion/collection/" + this.getId() + "/product/" + productID + "/sequence/" + sequence;
                var service = new EVAService<EVA.Babylon.UpdateFashionCollectionProductSequence, EVA.Core.EmptyResponseMessage>( "UpdateFashionCollectionProductSequence", callURL, EXMLHttpRequestMethod.PUT );

                // Build the request data
                // The base class will add things like SessionID so keep typescript
                // happy we're using an 'any' type object here
                //
                var requestData: any = {
                };

                // Perform the call and capture the resulting promise
                //
                service.call( requestData )
                .then( () =>
                {
                    // Return the fetch of the updated collection
                    //
                    this.fetch().then( resolve ).catch( reject );
                } )
                .catch( reject );
            }
        } );
    }
}

export class CollectionsEarnings extends FetchableItem<EVA.Babylon.FashionCollectionSummaryWithEarnings[], EVA.Babylon.GetFashionCoinEarningsForCollections, EVA.Babylon.GetFashionCoinEarningsForCollectionsResponse> {

    public fetch(): Promise<IEVAServiceResponse<EVA.Babylon.GetFashionCoinEarningsForCollections, EVA.Babylon.GetFashionCoinEarningsForCollectionsResponse>>
    {
        const requestData: EVA.Babylon.GetFashionCoinEarningsForCollections = {
            __response: undefined
        } ;

        return super.fetch( "GetFashionCoinEarningsForCollections", requestData, "/api/v1/fashion/collection/earnings", EXMLHttpRequestMethod.GET );
    }
}

export class CollectionProducts extends FetchablePagedList<EVA.Babylon.FashionCollectionProductDetails, EVA.Babylon.ListProductsForCollection, EVA.Babylon.ListProductsForCollectionResponse>
{
    /**
     * The collection to retrieve the product list for
     * @type {number}
     */
    private collectionID: number;

    /**
     * Sets the collection id to retrieve the product list for
     * @param  {number} collectionID        The collection identifier
     * @return {void}                       This method doesn't return anything
     */
    public setCollectionID( collectionID: number )
    {
        this.collectionID = collectionID;
    }

    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Babylon.ListProductsForCollection, EVA.Babylon.ListProductsForCollectionResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Babylon.ListProductsForCollection, EVA.Babylon.ListProductsForCollectionResponse>> {

        // Build the request data
        //
        var requestData: EVA.Babylon.ListProductsForCollection = <EVA.Babylon.ListProductsForCollection>{};

        // Apply PageConfig parameters directly
        //
        _.assign( requestData, _.omit( this.getPageConfiguration(), [ "Filter" ] ) );

        return super.fetch( "ListProductsForCollection", requestData, `/api/v1/fashion/collection/${this.collectionID}/product`, EXMLHttpRequestMethod.GET );
    }
}

export class CollectionCoverSheet extends FetchableItem<EVA.Babylon.CoverSheetDocument, EVA.Babylon.GetFashionCollectionCoverSheet, EVA.Babylon.GetFashionCollectionCoverSheetResponse>
{
    /**
     * The collection to retrieve the cover sheet for
     * @type {number}
     */
    private collectionID: number;

    /**
     * Sets the collection id to retrieve the cover sheet for
     * @param  {number} collectionID        The collection identifier
     * @return {void}                       This method doesn't return anything
     */
    public setCollectionID( collectionID: number )
    {
        this.collectionID = collectionID;
    }

    /**
     * Sets the current item details using the fetched service response.
     * Extending classes should override this method to pull the details from response
     * @param  {RES} response The response from the service call
     * @return {T}            The item details
     */
    public setData( response: any ): EVA.Babylon.CoverSheetDocument
    {
        // The default response is usually EVA.Core.GetResponse
        // An extending class will know exactly what is returned and where
        // the data for <T> can be found
        //
        return this.data = response.CoverSheet;
    }

    /**
     * Fetches the collection cover sheet
     * @return {Promise<IEVAServiceResponse<EVA.Babylon.GetFashionCollectionCoverSheet, EVA.Babylon.GetFashionCollectionCoverSheetResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Babylon.GetFashionCollectionCoverSheet, EVA.Babylon.GetFashionCollectionCoverSheetResponse>>
    {
        if ( this.getId() === undefined )
        {
            return Promise.reject( {
                status:     404,
                response:   "Not found"
            } );
        }
        else
        {
            // Build the request data
            //
            var requestData: EVA.Babylon.GetFashionCollectionCoverSheet = <EVA.Babylon.GetFashionCollectionCoverSheet>{};

            return super.fetch( "GetFashionCollectionCoverSheet", requestData, "/api/v1/fashion/collection/" + this.getId() + "/cover_sheet", EXMLHttpRequestMethod.GET );
        }
    }

    public update( coversheet: EVA.Babylon.CoverSheetDocument ): Promise<IEVAServiceResponse<EVA.Babylon.GetFashionCollectionCoverSheet, EVA.Babylon.GetFashionCollectionCoverSheetResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            if ( this.getId() === undefined )
            {
                reject( {
                    status:     404,
                    response:   "Not found"
                } );
            }
            else
            {
                // Update the cover sheet
                //
                var service = new EVAService<EVA.Babylon.SaveFashionCollectionCoverSheet, EVA.Core.EmptyResponseMessage>( "SaveFashionCollectionCoverSheet", "/api/v1/fashion/collection/" + this.getId() + "/cover_sheet", EXMLHttpRequestMethod.PUT );

                var requestData: EVA.Babylon.SaveFashionCollectionCoverSheet = <EVA.Babylon.SaveFashionCollectionCoverSheet>{
                    CollectionID: this.getId()
                ,   CoverSheet:   coversheet
                };

                // Perform the call and capture the resulting promise
                //
                service.call( requestData )
                .then( ( updateResult: IEVAServiceResponse<EVA.Babylon.SaveFashionCollectionCoverSheet, EVA.Core.EmptyResponseMessage> ) =>
                {
                    // Return the fetch of the updated user profile
                    //
                    this.fetch().then( resolve ).catch( reject );
                } )
                .catch( reject );
            }
        } );
    }

    public productSearch( query: string, pageSize?: number, page?: number, likedProductsPage?: number, createdProductsPage?: number ): Promise<IEVAServiceResponse<EVA.Babylon.CoverSheetProductsSearch, EVA.Babylon.CoverSheetProductsSearchResponse>>
    {
        // Update the cover sheet
        //
        var service = new EVAService<EVA.Babylon.CoverSheetProductsSearch, EVA.Babylon.CoverSheetProductsSearchResponse>( "CoverSheetProductsSearch" );

        var requestData: EVA.Babylon.CoverSheetProductsSearch = <any>{
            Query:               query
        ,   PageSize:            pageSize
        ,   Page:                page
        ,   LikedProductsPage:   likedProductsPage
        ,   CreatedProductsPage: createdProductsPage
        };

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData );
    }
}

export class Collections extends FetchablePagedList<EVA.Babylon.FashionCollectionDetail, EVA.Babylon.ListFashionCollectionsForUser, EVA.Babylon.ListFashionCollectionsForUserResponse>
{
    /**
     * The user to retrieve the collection list for
     * @type {number}
     */
    private userID: number;
    private filterCoverSheetExists: boolean;

    /**
     * Sets the user id to retrieve the collection list for
     * @param  {number} userID      The user identifier
     * @return {void}               This method doesn't return anything
     */
    public setUserID( userID: number )
    {
        this.userID = userID;
    }

    /**
     * Sets the user id to retrieve the collection list for
     * @param  {number} userID      The user identifier
     * @return {void}               This method doesn't return anything
     */
    public setCoverSheetFilter( enableFilter: boolean = true )
    {
        this.filterCoverSheetExists = enableFilter;
    }

    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Babylon.ListFashionCollectionsForUser, EVA.Babylon.ListFashionCollectionsForUserResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Babylon.ListFashionCollectionsForUser, EVA.Babylon.ListFashionCollectionsForUserResponse>> {

        // Build the request data
        //
        var requestData: EVA.Babylon.ListFashionCollectionsForUser = <EVA.Babylon.ListFashionCollectionsForUser>{};

        if ( _.isBoolean( this.filterCoverSheetExists ) ) {
            requestData.FilterCoverSheetExists = this.filterCoverSheetExists;
        }

        // Apply PageConfig parameters directly
        //
        _.assign( requestData, _.omit( this.getPageConfiguration(), [ "Filter" ] ) );

        return super.fetch( "ListFashionCollectionsForUser", requestData, `/api/v1/social/user/${this.userID}/collection`, EXMLHttpRequestMethod.GET );
    }
}

export class TrendingCollections extends FetchablePagedList<EVA.Babylon.FashionCollectionSummary, EVA.Babylon.ListTrendingFashionCollections, EVA.Babylon.ListTrendingFashionCollectionsResponse>
{
    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Babylon.ListTrendingFashionCollections, EVA.Babylon.ListTrendingFashionCollectionsResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Babylon.ListTrendingFashionCollections, EVA.Babylon.ListTrendingFashionCollectionsResponse>> {

        // Build the request data
        //
        var requestData: EVA.Babylon.ListTrendingFashionCollections = <EVA.Babylon.ListTrendingFashionCollections>{};

        // Apply PageConfig parameters directly
        //
        _.assign( requestData, _.omit( this.getPageConfiguration(), [ "Filter" ] ) );

        return super.fetch( "ListTrendingFashionCollections", requestData, `/api/v1/fashion/collection/trending`, EXMLHttpRequestMethod.GET );
    }
}

export class TopFashionCollections extends FetchablePagedList<EVA.Babylon.FashionCollectionSummary, EVA.Babylon.GetTopFashionCollections, EVA.Babylon.GetTopFashionCollectionsResponse> {

    /**
     * The user to retrieve the collection list for
     * @type {number}
     */
    private userID: number;

    /**
     * The creation time where you want the collection to be for
     * @type {string}
     */
    private creationTime: string;

    /**
     * The maximum number of items you want in your collection
     * @type {number}
     */
    private topCount: number;

    /**
     * Sets the user id to retrieve the collection list for
     * @param  {number} userID      The user identifier
     * @return {void}               This method doesn't return anything
     */
    public setUserID( userID: number )
    {
        this.userID = userID;
    }

    /**
     * Sets creation time to filter on
     * @param  {Date} time    The creation time to filter on
     * @return {void}         This method doesn't return anything
     */
    public setCreationTime( time: Date )
    {
        this.creationTime = time.toISOString();
    }

    /**
     * Sets top count to filter on
     * @param  {number} count  The top count to filter on
     * @return {void}          This method doesn't return anything
     */
    public setTopCount( count: number )
    {
        this.topCount = count;
    }

    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Babylon.GetTopFashionCollections, EVA.Babylon.GetTopFashionCollectionsResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Babylon.GetTopFashionCollections, EVA.Babylon.GetTopFashionCollectionsResponse>> {

        // Build the request data
        //
        var requestData: EVA.Babylon.GetTopFashionCollections = <EVA.Babylon.GetTopFashionCollections>{};

        // Check if we need to filter on user
        //
        if ( this.userID ) {
            requestData.UserID = this.userID;
        }

        // Check if we need to filter on creation time
        //
        if ( this.creationTime ) {
            requestData.CreationTime = this.creationTime;
        }

        // Checkif we need to filter on top count
        // By default we set it to 10
        //
        let topCountURI = 10;
        if ( this.topCount ) {
            topCountURI = this.topCount;
        }

        // Apply PageConfig parameters directly
        //
        _.assign( requestData, _.omit( this.getPageConfiguration(), [ "Filter" ] ) );

        return super.fetch( "GetTopFashionCollections", requestData, `/api/v1/fashion/collection/top/${topCountURI}`, EXMLHttpRequestMethod.GET );
    }
}

export class ProductCollections extends FetchablePagedList<EVA.Babylon.FashionCollectionDetail, EVA.Babylon.ListFashionCollectionsForProduct, EVA.Babylon.ListFashionCollectionsForProductResponse>
{
    /**
     * The user to retrieve the collection list for
     * @type {number}
     */
    private userID: number;

    /**
     * The product to retrieve the collection list for
     * @type {number}
     */
    private productID: number;

    /**
     * Sets the user id to retrieve the collection list for
     * @param  {number} userID      The user identifier
     * @return {void}               This method doesn't return anything
     */
    public setUserID( userID: number )
    {
        this.userID = userID;
    }

    /**
     * Sets the user id to retrieve the collection list for
     * @param  {number} productID      The user identifier
     * @return {void}               This method doesn't return anything
     */
    public setProductID( productID: number )
    {
        this.productID = productID;
    }

    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Babylon.ListFashionCollectionsForProduct, EVA.Babylon.ListFashionCollectionsForProductResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Babylon.ListFashionCollectionsForProduct, EVA.Babylon.ListFashionCollectionsForProductResponse>> {

        // Build the request data
        //
        var requestData: EVA.Babylon.ListFashionCollectionsForProduct = <EVA.Babylon.ListFashionCollectionsForProduct> {
            UserID: this.userID
        };

        // Apply PageConfig parameters directly
        //
        _.assign( requestData, _.omit( this.getPageConfiguration(), [ "Filter" ] ) );

        return super.fetch( "ListFashionCollectionsForProduct", requestData, `/api/v1/social/product/${this.productID}/collection`, EXMLHttpRequestMethod.GET );
    }
}
