/**
 * The DMS Page module provides access to content management pages from the EVA back-end * @preferred
 */

import { FetchableItem                      } from "./../base";
import { EVAService, IEVAServiceResponse    } from "../../service";

/**
 * Provides access to a content page available in EVA
 */
export class DmsPage extends FetchableItem<EVA.DMS.LivePageDto, EVA.DMS.DmsGetLivePage, EVA.DMS.DmsGetLivePageResponse> {
    /**
     * The request path to retrieve the page for
     * @type {string}
     */
    private requestPath: string;

    /**
     * The view ID to retrieve the page by selected view
     * @type {number}
     */
    private viewId: number;

    /**
     * Returns the current request path
     * @return {string}       The configured request path
     */
    public getRequestPath(): string {
        return this.requestPath;
    }

    /**
     * Set the request path to retrieve the page for
     * @param  {string} query The new request path
     * @return {string}       The configured request path
     */
    public setRequestPath( requestPath: string ): string {
        return this.requestPath = requestPath;
    }

    /**
     * Returns the current view ID
     * @return {number}       The configured view ID
     */
    public getViewId(): number {
        return this.viewId;
    }

    /**
     * Set the request path to retrieve the page by view ID
     * @param  {number} query The new view ID
     * @return {number}       The configured view ID
     */
    public setViewId( viewId: number ): number {
        return this.viewId = viewId;
    }

    /**
     * Fetches the page content
     * @return {Promise<IEVAServiceResponse<EVA.DMS.DmsGetLivePage, EVA.DMS.DmsGetLivePageResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.DMS.DmsGetLivePage, EVA.DMS.DmsGetLivePageResponse>> {

        if ( this.getViewId() )
        {
            // Build the request data
            //
            let requestData: EVA.DMS.DmsGetPageByViewID = <EVA.DMS.DmsGetPageByViewID>{
                ID: this.getViewId()
            };

            return super.fetch( "DmsGetPageByViewID", <any>requestData );
        }
        else if ( this.getId() )
        {
            // Build the request data
            //
            let requestData: EVA.DMS.DmsGetLivePage = <EVA.DMS.DmsGetLivePage>{
                ID: this.getId()
            };

            return super.fetch( "DmsGetLivePage", requestData );
        }
        else
        {
            // Build the request data
            //
            let requestByPathData: EVA.DMS.DmsGetLivePage = <EVA.DMS.DmsGetLivePage>{
                Path: this.requestPath
            };

            return super.fetch( "DmsGetLivePage", <any>requestByPathData );
        }
    }

    public listViews(): Promise<IEVAServiceResponse<EVA.DMS.DmsListViewsForPage, EVA.DMS.DmsListViewsForPageResponse>> {
        // Create the service instance
        //
        var service = new EVAService<EVA.DMS.DmsListViewsForPage, EVA.DMS.DmsListViewsForPageResponse>( "DmsListViewsForPage" );

        // Build the request data
        //
        let requestData: EVA.DMS.DmsListViewsForPage = <EVA.DMS.DmsListViewsForPage> {
            ID: this._id
        };

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData );
    }

    public saveLayout( blocks: EVA.DMS.WidgetLayoutBlockDto[] ): Promise<IEVAServiceResponse<EVA.DMS.DmsUpdateViewLayout, EVA.Core.EmptyResponseMessage>> {
        // Create the service instance
        //
        var service = new EVAService<EVA.DMS.DmsUpdateViewLayout, EVA.Core.EmptyResponseMessage>( "DmsUpdateViewLayout" );

        // Build the request data
        //
        let requestData: EVA.DMS.DmsUpdateViewLayout = <EVA.DMS.DmsUpdateViewLayout> {
            Blocks: blocks
        };

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData );
    }
}

/**
 * Provides access to a content page available in EVA
 */
export class DmsCompiledPage extends FetchableItem<EVA.DMS.CompiledPageDto, EVA.DMS.DmsGetCompiledPage, EVA.DMS.DmsGetCompiledPageResponse> {
    /**
     * The request path to retrieve the page for
     * @type {string}
     */
    private requestPath: string;

    /**
     * Returns the current request path
     * @return {string}       The configured request path
     */
    public getRequestPath(): string {
        return this.requestPath;
    }

    /**
     * Set the request path to retrieve the page for
     * @param  {string} query The new request path
     * @return {string}       The configured request path
     */
    public setRequestPath( requestPath: string ): string {
        return this.requestPath = requestPath;
    }

    /**
     * Fetches the page content
     * @return {Promise<IEVAServiceResponse<EVA.DMS.DmsGetCompiledPage, EVA.DMS.DmsGetCompiledPageResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.DMS.DmsGetCompiledPage, EVA.DMS.DmsGetCompiledPageResponse>> {

        if ( this.getId() )
        {
            // Build the request data
            //
            let requestData: EVA.DMS.DmsGetCompiledPage = <EVA.DMS.DmsGetCompiledPage>{
                ID: this.getId()
            };

            return super.fetch( "DmsGetCompiledPage", requestData );
        }
        else
        {
            // Build the request data
            //
            let requestByPathData: EVA.DMS.DmsGetCompiledPage = <EVA.DMS.DmsGetCompiledPage>{
                Path: this.requestPath
            };

            return super.fetch( "DmsGetCompiledPage", <any>requestByPathData );
        }
    }
}
