/**
 * The MakeUp module provides access to advanced content management features from the EVA back-end
 * @preferred
 */

import {
    EVAService,
    IEVAServiceResponse        } from "../service";
import {
    FetchableItem,
    FetchablePagedList         } from "./base";
import * as _                    from "lodash";
import { EXMLHttpRequestMethod } from "../xhr";

// MakeUp calls should never be cached
//
const disableServiceCache = true;

// We need to extend the response interface for MakeupGetRenderedPageByPath to include the PathContext
//
export interface MakeupGetRenderedPageByPathResponseWithContext extends EVA.Makeup.MakeupGetRenderedPageByPathResponse
{
    PathContext: EVA.Makeup.PathContext;
    QueryPath:   string;
}

export namespace MakeUp
{
    "use strict";

    /***************
     *    PAGES    *
     ***************/

    /**
     * Provides access to a make up content map
     */
    export function getPageMap( languageID?: string ): Promise<IEVAServiceResponse<EVA.Makeup.MakeupGetPageMap, EVA.Makeup.MakeupGetPageMapResponse>>
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Makeup.MakeupGetPageMap, EVA.Makeup.MakeupGetPageMapResponse>( "MakeupGetPageMap" );

        // Build the request data
        //
        var requestData: EVA.Makeup.MakeupGetPageMap = {
            LanguageID: languageID,
            __response: undefined
        };

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData, disableServiceCache );
    }

    /**
     * Provides access to a make up content page by path
     */
    export function getPageByRequestPath( path: string, languageID?: string ): Promise<IEVAServiceResponse<EVA.Makeup.MakeupGetPageByPath, EVA.Makeup.MakeupGetPageByPathResponse>>
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Makeup.MakeupGetPageByPath, EVA.Makeup.MakeupGetPageByPathResponse>( "MakeupGetPageByPath" );

        // Build the request data
        //
        var requestData: EVA.Makeup.MakeupGetPageByPath = {
            Path:       path,
            LanguageID: languageID,
            __response: undefined
        };

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData, disableServiceCache );
    }

    /**
     * Provides access to a make up content page by id
     */
    export function getPageByID( ID: string, languageID?: string, version?: number ): Promise<IEVAServiceResponse<EVA.Makeup.MakeupGetPageByID, EVA.Makeup.MakeupGetPageByIdResponse>>
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Makeup.MakeupGetPageByID, EVA.Makeup.MakeupGetPageByIdResponse>( "MakeupGetPageByID" );

        // Build the request data
        //
        var requestData: EVA.Makeup.MakeupGetPageByID = {
            ID:         ID,
            LanguageID: languageID,
            Version:    version,
            __response: undefined
        };

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData, disableServiceCache );
    }

    /**
     * Provides access to a rendered make up content page by path
     */
    export function getRenderedPageByRequestPath( path: string, languageID?: string ): Promise<IEVAServiceResponse<EVA.Makeup.MakeupGetRenderedPageByPath, MakeupGetRenderedPageByPathResponseWithContext>>
    {
        // Create our service instance
        //
        var callURL = `/api/v1/makeup/page/path/rendered`;
        var service = new EVAService<EVA.Makeup.MakeupGetRenderedPageByPath, MakeupGetRenderedPageByPathResponseWithContext>( "MakeupGetRenderedPageByPath", callURL, EXMLHttpRequestMethod.GET );

        // Build the request data
        //
        var requestData: EVA.Makeup.MakeupGetRenderedPageByPath = {
            Path:       path,
            LanguageID: languageID,
            __response: undefined
        };

        // Perform the call and capture the resulting promise
        //
        return new Promise( ( resolve: any, reject: any ) =>
        {
            service.call( requestData, disableServiceCache )
            .then( ( result ) => {

                result.response.PathContext = {
                    Path:       result.request.Path,
                    Parameters: []
                };

                var pathParams = _.map( ( result.xhr.getResponseHeader( "EVA-Makeup-Path-Params" ) || "" ).split( "," ), decodeURIComponent );
                _.each( pathParams, ( pathParam ) => {
                    if ( _.size( pathParam.split( "=" ) ) === 2 ) {
                        result.response.PathContext.Parameters.push( {
                            Key:   _.trim( <string> _.get( pathParam.split( "=" ), 0 ) ),
                            Value: _.trim( <string> _.get( pathParam.split( "=" ), 1 ) )
                        } );
                    }
                } );

                resolve( result );
            } )
            .catch( reject );
        } );
    }

    export function getRenderedPagesByPartialPath( path: string, languageID?: string ): Promise<IEVAServiceResponse<EVA.Makeup.MakeupGetRenderedPagesByPartialPath, EVA.Makeup.MakeupGetRenderedPagesByPartialPathResponse>>
    {
        // Create our service instance
        //
        var callURL = `/api/v1//makeup/page/partial_path/rendered`;
        var service = new EVAService<EVA.Makeup.MakeupGetRenderedPagesByPartialPath, EVA.Makeup.MakeupGetRenderedPagesByPartialPathResponse>( "MakeupGetRenderedPagesByPartialPath" );

        // Build the request data
        //
        var requestData: EVA.Makeup.MakeupGetRenderedPagesByPartialPath = {
            PartialPath: path,
            LanguageID: languageID,
            __response: undefined
        };

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData, disableServiceCache );
    }

    /**
     * Provides access to a make up content page by id
     */
    export function getRenderedPageByID( ID: string, languageID?: string, version?: number ): Promise<IEVAServiceResponse<EVA.Makeup.MakeupGetRenderedPageByID, EVA.Makeup.MakeupGetRenderedPageByIDResponse>>
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Makeup.MakeupGetRenderedPageByID, EVA.Makeup.MakeupGetRenderedPageByIDResponse>( "MakeupGetRenderedPageByID" );

        // Build the request data
        //
        var requestData: EVA.Makeup.MakeupGetRenderedPageByID = {
            ID:         ID,
            LanguageID: languageID,
            Version:    version,
            __response: undefined
        };

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData, disableServiceCache );
    }

    /**
     * Provides access to create a new Makeup page
     * @param page The MakeupPageDto
     */
    export function createPage( page: EVA.Makeup.PageDto ): Promise<IEVAServiceResponse<EVA.Makeup.MakeupGetPageByID, EVA.Makeup.MakeupGetPageByIdResponse>>
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Makeup.MakeupCreatePage, EVA.Makeup.MakeupCreatePageResponse>( "MakeupCreatePage" );

        // Build the request data
        //
        var requestData: EVA.Makeup.MakeupCreatePage = {
            Page:       page,
            __response: undefined
        };

        // Build the request
        //
        return new Promise( ( resolve, reject ) =>
        {
            service.call( requestData, disableServiceCache )
            .then( ( response ) => {
                getPageByID( response.response.ID ).then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }

    /**
     * Provides access to replace a MakeUp page
     * @param id   The ID of the Makeup page
     * @param page The MakeUpPageDto
     */
    export function replacePage( page: EVA.Makeup.PageDto ): Promise<IEVAServiceResponse<EVA.Makeup.MakeupGetPageByID, EVA.Makeup.MakeupGetPageByIdResponse>>
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Makeup.MakeupReplacePage, EVA.Core.EmptyResponseMessage>( "MakeupReplacePage" );

        // Build the request data
        //
        var requestData: EVA.Makeup.MakeupReplacePage = {
            Page:       page,
            ID:         page.ID,
            __response: undefined
        };

        // Build the request
        //
        return new Promise( ( resolve, reject ) =>
        {
            service.call( requestData, disableServiceCache )
            .then( () => {
                getPageByID( requestData.ID ).then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }

    /**
     * Convenience method to upsert a Makeup page
     * @param id   The ID of the Makeup page
     * @param page The MakeUpPageDto
     */
    export function upsertPage( page: EVA.Makeup.PageDto ): Promise<IEVAServiceResponse<EVA.Makeup.MakeupGetPageByID, EVA.Makeup.MakeupGetPageByIdResponse>>
    {
        // Build the request
        //
        return new Promise( ( resolve, reject ) =>
        {
            getPageByID( page.ID, page.LanguageID )
            .then( ( fetchResult ) =>
            {
                const existingPage = fetchResult.response.Page;
                if ( existingPage ) {
                    // Page is an immutable object so first clone the damn thing
                    //
                    let payload = _.clone( page );
                    payload.ID = existingPage.ID;
                    replacePage( payload ).then( resolve ).catch( reject );
                } else {
                    createPage( page ).then( resolve ).catch( reject );
                }
            } )
            .catch( reject );
        } );
    }

    /**
     * Convenience method to retrieve the latest version of a makeup page
     * @param id The ID of the Makeup page
     * @param languageID The language
     */
    export function getCurrentPageVersion( id: string, languageID: string ): Promise<IEVAServiceResponse<EVA.Makeup.MakeupGetCurrentPageVersion, EVA.Makeup.MakeupGetCurrentPageVersionResponse>>
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Makeup.MakeupGetCurrentPageVersion, EVA.Makeup.MakeupGetCurrentPageVersionResponse>( "MakeupGetCurrentPageVersion" );

        // Build the request data
        //
        var requestData: EVA.Makeup.MakeupGetCurrentPageVersion = {
            ID:         id,
            LanguageID : languageID,
            __response: undefined
        };

        // Build the request
        //
        return new Promise( ( resolve, reject ) =>
        {
            service.call( requestData, disableServiceCache ).then( resolve ).catch( reject );
        } );
    }

    /**
     * Convenience method to publish a version of a Makeup page
     * @param id The ID of the Makeup page
     * @param languageID The language
     */
    export function publishPageVersion( id: string, languageID: string, version: number ): Promise<IEVAServiceResponse<EVA.Makeup.MakeupPublishPageVersion, EVA.Core.EmptyResponseMessage>> {

        // Create our service instance
        //
        var service = new EVAService<EVA.Makeup.MakeupPublishPageVersion, EVA.Core.EmptyResponseMessage>( "MakeupPublishPageVersion" );

        // Build the request data
        //
        var requestData: EVA.Makeup.MakeupPublishPageVersion = {
            ID:         id,
            LanguageID: languageID,
            Version:    version,
            __response: undefined
        };

        // Build the request
        //
        return new Promise( ( resolve, reject ) =>
        {
            service.call( requestData, disableServiceCache ).then( resolve ).catch( reject );
        } );
    }

    /**
     * Provides access to delete a Makeup page
     * @param id The ID of the Makeup page
     */
    export function deletePage( id: string, languageID?: string, version?: number ): Promise<IEVAServiceResponse<EVA.Makeup.MakeupDeletePage, EVA.Core.EmptyResponseMessage>>
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Makeup.MakeupDeletePage, EVA.Core.EmptyResponseMessage>( "MakeupDeletePage" );

        // Build the request data
        //
        var requestData: EVA.Makeup.MakeupDeletePage = {
            ID:         id,
            LanguageID: languageID,
            Version:    version,
            __response: undefined
        };

        // Build the request
        //
        return service.call( requestData, disableServiceCache );
    }

    /****************
     *    BLOCKS    *
     ****************/
     export function getBlockByID( ID: string, version?: number ): Promise<IEVAServiceResponse<EVA.Makeup.MakeupGetBlockByID, EVA.Makeup.MakeupGetBlockByIDResponse>>
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Makeup.MakeupGetBlockByID, EVA.Makeup.MakeupGetBlockByIDResponse>( "MakeupGetBlockByID" );

        // Build the request data
        //
        var requestData: EVA.Makeup.MakeupGetBlockByID = {
            ID:         ID,
            Version:    version,
            __response: undefined
        };

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData, disableServiceCache );
    }

     export function getRenderedBlockByID( ID: string, version?: number ): Promise<IEVAServiceResponse<EVA.Makeup.MakeupGetRenderedBlockByID, EVA.Makeup.MakeupGetRenderedBlockByIDResponse>>
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Makeup.MakeupGetRenderedBlockByID, EVA.Makeup.MakeupGetRenderedBlockByIDResponse>( "MakeupGetRenderedBlockByID" );

        // Build the request data
        //
        var requestData: EVA.Makeup.MakeupGetRenderedBlockByID = {
            ID:         ID,
            Version:    version,
            __response: undefined
        };

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData, disableServiceCache );
    }

    /**
     * Provides access to create a new Makeup block
     * @param block The BlockDto to create
     */
    export function createBlock( block: EVA.Makeup.BlockDto ): Promise<IEVAServiceResponse<EVA.Makeup.MakeupGetBlockByID, EVA.Makeup.MakeupGetBlockByIDResponse>>
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Makeup.MakeupCreateBlock, EVA.Makeup.MakeupCreateBlockResponse>( "MakeupCreateBlock" );

        // Build the request data
        //
        var requestData: EVA.Makeup.MakeupCreateBlock = {
            Block:      block,
            __response: undefined
        };

        // Build the request
        //
        return new Promise( ( resolve, reject ) =>
        {
            service.call( requestData, disableServiceCache )
            .then( ( response ) => {
                getBlockByID( response.response.ID ).then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }

    /**
     * Convenience method to upsert a Makeup block
     * @param id    The ID of the Makeup block
     * @param block The MakeUpBlockDto
     */
    export function upsertBlock( block: EVA.Makeup.BlockDto ): Promise<IEVAServiceResponse<EVA.Makeup.MakeupGetBlockByID, EVA.Makeup.MakeupGetBlockByIDResponse>>
    {
        // Build the request
        //
        return new Promise( ( resolve, reject ) =>
        {
            getBlockByID( block.ID )
            .then( ( fetchResult ) =>
            {
                const existingBlock = fetchResult.response.Block;
                if ( existingBlock ) {
                    // Block is an immutable object so first clone the damn thing
                    //
                    let payload = _.clone( block );
                    payload.ID = existingBlock.ID;
                    replaceBlock( payload ).then( resolve ).catch( reject );
                } else {
                    createBlock( block ).then( resolve ).catch( reject );
                }
            } )
            .catch( reject );
        } );
    }

    /**
     * Provides access to replace a MakeUp block
     * @param block The BlockDto
     */
    export function replaceBlock( block: EVA.Makeup.BlockDto ): Promise<IEVAServiceResponse<EVA.Makeup.MakeupGetBlockByID, EVA.Makeup.MakeupGetBlockByIDResponse>>
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Makeup.MakeupUpdateBlock, EVA.Core.EmptyResponseMessage>( "MakeupUpdateBlock" );

        // Build the request data
        //
        var requestData: EVA.Makeup.MakeupUpdateBlock = {
            Block:      block,
            __response: undefined
        };

        // Build the request
        //
        return new Promise( ( resolve, reject ) =>
        {
            service.call( requestData, disableServiceCache )
            .then( ( response ) => {
                getBlockByID( requestData.Block.ID ).then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }

    /**
     * Provides access to delete a Makeup block
     * @param id The ID of the Makeup block
     */
    export function deleteBlock( id: string, version?: number ): Promise<IEVAServiceResponse<EVA.Makeup.MakeupDeleteBlock, EVA.Core.EmptyResponseMessage>>
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Makeup.MakeupDeleteBlock, EVA.Core.EmptyResponseMessage>( "MakeupDeleteBlock" );

        // Build the request data
        //
        var requestData: EVA.Makeup.MakeupDeleteBlock = {
            ID:         id,
            Version:    version,
            __response: undefined
        };

        // Build the request
        //
        return service.call( requestData, disableServiceCache );
    }

    /**
     * Convenience method to retrieve the latest version of a makeup block
     * @param id The ID of the Makeup block
     */
    export function getCurrentBlockVersion( id: string ): Promise<IEVAServiceResponse<EVA.Makeup.MakeupGetCurrentBlockVersion, EVA.Makeup.MakeupGetCurrentBlockVersionResponse>>
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Makeup.MakeupGetCurrentBlockVersion, EVA.Makeup.MakeupGetCurrentBlockVersionResponse>( "MakeupGetCurrentBlockVersion" );

        // Build the request data
        //
        var requestData: EVA.Makeup.MakeupGetCurrentBlockVersion = {
            ID:         id,
            __response: undefined
        };

        // Build the request
        //
        return new Promise( ( resolve, reject ) =>
        {
            service.call( requestData, disableServiceCache ).then( resolve ).catch( reject );
        } );
    }

    /**
     * Convenience method to publish a version of a Makeup block
     * @param id The ID of the Makeup block
     * @param version The version to set
     */
    export function publishBlockVersion( id: string, version: number ): Promise<IEVAServiceResponse<EVA.Makeup.MakeupPublishBlockVersion, EVA.Core.EmptyResponseMessage>> {

        // Create our service instance
        //
        var service = new EVAService<EVA.Makeup.MakeupPublishBlockVersion, EVA.Core.EmptyResponseMessage>( "MakeupPublishBlockVersion" );

        // Build the request data
        //
        var requestData: EVA.Makeup.MakeupPublishBlockVersion = {
            ID:         id,
            Version:    version,
            __response: undefined
        };

        // Build the request
        //
        return new Promise( ( resolve, reject ) =>
        {
            service.call( requestData, disableServiceCache ).then( resolve ).catch( reject );
        } );
    }

    /****************
     *    MENU      *
     ****************/
     export function getMenuByID( ID: string, languageID?: string ): Promise<IEVAServiceResponse<EVA.Makeup.MakeupGetMenuByID, EVA.Makeup.MakeupGetMenuByIDResponse>>
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Makeup.MakeupGetMenuByID, EVA.Makeup.MakeupGetMenuByIDResponse>( "MakeupGetMenuByID" );

        // Build the request data
        //
        var requestData: EVA.Makeup.MakeupGetMenuByID = {
            ID:         ID,
            LanguageID: languageID,
            __response: undefined
        };

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData, disableServiceCache );
    }

    /**
     * Provides access to create a new Makeup block
     * @param block The MenuDto to create
     */
    export function createMenu( block: EVA.Makeup.MenuDto ): Promise<IEVAServiceResponse<EVA.Makeup.MakeupGetMenuByID, EVA.Makeup.MakeupGetMenuByIDResponse>>
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Makeup.MakeupCreateMenu, EVA.Makeup.MakeupCreateMenuResponse>( "MakeupCreateMenu" );

        // Build the request data
        //
        var requestData: EVA.Makeup.MakeupCreateMenu = {
            Menu:      block,
            __response: undefined
        };

        // Build the request
        //
        return new Promise( ( resolve, reject ) =>
        {
            service.call( requestData, disableServiceCache )
            .then( ( response ) => {
                getMenuByID( response.response.ID ).then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }

    /**
     * Convenience method to upsert a Makeup block
     * @param id    The ID of the Makeup block
     * @param block The MakeUpMenuDto
     */
    export function upsertMenu( block: EVA.Makeup.MenuDto ): Promise<IEVAServiceResponse<EVA.Makeup.MakeupGetMenuByID, EVA.Makeup.MakeupGetMenuByIDResponse>>
    {
        // Build the request
        //
        return new Promise( ( resolve, reject ) =>
        {
            getMenuByID( block.ID )
            .then( ( fetchResult ) =>
            {
                const existingMenu = fetchResult.response.Menu;
                if ( existingMenu ) {
                    // Page is an immutable object so first clone the damn thing
                    //
                    let payload = _.clone( block );
                    payload.ID = existingMenu.ID;
                    replaceMenu( payload ).then( resolve ).catch( reject );
                } else {
                    createMenu( block ).then( resolve ).catch( reject );
                }
            } )
            .catch( reject );
        } );
    }

    /**
     * Provides access to replace a MakeUp block
     * @param menu The MenuDto
     */
    export function replaceMenu( menu: EVA.Makeup.MenuDto ): Promise<IEVAServiceResponse<EVA.Makeup.MakeupGetMenuByID, EVA.Makeup.MakeupGetMenuByIDResponse>>
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Makeup.MakeupReplaceMenu, EVA.Core.EmptyResponseMessage>( "MakeupReplaceMenu" );

        // Build the request data
        //
        var requestData: EVA.Makeup.MakeupReplaceMenu = {
            ID:         menu.ID,
            Menu:       menu,
            __response: undefined
        };

        // Build the request
        //
        return new Promise( ( resolve, reject ) =>
        {
            service.call( requestData, disableServiceCache )
            .then( ( response ) => {
                getMenuByID( requestData.Menu.ID ).then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }

    /**
     * Provides access to delete a Makeup block
     * @param id The ID of the Makeup block
     */
    export function deleteMenu( id: string, languageID?: string ): Promise<IEVAServiceResponse<EVA.Makeup.MakeupDeleteMenu, EVA.Core.EmptyResponseMessage>>
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Makeup.MakeupDeleteMenu, EVA.Core.EmptyResponseMessage>( "MakeupDeleteMenu" );

        // Build the request data
        //
        var requestData: EVA.Makeup.MakeupDeleteMenu = {
            ID:         id,
            LanguageID: languageID,
            __response: undefined
        };

        // Build the request
        //
        return service.call( requestData, disableServiceCache );
    }

    /****************************
     *    CONFIGURATION PROFILE *
     ****************************
     */
    export function getConfigurationProfile( languageID?: string ): Promise<IEVAServiceResponse<EVA.Makeup.MakeupGetConfigurationProfile, EVA.Makeup.MakeupGetConfigurationProfileResponse>>
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Makeup.MakeupGetConfigurationProfile, EVA.Makeup.MakeupGetConfigurationProfileResponse>( "MakeupGetConfigurationProfile" );

        // Build the request data
        //
        var requestData: EVA.Makeup.MakeupGetConfigurationProfile = {
            LanguageID: languageID,
            __response: undefined
        };

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData, disableServiceCache );
    }

    /**
     * Provides access to replace a ConfigurationProfile
     * @param profile The ConfigurationProfile
     */
    export function replaceConfigurationProfile( profile: EVA.Makeup.ConfigurationProfileDto, languageID?: string ): Promise<IEVAServiceResponse<EVA.Makeup.MakeupGetConfigurationProfile, EVA.Makeup.MakeupGetConfigurationProfileResponse>>
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Makeup.MakeupReplaceConfigurationProfile, EVA.Core.EmptyResponseMessage>( "MakeupReplaceConfigurationProfile" );

        // Build the request data
        //
        var requestData: EVA.Makeup.MakeupReplaceConfigurationProfile = {
            Configuration: profile,
            LanguageID:    languageID || profile.LanguageID,
            __response:    undefined
        };

        // Build the request
        //
        return new Promise( ( resolve, reject ) =>
        {
            service.call( requestData, disableServiceCache )
            .then( ( response ) => {
                getConfigurationProfile().then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }
}

export class MakeupListPageVersions extends FetchablePagedList<EVA.Makeup.PageVersionDto, EVA.Makeup.MakeupListPageVersions, EVA.Makeup.MakeupListPageVersionsResponse>
{
    /**
     * The make up page identifier
     * @type {string}
     */
    private page_id: string;

    /**
     * The make up page language
     * @type {string}
     */
    private languageID: string;

    /**
     * Returns the current page identifier
     * @return {string}       The page identifier
     */
    public getPageID(): string {
        return this.page_id;
    }

    /**
     * Set the make up page identifier
     * @param  {string} page_id The page identifier
     * @return {string}         The currently page identifier
     */
    public setPageID( page_id: string ): string {
        return this.page_id = page_id;
    }

    /**
     * Set the make up page language
     * @param  {string} page_id The page language
     * @return {string}         The currently page language
     */
    public setLanguageID( languageID: string ): string {
        return this.languageID = languageID;
    }

    /**
     * Returns the current page language
     * @return {string}       The page language
     */
    public getLanguageID(): string {
        return this.languageID;
    }

    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Makeup.MakeupListPageVersions, EVA.Makeup.MakeupListPageVersionsResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Makeup.MakeupListPageVersions, EVA.Makeup.MakeupListPageVersionsResponse>> {

        // Build the request data
        //
        var requestData: EVA.Makeup.MakeupListPageVersions = <EVA.Makeup.MakeupListPageVersions>{
            ID: this.page_id
        };

        if ( this.languageID ) {
            requestData.LanguageID = this.languageID;
        }

        // Apply PageConfig parameters directly
        //
        _.assign( requestData, _.omit( this.getPageConfiguration(), [ "Filter" ] ) );

        return super.fetch( "MakeupListPageVersions", requestData, `/api/v1/makeup/page/${requestData.ID}/version_history`, EXMLHttpRequestMethod.GET );
    }
}

export class MakeupListBlocks extends FetchablePagedList<EVA.Makeup.BlockDto, EVA.Makeup.MakeupListBlocks, EVA.Makeup.MakeupListBlocksResponse>
{
    /**
     * The specifc type of blocks to retrieve
     * @type {string}
     */
    private type: string;

    /**
     * Returns the current requested block type
     * @return {string}       The block type
     */
    public getType(): string {
        return this.type;
    }

    /**
     * Set the block type to retrieve
     * @param  {string} type The new block type
     * @return {string}       The currently set block type
     */
    public setType( type: string ): string {
        return this.type = type;
    }

    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Makeup.MakeupListBlocks, EVA.Makeup.MakeupListBlocksResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Makeup.MakeupListBlocks, EVA.Makeup.MakeupListBlocksResponse>> {

        // Build the request data
        //
        var requestData: EVA.Makeup.MakeupListBlocks = <EVA.Makeup.MakeupListBlocks>{};
        if ( this.type ) {
            requestData.Type = this.type;
        }

        // Apply PageConfig parameters directly
        //
        _.assign( requestData, _.omit( this.getPageConfiguration(), [ "Filter" ] ) );

        return super.fetch( "MakeupListBlocks", requestData, `/api/v1/makeup/block`, EXMLHttpRequestMethod.GET );
    }
}

export class MakeupListBlockVersions extends FetchablePagedList<EVA.Makeup.BlockDto, EVA.Makeup.MakeupListBlockVersions, EVA.Makeup.MakeupListBlockVersionsResponse>
{
    /**
     * The make up block identifier
     * @type {string}
     */
    private block_id: string;

    /**
     * Returns the current block identifier
     * @return {string}       The block identifier
     */
    public getBlockID(): string {
        return this.block_id;
    }

    /**
     * Set the make up block identifier
     * @param  {string} block_id The block identifier
     * @return {string}         The currently block identifier
     */
    public setBlockID( block_id: string ): string {
        return this.block_id = block_id;
    }

    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Makeup.MakeupListBockVersions, EVA.Makeup.MakeupListBlockVersionsResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Makeup.MakeupListBlockVersions, EVA.Makeup.MakeupListBlockVersionsResponse>> {

        // Build the request data
        //
        var requestData: EVA.Makeup.MakeupListBlockVersions = <EVA.Makeup.MakeupListBlockVersions>{
            ID: this.block_id
        };

        // Apply PageConfig parameters directly
        //
        _.assign( requestData, _.omit( this.getPageConfiguration(), [ "Filter" ] ) );

        return super.fetch( "MakeupListBlockVersions", requestData, `/api/v1/makeup/block/${requestData.ID}/version_history`, EXMLHttpRequestMethod.GET );
    }
}

export class MakeupListMenus extends FetchablePagedList<EVA.Makeup.MenuDto, EVA.Makeup.MakeupListMenus, EVA.Makeup.MakeupListMenusResponse>
{
    /**
     * The language identifier
     * @type {string}
     */
    private languageID: string;

    /**
     * Returns the current language identifier
     * @return {string} The language identifier
     */
    public getLanguageID(): string {
        return this.languageID;
    }

    /**
     * Set the language identifier
     * @param  {string} language The language identifier
     */
    public setLanguageID( languageID: string ): string {
        return this.languageID = languageID;
    }

    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Makeup.MakeupListMenus, EVA.Makeup.MakeupListMenusResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Makeup.MakeupListMenus, EVA.Makeup.MakeupListMenusResponse>> {

        // Build the request data
        //
        var requestData: EVA.Makeup.MakeupListMenus = <EVA.Makeup.MakeupListMenus>{};

        // Set the optional language flag
        //
        if ( this.languageID ) {
            requestData.LanguageID = this.languageID;
        }

        // Apply PageConfig parameters directly
        //
        _.assign( requestData, _.omit( this.getPageConfiguration(), [ "Filter" ] ) );

        return super.fetch( "MakeupListMenus", requestData, `/api/v1/makeup/menu`, EXMLHttpRequestMethod.GET );
    }
}
