/**
 * The orders module provides access to the order data from the EVA back-end
 * @preferred
 */
import { FetchableItem, FetchablePagedList } from "./base";
import { IEVAServiceResponse } from "../service";
import { LoggedInUser, User } from "./user";
import { Payments } from "./payments";
export interface IEVAProduceInvoiceOptions {
    downloadInvoice?: boolean;
    emailAddress?: string;
    invoiceId?: number;
    printInvoice?: boolean;
    remark?: string;
    stationId?: string;
}
export interface IEVAProduceDocumentsOptions {
    printInvoice?: boolean;
    emailInvoice?: boolean;
    printReceipt?: boolean;
    stationId?: number;
    orderID?: number;
    emailAddress?: string;
    remark?: string;
}
/**
 * Provides access to an order available in EVA
 */
export declare class Order extends FetchableItem<EVA.Core.OrderDto, EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse> {
    /**
     * The payment options for the order
     * @type {Payment}
     */
    payment: Payments;
    /**
     * The current open amount still to be paid for the order
     * @type {number}
     */
    private openAmount;
    /**
     * The value of the items in the order with VAT
     * @type {number}
     */
    private totalAmountInTax;
    /**
     * Order state indicating the order is completed
     * @type {boolean}
     */
    private _isCompleted;
    /**
     * Order state indicating the order is shipped
     * @type {boolean}
     */
    private _isShipped;
    /**
     * Order state indicating the order is paid
     * @type {boolean}
     */
    private _isPaid;
    /**
     * Order state indicating the order is invoiced
     * @type {boolean}
     */
    private _isInvoiced;
    /**
     * Order state indicating the order is cancelled
     * @type {boolean}
     */
    private _isCancelled;
    /**
     * The list of organizations where orderline items are reserved for pickup
     * @type {Array<any>}
     */
    private _stockOrganizationUnits;
    /**
     * The list of requested dates for orderline item deliveries
     * @type {Array<any>}
     */
    private _requestedDates;
    /**
     * Order requirement validation data
     * @type {EVA.Core.Validation.RequiredData}
     */
    private requirements;
    /**
     * This class wraps an order from EVA and the operations you can perform on it.
     * You should supply the order identifier when creating an instance of this class
     *
     * @param  {number | string}    id      The identifier of the order to retrieve
     * @return {void}                       This method doesn't return anything
     */
    constructor(id?: number | string);
    /**
     * Returns if the order state 'completed' applies to the order
     * @return {boolean} Indicates the state is active
     */
    isCompleted(): boolean;
    /**
     * Returns if the order state 'shipped' applies to the order
     * @return {boolean} Indicates the state is active
     */
    isShipped(): boolean;
    /**
     * Returns if the order state 'paid' applies to the order
     * @return {boolean} Indicates the state is active
     */
    isPaid(): boolean;
    /**
     * Returns if the order state 'invoiced' applies to the order
     * @return {boolean} Indicates the state is active
     */
    isInvoiced(): boolean;
    /**
     * Returns if the order state 'cancelled' applies to the order
     * @return {boolean} Indicates the state is active
     */
    isCancelled(): boolean;
    /**
     * Returns the current open amount still to be paid for the order
     * @return {number}
     */
    getOpenAmount(): number;
    /**
     * Returns the total value of all the items in the order with VAT
     * @return {number} The value of the items in the order with VAT
     */
    getTotalAmountInTax(): number;
    /**
     * Return order lines by type
     * @return {Array<EVA.Core.OrderLineDto>} Array of order lines
     */
    getOrderLinesByType(orderLineType: number | EVA.Core.OrderLineTypes): Array<EVA.Core.OrderLineDto>;
    /**
     * Return the total amount of shipping costs
     * @return {number} The total value of the shipping costs
     */
    getTotalShippingAmount(inTax?: boolean): number;
    /**
     * Return the total amount of ShippingCosts
     * @return {number} The total value of the shipping costs
     */
    getTotalDiscountAmount(inTax?: boolean): number;
    /**
     * Returns the current order requirement validation data
     * @return {EVA.Core.Validation.RequiredData} The order requirement validation data
     */
    getRequirements(): EVA.Core.RequiredData;
    /**
     * Checks if there are any invalid requirements in the current order requirement validation data
     * @return {boolean} Indicates if the order requirements are all met or not
     */
    isValid(): boolean;
    /**
     * Fetches and updates the order requirement validation data
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetRequiredDataForOrder, EVA.Core.Services.GetRequiredDataForOrderResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    validate(requiredFor?: number, storeRequirements?: boolean): Promise<IEVAServiceResponse<EVA.Core.Services.GetRequiredDataForOrder, EVA.Core.Services.GetRequiredDataForOrderResponse>>;
    /**
     * Returns the list of different organizations where orderline items are te be picked up
     * @return {number} The list of organizations
     */
    readonly stockOrganizationUnits: Array<any>;
    /**
     * Returns the list of different organizations where orderline items are te be picked up
     * @return {number} The list of organizations
     */
    readonly requestedDates: Array<any>;
    /**
     * Fetches the order details
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    fetch(): Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>;
    /**
     * Sets the current item details using the fetched service response.
     * Extending classes should override this method to pull the details from response
     * @param  {EVA.Core.Services.GetOrderResponse}  response    The response from the service call
     * @return {EVA.Core.OrderDto}              The order details
     */
    setData(response: any): EVA.Core.OrderDto;
    /**
     * Attaches a customer to the order
     * @param  {LoggedInUser}       customer       The current user class that is to be set as the customer for the order
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    setCustomer(customer: LoggedInUser | User): Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>;
    /**
     * Sets the currency for the order
     * @param  {string}       currencyID       The identifier of the currency
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    setCurrency(currencyID: string): Promise<{}>;
    /**
     * Attaches a customer to the order
     * @param  {LoggedInUser}       customer       The current user class that is to be set as the customer for the order
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    setPickupOrganizationUnit(organizationUnitID: number): Promise<IEVAServiceResponse<EVA.Core.Services.SetPickupOrganizationUnit, EVA.Core.Services.GetOrderResponse>>;
    /**
     * Places the order. It's usually wise to call validate(...) before trying to place an order
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.PlaceOrder, EVA.Core.Services.PlaceOrderResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    placeOrder(): Promise<IEVAServiceResponse<EVA.Core.Services.PlaceOrder, EVA.Core.Services.PlaceOrderResponse>>;
    produceInvoice(options: IEVAProduceInvoiceOptions): Promise<IEVAServiceResponse<EVA.Core.Services.ProduceInvoice, EVA.Core.ResourceResponseMessage>>;
    produceDocuments(options: IEVAProduceDocumentsOptions): Promise<IEVAServiceResponse<EVA.Core.Services.ProduceDocuments, EVA.Core.Services.ProduceDocumentsResponse>>;
    /**
     * Get the download url for this order
     *
     * @return {Promise<string>} The URL to the download of this order
     */
    getDownloadUrl(): Promise<string>;
    getReturnableStatusForOrder(): Promise<IEVAServiceResponse<EVA.Core.Services.GetReturnableStatusForOrder, EVA.Core.Services.GetReturnableStatusForOrderResponse>>;
    getReturnOrdersForOrder(): Promise<IEVAServiceResponse<EVA.CRM.Core.GetReturnOrdersForOrder, EVA.CRM.Core.GetReturnOrdersForOrderResponse>>;
    createCustomerReturn(lines: Array<EVA.CRM.Core.CustomerReturnLine>): Promise<IEVAServiceResponse<EVA.CRM.Core.CreateCustomerReturn, EVA.Core.EmptyResponseMessage>>;
    /**
     * For B2B orders
     *
     * Fetches the order view details
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetConfigurableOrderView, EVA.Core.Services.GetConfigurableOrderViewResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    fetchOrderView(logicalLevel?: string, groupByProperty?: string): Promise<IEVAServiceResponse<EVA.Core.Services.GetConfigurableOrderView, EVA.Core.Services.GetConfigurableOrderViewResponse>>;
    /**
     * Set the order reference
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.SetCustomerReferenceOnOrder, EVA.Core.EmptyResponseMessage>>} A promise resolving when the fetch from the back-end has completed
     */
    setOrderReference(reference: string): Promise<IEVAServiceResponse<EVA.Core.Services.SetCustomerReferenceOnOrder, EVA.Core.EmptyResponseMessage>>;
    /**
     * Set the shopping basket to an existing order. Removes any existing items in the shopping cart
     * @param  {number|Order}       order       Either an order id or an order class to set as the shopping cart
     * @return {Promise}                        A promise resolving when the call to the back-end has completed
     */
    attach(order: number | Order): Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>;
    /**
     * Clears the entire shopping basket
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>} A promise resolving when the call to the back-end has completed
     */
    detach(): Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>;
    /**
     * Set the order reference
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.ListOrderLines, EVA.Core.Services.ListOrderLinesResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    listOrderLines(orderLineIds?: number[], start?: number, limit?: number): Promise<IEVAServiceResponse<EVA.Core.Services.ListOrderLines, EVA.Core.Services.ListOrderLinesResponse>>;
    /**
     * Update customer address data for this order
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.ListOrderLines, EVA.Core.Services.ListOrderLinesResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    copyAddressesFromCustomer(): Promise<IEVAServiceResponse<EVA.Core.Services.UpdateOrderAddresses, EVA.Core.Services.UpdateOrderAddressesResponse>>;
    /**
     * Provides access to a list of shipments for this order
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.ListOrderShipments, EVA.Core.Services.ListOrderShipmentsResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    listShipments(): Promise<IEVAServiceResponse<EVA.Core.Services.ListOrderShipments, EVA.Core.Services.ListOrderShipmentsResponse>>;
    /**
     * Provides access to a list of blobs for this order
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.ListBlobsForOrder, EVA.Core.Services.ListBlobsForOrderResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    listBlobs(): Promise<IEVAServiceResponse<EVA.Core.Services.ListBlobsForOrder, EVA.Core.Services.ListBlobsForOrderResponse>>;
    /**
     * Set the billing address of the order using an AddressBook item
     *
     * @param {number} addressBookId The ID of the AddressBook item
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    setBillingAddressFromAddressBook(addressBookId: number): Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>;
    /**
     * Set the billing address of the order using an raw address
     *
     * @param {number} address The address to set as billing address on the order
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    setBillingAddress(address: EVA.Core.AddressDto): Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>;
    /**
     * Set the shipping address of the order using an AddressBook item
     *
     * @param {number} addressBookId The ID of the AddressBook item
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>} A promise resolving when the call to the back-end has completed
     */
    setShippingAddressFromAddressBook(addressBookId: number): Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>;
    /**
     * Set the shipping address of the order using an raw address
     *
     * @param {number} address The address to set as shipping address on the order
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    setShippingAddress(address: EVA.Core.AddressDto): Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>;
    /**
     * Sets a custom order data field
     *
     * @param {string} key The field name
     * @param {string} value The value (serialized string)
     * @returns {Promise<IEVAServiceResponse<EVA.Core.Services.SetCustomOrderData, EVA.Core.EmptyResponseMessage>>}
     * @memberof Order
     */
    setCustomOrderData(key: string, value: string): Promise<IEVAServiceResponse<EVA.Core.Services.SetCustomOrderData, EVA.Core.EmptyResponseMessage>>;
    getCustomOrderData(key: string): Promise<IEVAServiceResponse<EVA.Core.Services.GetCustomOrderData, EVA.Core.Services.GetCustomOrderDataResponse>>;
}
/**
 * Provides access to a list of customer orders available in EVA
 */
export declare class CustomerOrderList extends FetchablePagedList<Order, EVA.Core.Services.ListOrdersForCustomer, EVA.Core.Services.ListOrdersForCustomerResponse> {
    /**
     * Sets the current item details using the fetched service response.
     * Extending classes should override this method to pull the details from response
     * @param  {RES} response The response from the service call
     * @return {T}            The item details
     */
    setData(response: any): Order[];
    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Core.OrderDto, EVA.Core.Services.ListOrdersForCustomer, EVA.Core.Services.ListOrdersForCustomerResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    fetch(): Promise<IEVAServiceResponse<EVA.Core.Services.ListOrdersForCustomer, EVA.Core.Services.ListOrdersForCustomerResponse>>;
}
/**
 * Provides access to a list of customer orders available in EVA
 */
export declare class SearchOrderList extends FetchablePagedList<EVA.Core.OrderSearchResultItem, EVA.Core.Services.SearchOrders, EVA.Core.Services.SearchOrdersResponse> {
    /**
     * The query string to search for in the products catalog
     * @type {string}
     */
    private query;
    /**
     * The list of order ids to fetch
     * @type {Array<number>}
     */
    private orderIds;
    /**
     * The organization unit to retrieve the orders for
     * @type {number}
     */
    private organizationID;
    /**
     * Returns the current query string
     * @return {string}       The configured query string
     */
    getQuery(): string;
    /**
     * Set the organization id to search for
     * @param  {number} query The organization id
     * @return {number}       The configured organization id
     */
    setOrganizationID(id: number): number;
    /**
     * Returns the current organization id
     * @return {number}       The configured organization id
     */
    getOrganizationID(): number;
    /**
     * Set the query to search for
     * @param  {string} query The new query string
     * @return {string}       The configured query string
     */
    setQuery(query: string): string;
    /**
     * Returns the current list of order ids
     * @return {Array<number>} The configured list of order ids
     */
    getOrderIDs(): Array<number>;
    /**
     * Adds an order ID to the list of orders to retrieve
     * @return {Array<number>} The configured list of order ids
     */
    addOrderID(id: number): Array<number>;
    /**
     * Adds an order ID to the list of orders to retrieve
     * @return {Array<number>} The configured list of order ids
     */
    setOrderIDs(ids: number[]): Array<number>;
    /**
     * Clears the configured list of order ids
     * @return {Array<number>} The configured list of order ids
     */
    clearOrderIDs(): Array<number>;
    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.SearchOrders, EVA.Core.Services.SearchOrdersResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    fetch(): Promise<IEVAServiceResponse<EVA.Core.Services.SearchOrders, EVA.Core.Services.SearchOrdersResponse>>;
}
/**
 * Provides access to a list of customer orders available in EVA
 */
export declare class OrdersWithReferenceList extends FetchablePagedList<EVA.Core.OrderWithReference, EVA.Core.Services.ListOrdersWithReference, EVA.Core.Services.ListOrdersWithReferenceResponse> {
    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Core.OrderDto, EVA.Core.Services.ListOrdersWithReference, EVA.Core.Services.ListOrdersWithReferenceResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    fetch(): Promise<IEVAServiceResponse<EVA.Core.Services.ListOrdersWithReference, EVA.Core.Services.ListOrdersWithReferenceResponse>>;
}
