/**
 * The orders module provides access to the order data from the EVA back-end
 * @preferred
 */
import { FetchableItem, FetchablePagedList  } from "./base";
import { EVAService, IEVAServiceResponse    } from "../service";
import { LoggedInUser, User                 } from "./user";
import { Payments                           } from "./payments";
import * as _                                 from "lodash";

export interface IEVAProduceInvoiceOptions {
    downloadInvoice?:   boolean;
    emailAddress?:      string;
    invoiceId?:         number;
    printInvoice?:      boolean;
    remark?:            string;
    stationId?:         string;
}

export interface IEVAProduceDocumentsOptions {
    printInvoice?:      boolean;
    emailInvoice?:      boolean;
    printReceipt?:      boolean;
    stationId?:         number;
    orderID?:           number;
    emailAddress?:      string;
    remark?:            string;
}

/**
 * Provides access to an order available in EVA
 */
export class Order extends FetchableItem<EVA.Core.OrderDto, EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse> {

    /**
     * The payment options for the order
     * @type {Payment}
     */
    public payment: Payments;

    /**
     * The current open amount still to be paid for the order
     * @type {number}
     */
    private openAmount: number;

    /**
     * The value of the items in the order with VAT
     * @type {number}
     */
    private totalAmountInTax: number;

    /**
     * Order state indicating the order is completed
     * @type {boolean}
     */
    private _isCompleted: boolean;

    /**
     * Order state indicating the order is shipped
     * @type {boolean}
     */
    private _isShipped: boolean;

    /**
     * Order state indicating the order is paid
     * @type {boolean}
     */
    private _isPaid: boolean;

    /**
     * Order state indicating the order is invoiced
     * @type {boolean}
     */
    private _isInvoiced: boolean;

    /**
     * Order state indicating the order is cancelled
     * @type {boolean}
     */
    private _isCancelled: boolean;

    /**
     * The list of organizations where orderline items are reserved for pickup
     * @type {Array<any>}
     */
    private _stockOrganizationUnits: Array<any>;

    /**
     * The list of requested dates for orderline item deliveries
     * @type {Array<any>}
     */
    private _requestedDates: Array<any>;

    /**
     * Order requirement validation data
     * @type {EVA.Core.Validation.RequiredData}
     */
    private requirements: EVA.Core.RequiredData;

    /**
     * This class wraps an order from EVA and the operations you can perform on it.
     * You should supply the order identifier when creating an instance of this class
     *
     * @param  {number | string}    id      The identifier of the order to retrieve
     * @return {void}                       This method doesn't return anything
     */
    constructor( id?: number | string ) {
        super( id );

        this.payment = new Payments( this );
    }

    /**
     * Returns if the order state 'completed' applies to the order
     * @return {boolean} Indicates the state is active
     */
    public isCompleted(): boolean {
        return this._isCompleted === true;
    }

    /**
     * Returns if the order state 'shipped' applies to the order
     * @return {boolean} Indicates the state is active
     */
    public isShipped(): boolean {
        return this._isShipped === true;
    }

    /**
     * Returns if the order state 'paid' applies to the order
     * @return {boolean} Indicates the state is active
     */
    public isPaid(): boolean {
        return this._isPaid === true;
    }

    /**
     * Returns if the order state 'invoiced' applies to the order
     * @return {boolean} Indicates the state is active
     */
    public isInvoiced(): boolean {
        return this._isInvoiced === true;
    }

    /**
     * Returns if the order state 'cancelled' applies to the order
     * @return {boolean} Indicates the state is active
     */
    public isCancelled(): boolean {
        return this._isCancelled === true;
    }

    /**
     * Returns the current open amount still to be paid for the order
     * @return {number}
     */
    public getOpenAmount(): number {
        return this.openAmount;
    }

    /**
     * Returns the total value of all the items in the order with VAT
     * @return {number} The value of the items in the order with VAT
     */
    public getTotalAmountInTax(): number {
        return this.totalAmountInTax;
    }

    /**
     * Return order lines by type
     * @return {Array<EVA.Core.OrderLineDto>} Array of order lines
     */
    public getOrderLinesByType( orderLineType: number|EVA.Core.OrderLineTypes ): Array<EVA.Core.OrderLineDto>
    {
        return _.filter( this.data.Lines, ( line ) =>
        {
            return line.Type === orderLineType;
        } );
    }

    /**
     * Return the total amount of shipping costs
     * @return {number} The total value of the shipping costs
     */
    public getTotalShippingAmount( inTax: boolean = true ): number
    {
        let targetField: string;
        if ( inTax )
        {
            targetField = "TotalAmountInTax";
        }
        else
        {
            targetField = "TotalAmount";
        }
        return _.sumBy( this.getOrderLinesByType( 5 ), targetField );
    }

    /**
     * Return the total amount of ShippingCosts
     * @return {number} The total value of the shipping costs
     */
    public getTotalDiscountAmount( inTax: boolean = true ): number
    {
        let targetField: string;
        if ( inTax )
        {
            targetField = "TotalAmountInTax";
        }
        else
        {
            targetField = "TotalAmount";
        }
        return _.sumBy( this.getOrderLinesByType( 1 ), targetField );
    }

    /**
     * Returns the current order requirement validation data
     * @return {EVA.Core.Validation.RequiredData} The order requirement validation data
     */
    public getRequirements(): EVA.Core.RequiredData {
        return this.requirements;
    }

    /**
     * Checks if there are any invalid requirements in the current order requirement validation data
     * @return {boolean} Indicates if the order requirements are all met or not
     */
    public isValid(): boolean {
        var customerRequirementsValid:  boolean = _.find( <any>_.get( this.requirements, "Customer"   ), { IsValid: false } ) === undefined;
        var orderRequirementsValid:     boolean = _.find( <any>_.get( this.requirements, "Order"      ), { IsValid: false } ) === undefined;
        var orderLineRequirementsValid: boolean = _.find( <any>_.get( this.requirements, "OrderLines" ), { IsValid: false } ) === undefined;

        return customerRequirementsValid && orderRequirementsValid && orderLineRequirementsValid;
    }

    /**
     * Fetches and updates the order requirement validation data
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetRequiredDataForOrder, EVA.Core.Services.GetRequiredDataForOrderResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public validate( requiredFor: number = 15, storeRequirements: boolean = true ): Promise<IEVAServiceResponse<EVA.Core.Services.GetRequiredDataForOrder, EVA.Core.Services.GetRequiredDataForOrderResponse>> {
        // Create our service instance
        //
        var service = new EVAService<EVA.Core.Services.GetRequiredDataForOrder, EVA.Core.Services.GetRequiredDataForOrderResponse>( "GetRequiredDataForOrder" );

        // Build the request data
        // The base class will add things like SessionID so to keep typescript
        // happy we're using an 'any' type object here
        //
        // RequiredFor {
        //   PlaceOrder = 1,
        //   Payment = 2,
        //   Ship = 4,
        //   Invoice = 8,
        //   All = 15,
        // }
        //
        var requestData: any = {
            OrderID:         this.getId(),
            RequiredFor:     requiredFor
        };

        return new Promise( ( resolve, reject ) => {
            service.call( requestData )
            .then( ( serviceResult ) =>
            {
                if ( storeRequirements )
                {
                    this.requirements = serviceResult.response.RequiredData;
                }

                resolve( serviceResult );
            } )
            .catch( reject );
        } );
    }

    /**
     * Returns the list of different organizations where orderline items are te be picked up
     * @return {number} The list of organizations
     */
    get stockOrganizationUnits(): Array<any>
    {
        return this._stockOrganizationUnits;
    }

    /**
     * Returns the list of different organizations where orderline items are te be picked up
     * @return {number} The list of organizations
     */
    get requestedDates(): Array<any>
    {
        return this._requestedDates;
    }

    /**
     * Fetches the order details
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>> {
        // Build the request data
        //
        var requestOrderData: EVA.Core.Services.GetOrder = <EVA.Core.Services.GetOrder>{
            OrderID: this.getId()
        };

        return super.fetch( "GetOrder", requestOrderData );
    }

    /**
     * Sets the current item details using the fetched service response.
     * Extending classes should override this method to pull the details from response
     * @param  {EVA.Core.Services.GetOrderResponse}  response    The response from the service call
     * @return {EVA.Core.OrderDto}              The order details
     */
    public setData( response: any ): EVA.Core.OrderDto {

        // Collect the data exposed outside of the OrderDto
        //
        this.openAmount       = response.OpenAmount         || 0;
        this.totalAmountInTax = response.TotalAmountInTax   || 0;
        this._isCompleted     = response.IsCompleted    === true;
        this._isShipped       = response.IsShipped      === true;
        this._isPaid          = response.IsPaid         === true;
        this._isInvoiced      = response.IsInvoiced     === true;
        this._isCancelled     = response.IsCancelled    === true;

        // Set the order data
        //
        this.data = response.Result || response.Order || response.ShoppingCart || response;

        // Set or update the ID
        //
        this.id = this.data.ID;

        // Collect the stock organizations from the order lines
        // This used to be per line but is now centralised on the order
        //
        this._stockOrganizationUnits = [ this.data.OrganizationUnitID ];

        // Collect the requested delivery dates from the order lines
        //
        this._requestedDates = _.without( _.uniq( _.map( this.data.Lines, ( line ) =>
        {
            if ( line.LineActionType === 4 && line.RequestedDate )
            {
                return line.RequestedDate;
            }
        } ) ), undefined );

        return this.data;
    }

    /**
     * Attaches a customer to the order
     * @param  {LoggedInUser}       customer       The current user class that is to be set as the customer for the order
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public setCustomer( customer: LoggedInUser|User ): Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create our service instance
            //
            var service = new EVAService<EVA.Core.Services.AttachCustomerToOrder, EVA.Core.EmptyResponseMessage>( "AttachCustomerToOrder" );

            // Build the request data
            // The base class will add things like SessionID so to keep typescript
            // happy we're using an 'any' type object here
            //
            var requestData: any = {
                UserID:  customer.getId(),
                OrderID: this.getId()
            };

            // Perform the call and capture the resulting promise
            //
            service.call( requestData )
            .then( () =>
            {
                this.fetch().then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }

    /**
     * Sets the currency for the order
     * @param  {string}       currencyID       The identifier of the currency
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public setCurrency( currencyID: string )
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            var service = new EVAService<EVA.Core.Services.UpdateOrderCurrency, EVA.Core.EmptyResponseMessage>( "UpdateOrderCurrency" );

            var requestData: EVA.Core.Services.UpdateOrderCurrency = {
                OrderID:    <number>this.getId(),
                CurrencyID: currencyID,
                SessionID:  null,
                __response: null
            };

            // Perform the call and capture the resulting promise
            //
            service.call( requestData )
            .then( () =>
            {
                this.fetch().then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }

    /**
     * Attaches a customer to the order
     * @param  {LoggedInUser}       customer       The current user class that is to be set as the customer for the order
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public setPickupOrganizationUnit( organizationUnitID: number ): Promise<IEVAServiceResponse<EVA.Core.Services.SetPickupOrganizationUnit, EVA.Core.Services.GetOrderResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create our service instance
            //
            var service = new EVAService<EVA.Core.Services.SetPickupOrganizationUnit, EVA.Core.ShoppingCartResponse>( "SetPickupOrganizationUnit" );

            // Build the request data
            // The base class will add things like SessionID so to keep typescript
            // happy we're using an 'any' type object here
            //
            var requestData: EVA.Core.Services.SetPickupOrganizationUnit = {
                OrganizationUnitID: organizationUnitID,
                OrderID:            this.getId() as number,
                SessionID:          undefined,
                __response:         undefined
            };

            // Perform the call and capture the resulting promise
            //
            service.call( requestData )
            .then( ( serviceResult ) =>
            {
                this.setData( serviceResult.response );
                resolve( serviceResult );
            } )
            .catch( reject );
        } );
    }

    /**
     * Places the order. It's usually wise to call validate(...) before trying to place an order
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.PlaceOrder, EVA.Core.Services.PlaceOrderResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public placeOrder(): Promise<IEVAServiceResponse<EVA.Core.Services.PlaceOrder, EVA.Core.Services.PlaceOrderResponse>> {

        // Create our service instance
        //
        var service = new EVAService<EVA.Core.Services.PlaceOrder, EVA.Core.Services.PlaceOrderResponse>( "PlaceOrder" );

        // Build the request data
        // The base class will add things like SessionID so to keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: any = {
            OrderID:         this.getId()
        };

        // Perform the call and capture the resulting promise
        //
        var promise = service.call( requestData );

        return promise;
    }

    public produceInvoice( options: IEVAProduceInvoiceOptions ): Promise<IEVAServiceResponse<EVA.Core.Services.ProduceInvoice, EVA.Core.ResourceResponseMessage>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create our service instance
            //
            var service = new EVAService<EVA.Core.Services.ProduceInvoice, EVA.Core.ResourceResponseMessage>( "ProduceInvoice" );

            // Build the request data
            // The base class will add things like SessionID so to keep typescript
            // happy we're using an 'any' type object here
            //
            var requestData: any = {
                OrderID: this.data.ID
            };

            // Property mapping between IEVAProduceInvoiceOptions and the service request
            //
            let items: any = {
                "DownloadInvoice":    options.downloadInvoice
            ,   "EmailAddress":       options.emailAddress
            ,   "PrintInvoice":       options.printInvoice
            ,   "Remark":             options.remark
            ,   "StationID":          options.stationId
            };

            let key: string;

            for ( key in items )
            {
                if ( items[ key ] )
                {
                    requestData[ key ] = items[ key ];
                }
            }

            // Perform the call and capture the resulting promise
            //
            service.call( requestData )
            .then( resolve, reject );
        } );
    }

    public produceDocuments( options: IEVAProduceDocumentsOptions ): Promise<IEVAServiceResponse<EVA.Core.Services.ProduceDocuments, EVA.Core.Services.ProduceDocumentsResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create our service instance
            //
            var service = new EVAService<EVA.Core.Services.ProduceDocuments, EVA.Core.Services.ProduceDocumentsResponse>( "ProduceDocuments" );

            // Build the request data
            // The base class will add things like SessionID so to keep typescript
            // happy we're using an 'any' type object here
            //
            var requestData: any = {
                PrintInvoice:       options.printInvoice
            ,   PrintReceipt:       options.printReceipt
            ,   EmailInvoice:       options.emailInvoice
            ,   EmailAddress:       options.emailAddress
            ,   StationID:          options.stationId
            ,   OrderID:            this.id
            ,   Remark:             options.remark
            };

            // Perform the call and capture the resulting promise
            //
            return service.call( requestData ).then( resolve, reject );
        } );
    }

    /**
     * Get the download url for this order
     *
     * @return {Promise<string>} The URL to the download of this order
     */
    public getDownloadUrl(): Promise<string>
    {
        return this.produceInvoice( { downloadInvoice: true } )
        .then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.ProduceInvoice, EVA.Core.ResourceResponseMessage> ) =>
        {
            return serviceResult.response.Url;
        } );
    }

    public getReturnableStatusForOrder(): Promise<IEVAServiceResponse<EVA.Core.Services.GetReturnableStatusForOrder, EVA.Core.Services.GetReturnableStatusForOrderResponse>> {
       return new Promise( ( resolve: any, reject: any ) =>
       {
           var service = new EVAService<EVA.Core.Services.GetReturnableStatusForOrder, EVA.Core.Services.GetReturnableStatusForOrderResponse>( "GetReturnableStatusForOrder" );

           var requestData: any = {
               OrderID: this.id
           };
           service.call( requestData )
           .then( (returnResolve) => { resolve(returnResolve); }, reject );
       } );
    }

    public getReturnOrdersForOrder(): Promise<IEVAServiceResponse<EVA.CRM.Core.GetReturnOrdersForOrder, EVA.CRM.Core.GetReturnOrdersForOrderResponse>> {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            var service = new EVAService<EVA.CRM.Core.GetReturnOrdersForOrder, EVA.CRM.Core.GetReturnOrdersForOrderResponse>( "GetReturnOrdersForOrder" );

            var requestData: EVA.CRM.Core.GetReturnOrdersForOrder = {
                OrderID: this.id as number,
                __response: undefined
            };

            service.call( requestData )
            .then( (returnResolve) => { resolve( returnResolve ); }, reject );
        } );
    }

    public createCustomerReturn( lines: Array<EVA.CRM.Core.CustomerReturnLine>): Promise<IEVAServiceResponse<EVA.CRM.Core.CreateCustomerReturn, EVA.Core.EmptyResponseMessage>> {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            var service = new EVAService<EVA.CRM.Core.CreateCustomerReturn, EVA.Core.EmptyResponseMessage>( "CreateCustomerReturn" );

            var requestData: any = {
                OrderID: this.id,
                Lines: lines
            };

            service.call( requestData )
            .then( (returnResolve) => { resolve(returnResolve); }, reject );
        } );
    }

   /**
    * For B2B orders
    *
    * Fetches the order view details
    * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetConfigurableOrderView, EVA.Core.Services.GetConfigurableOrderViewResponse>>} A promise resolving when the fetch from the back-end has completed
    */
    public fetchOrderView( logicalLevel?: string, groupByProperty?: string ): Promise<IEVAServiceResponse<EVA.Core.Services.GetConfigurableOrderView, EVA.Core.Services.GetConfigurableOrderViewResponse>> {
        // Build the request data
        //
        var service = new EVAService<EVA.Core.Services.GetConfigurableOrderView, EVA.Core.Services.GetConfigurableOrderViewResponse>( "GetConfigurableOrderView" );
        var requestData: EVA.Core.Services.GetConfigurableOrderView = <EVA.Core.Services.GetConfigurableOrderView>{
            OrderID:             this.getId(),
            LogicalLevel:        logicalLevel,
            GroupByPropertyType: groupByProperty
        };

        return service.call( requestData );
    }

   /**
    * Set the order reference
    * @return {Promise<IEVAServiceResponse<EVA.Core.Services.SetCustomerReferenceOnOrder, EVA.Core.EmptyResponseMessage>>} A promise resolving when the fetch from the back-end has completed
    */
    public setOrderReference( reference: string ): Promise<IEVAServiceResponse<EVA.Core.Services.SetCustomerReferenceOnOrder, EVA.Core.EmptyResponseMessage>> {
        // Build the request data
        //
        var service = new EVAService<EVA.Core.Services.SetCustomerReferenceOnOrder, EVA.Core.EmptyResponseMessage>( "SetCustomerReferenceOnOrder" );
        var requestData: EVA.Core.Services.SetCustomerReferenceOnOrder = <EVA.Core.Services.SetCustomerReferenceOnOrder>{
            OrderID:   this.getId(),
            Reference: reference
        };

        return service.call( requestData );
    }

    /**
     * Set the shopping basket to an existing order. Removes any existing items in the shopping cart
     * @param  {number|Order}       order       Either an order id or an order class to set as the shopping cart
     * @return {Promise}                        A promise resolving when the call to the back-end has completed
     */
    public attach( order: number|Order): Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create our service instance
            //
            var service = new EVAService<EVA.Core.Services.AttachOrderToSession, EVA.Core.EmptyResponseMessage>( "AttachOrderToSession" );

            var orderId: number|string;
            if ( order instanceof Order )
            {
                orderId = order.getId();
            }
            else
            {
                orderId = <number>order;
            }

            // Build the request data
            // The base class will add things like SessionID so to keep typescript
            // happy we're using an 'any' type object here
            //
            var requestData: any = {
                OrderID: orderId,
            };

            // Perform the call and capture the resulting promise
            //
            service.call( requestData )
            .then( () =>
            {
                this.fetch().then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }

    /**
     * Clears the entire shopping basket
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>} A promise resolving when the call to the back-end has completed
     */
    public detach(): Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create our service instance
            //
            var service = new EVAService<EVA.Core.Services.DetachOrderFromSession, EVA.Core.EmptyResponseMessage>( "DetachOrderFromSession" );

            // Perform the call and capture the resulting promise
            //
            service.call()
            .then( () =>
            {
                this.fetch().then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }

   /**
    * Set the order reference
    * @return {Promise<IEVAServiceResponse<EVA.Core.Services.ListOrderLines, EVA.Core.Services.ListOrderLinesResponse>>} A promise resolving when the fetch from the back-end has completed
    */
    public listOrderLines( orderLineIds?: number[], start = 0, limit = 20 ): Promise<IEVAServiceResponse<EVA.Core.Services.ListOrderLines, EVA.Core.Services.ListOrderLinesResponse>> {

        var service = new EVAService<EVA.Core.Services.ListOrderLines, EVA.Core.Services.ListOrderLinesResponse>( "ListOrderLines" );

        // Build the request data
        //
        var requestData: EVA.Core.Services.ListOrderLines = <EVA.Core.Services.ListOrderLines>{
            OrderID:   this.getId()
        };

        if ( !_.isEmpty( orderLineIds ) ) {
            requestData.PageConfig = {
                Start:         0,
                Limit:         orderLineIds.length || limit,
                SortProperty:  "ID",
                SortDirection: 0,
                Filter: {
                    "ID":      _.map( orderLineIds, ( id ) => `${id}` ).join("," )
                }
            };
        }

        return service.call( requestData );
    }

    /**
     * Update customer address data for this order
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.ListOrderLines, EVA.Core.Services.ListOrderLinesResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public copyAddressesFromCustomer() {

        var service = new EVAService<EVA.Core.Services.UpdateOrderAddresses, EVA.Core.Services.UpdateOrderAddressesResponse>( "UpdateOrderAddresses" );

        // Build the request data
        //
        var requestData: EVA.Core.Services.UpdateOrderAddresses = <EVA.Core.Services.UpdateOrderAddresses>{
            OrderID:   this.getId()
        ,   CopyBillingAddressFromCustomer: true
        ,   CopyShippingAddressFromCustomer: true
        };

        return service.call( requestData );
    }

    /**
     * Provides access to a list of shipments for this order
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.ListOrderShipments, EVA.Core.Services.ListOrderShipmentsResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public listShipments(): Promise<IEVAServiceResponse<EVA.Core.Services.ListOrderShipments, EVA.Core.Services.ListOrderShipmentsResponse>> {

        var service = new EVAService<EVA.Core.Services.ListOrderShipments, EVA.Core.Services.ListOrderShipmentsResponse>( "ListOrderShipments" );

        // Build the request data
        //
        var requestData: EVA.Core.Services.ListOrderShipments = <EVA.Core.Services.ListOrderShipments>{
            OrderID:   this.getId()
        };

        return service.call( requestData );
    }

    /**
     * Provides access to a list of blobs for this order
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.ListBlobsForOrder, EVA.Core.Services.ListBlobsForOrderResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public listBlobs(): Promise<IEVAServiceResponse<EVA.Core.Services.ListBlobsForOrder, EVA.Core.Services.ListBlobsForOrderResponse>> {

        var service = new EVAService<EVA.Core.Services.ListBlobsForOrder, EVA.Core.Services.ListBlobsForOrderResponse>( "ListBlobsForOrder" );

        // Build the request data
        //
        var requestData: EVA.Core.Services.ListBlobsForOrder = <EVA.Core.Services.ListBlobsForOrder>{
            OrderID:   this.getId()
        };

        return service.call( requestData );
    }

    /**
     * Set the billing address of the order using an AddressBook item
     *
     * @param {number} addressBookId The ID of the AddressBook item
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public setBillingAddressFromAddressBook( addressBookId: number ): Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            var service = new EVAService<EVA.Core.Services.UpdateOrderBillingAddress, EVA.Core.Services.UpdateOrderAddressResponse>( "UpdateOrderBillingAddress" );

            // Build the request data
            //
            var requestData: EVA.Core.Services.UpdateOrderBillingAddress = <EVA.Core.Services.UpdateOrderBillingAddress>{
                OrderID:        this.getId(),
                AddressBookID:  addressBookId
            };

            // Perform the call and capture the resulting promise
            //
            service.call( requestData )
            .then( () =>
            {
                this.fetch().then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }

    /**
     * Set the billing address of the order using an raw address
     *
     * @param {number} address The address to set as billing address on the order
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public setBillingAddress( address: EVA.Core.AddressDto ): Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            var service = new EVAService<EVA.Core.Services.UpdateOrderBillingAddress, EVA.Core.Services.UpdateOrderAddressResponse>( "UpdateOrderBillingAddress" );

            // Build the request data
            //
            var requestData: EVA.Core.Services.UpdateOrderBillingAddress = <EVA.Core.Services.UpdateOrderBillingAddress>{
                OrderID:        this.getId(),
                Address:        address
            };

            // Perform the call and capture the resulting promise
            //
            service.call( requestData )
            .then( () =>
            {
                this.fetch().then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }

    /**
     * Set the shipping address of the order using an AddressBook item
     *
     * @param {number} addressBookId The ID of the AddressBook item
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>} A promise resolving when the call to the back-end has completed
     */
    public setShippingAddressFromAddressBook( addressBookId: number ): Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            var service = new EVAService<EVA.Core.Services.UpdateOrderShippingAddress, EVA.Core.Services.UpdateOrderAddressResponse>( "UpdateOrderShippingAddress" );

            // Build the request data
            //
            var requestData: EVA.Core.Services.UpdateOrderShippingAddress = <EVA.Core.Services.UpdateOrderShippingAddress>{
                OrderID:        this.getId(),
                AddressBookID:  addressBookId
            };

            // Perform the call and capture the resulting promise
            //
            service.call( requestData )
            .then( () =>
            {
                this.fetch().then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }

    /**
     * Set the shipping address of the order using an raw address
     *
     * @param {number} address The address to set as shipping address on the order
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public setShippingAddress( address: EVA.Core.AddressDto ): Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            var service = new EVAService<EVA.Core.Services.UpdateOrderShippingAddress, EVA.Core.Services.UpdateOrderAddressResponse>( "UpdateOrderShippingAddress" );

            // Build the request data
            //
            var requestData: EVA.Core.Services.UpdateOrderShippingAddress = <EVA.Core.Services.UpdateOrderShippingAddress>{
                OrderID:        this.getId(),
                Address:        address
            };

            // Perform the call and capture the resulting promise
            //
            service.call( requestData )
            .then( () =>
            {
                this.fetch().then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }


    /**
     * Sets a custom order data field
     *
     * @param {string} key The field name
     * @param {string} value The value (serialized string)
     * @returns {Promise<IEVAServiceResponse<EVA.Core.Services.SetCustomOrderData, EVA.Core.EmptyResponseMessage>>}
     * @memberof Order
     */
    public setCustomOrderData( key: string, value: string ): Promise<IEVAServiceResponse<EVA.Core.Services.SetCustomOrderData, EVA.Core.EmptyResponseMessage>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create our service instance
            //
            var service = new EVAService<EVA.Core.Services.SetCustomOrderData, EVA.Core.EmptyResponseMessage>( "SetCustomOrderData" );

            // Build the request data
            // The base class will add things like SessionID so to keep typescript
            // happy we're using an 'any' type object here
            //
            var requestData: Partial<EVA.Core.Services.SetCustomOrderData> = {
                OrderID:                  this.getId() as number,
                Key:                      key,
                Value:                    value,
                IfKeyAlreadyExistsOption: 0 // Overwrite
            };

            // Perform the call and capture the resulting promise
            //
            service.call( requestData as EVA.Core.Services.SetCustomOrderData )
            .then( resolve )
            .catch( reject );
        } );
    }

    public getCustomOrderData( key: string ): Promise<IEVAServiceResponse<EVA.Core.Services.GetCustomOrderData, EVA.Core.Services.GetCustomOrderDataResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create our service instance
            //
            var service = new EVAService<EVA.Core.Services.GetCustomOrderData, EVA.Core.Services.GetCustomOrderDataResponse>( "GetCustomOrderData" );

            // Build the request data
            // The base class will add things like SessionID so to keep typescript
            // happy we're using an 'any' type object here
            //
            var requestData: Partial<EVA.Core.Services.GetCustomOrderData> = {
                OrderID:                  this.getId() as number,
                Key:                      key,
            };

            // Perform the call and capture the resulting promise
            //
            service.call( requestData as EVA.Core.Services.GetCustomOrderData )
            .then( ( result ) =>
            {
                // Current response is having a serialisation issue and will return double qouted
                //
                // TODO: Remove this result parsing once the backend fixes it
                //
                result.response.Values = _.map( result.response.Values, ( value ) =>
                {
                    if ( value ) { value = JSON.parse( <any>value ); }
                    return value;
                } );

                resolve( result );
            } )
            .catch( reject );
        } );
    }
}

/**
 * Provides access to a list of customer orders available in EVA
 */
export class CustomerOrderList extends FetchablePagedList<Order, EVA.Core.Services.ListOrdersForCustomer, EVA.Core.Services.ListOrdersForCustomerResponse> {

    /**
     * Sets the current item details using the fetched service response.
     * Extending classes should override this method to pull the details from response
     * @param  {RES} response The response from the service call
     * @return {T}            The item details
     */
    public setData( response: any ): Order[] {
        // The default response is usually EVA.Core.GetResponse
        // An extending class will know exactly what is returned and where
        // the data for <T> can be found
        //
        return this.data = _.map( response.Result.Page, ( page: any ) =>
        {
            let order = new Order();

            order.setData( page );

            return order;
        } );
    }

    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Core.OrderDto, EVA.Core.Services.ListOrdersForCustomer, EVA.Core.Services.ListOrdersForCustomerResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Core.Services.ListOrdersForCustomer, EVA.Core.Services.ListOrdersForCustomerResponse>> {

        // Build the request data
        //
        var requestData: EVA.Core.Services.ListOrdersForCustomer = <EVA.Core.Services.ListOrdersForCustomer>{
            PageConfig: this.getPageConfiguration()
        };

        return super.fetch( "ListOrdersForCustomer", requestData );
    }
}

/**
 * Provides access to a list of customer orders available in EVA
 */
export class SearchOrderList extends FetchablePagedList<EVA.Core.OrderSearchResultItem, EVA.Core.Services.SearchOrders, EVA.Core.Services.SearchOrdersResponse> {
    /**
     * The query string to search for in the products catalog
     * @type {string}
     */
    private query: string = "";

    /**
     * The list of order ids to fetch
     * @type {Array<number>}
     */
    private orderIds: Array<number> = [];

    /**
     * The organization unit to retrieve the orders for
     * @type {number}
     */
    private organizationID: number;

    /**
     * Returns the current query string
     * @return {string}       The configured query string
     */
    public getQuery(): string
    {
        return this.query;
    }

    /**
     * Set the organization id to search for
     * @param  {number} query The organization id
     * @return {number}       The configured organization id
     */
    public setOrganizationID( id: number ): number
    {
        return this.organizationID = id;
    }

    /**
     * Returns the current organization id
     * @return {number}       The configured organization id
     */
    public getOrganizationID(): number
    {
        return this.organizationID;
    }

    /**
     * Set the query to search for
     * @param  {string} query The new query string
     * @return {string}       The configured query string
     */
    public setQuery( query: string ): string
    {
        return this.query = query;
    }

    /**
     * Returns the current list of order ids
     * @return {Array<number>} The configured list of order ids
     */
    public getOrderIDs(): Array<number>
    {
        return this.orderIds;
    }

    /**
     * Adds an order ID to the list of orders to retrieve
     * @return {Array<number>} The configured list of order ids
     */
    public addOrderID( id: number ): Array<number>
    {
        if ( !_.isArray( this.orderIds) )
        {
            this.orderIds = [];
        }
        this.orderIds.push( id );
        return this.orderIds;
    }

    /**
     * Adds an order ID to the list of orders to retrieve
     * @return {Array<number>} The configured list of order ids
     */
    public setOrderIDs( ids: number[] ): Array<number>
    {
        if ( _.isArray( ids ) )
        {
            this.orderIds = ids;
        }
        return this.orderIds;
    }

    /**
     * Clears the configured list of order ids
     * @return {Array<number>} The configured list of order ids
     */
    public clearOrderIDs(): Array<number>
    {
        return this.orderIds = [];
    }

    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.SearchOrders, EVA.Core.Services.SearchOrdersResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Core.Services.SearchOrders, EVA.Core.Services.SearchOrdersResponse>> {

        // Build the request data
        //
        var requestData: EVA.Core.Services.SearchOrders = <EVA.Core.Services.SearchOrders>{
            PageConfig: this.getPageConfiguration(),
        };

        // Amend request with optional parameters
        //
        if ( this.query ) {
            requestData.Query = this.query;
        }
        if ( !_.isEmpty( this.orderIds ) ) {
            requestData.OrderIDs = this.orderIds;
        }
        if ( this.organizationID ) {
            requestData.OrganizationUnitID = this.organizationID;
        }

        return super.fetch( "SearchOrders", requestData );
    }
}

/**
 * Provides access to a list of customer orders available in EVA
 */
export class OrdersWithReferenceList extends FetchablePagedList<EVA.Core.OrderWithReference, EVA.Core.Services.ListOrdersWithReference, EVA.Core.Services.ListOrdersWithReferenceResponse> {
    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Core.OrderDto, EVA.Core.Services.ListOrdersWithReference, EVA.Core.Services.ListOrdersWithReferenceResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Core.Services.ListOrdersWithReference, EVA.Core.Services.ListOrdersWithReferenceResponse>> {

        // Build the request data
        //
        var requestData: EVA.Core.Services.ListOrdersWithReference = <EVA.Core.Services.ListOrdersWithReference>{
            PageConfig: this.getPageConfiguration()
        };

        return super.fetch( "ListOrdersWithReference", requestData );
    }
}
