/**
 * The payments module provides the means to pay for an order using an online payment provider
 * @preferred
 */
import { IEVAServiceResponse } from "../service";
import { ShoppingCart } from "./shopping-cart";
import { Order } from "./orders";
/**
 * Provides access to order payment methods available in EVA
 */
export declare class Payments {
    /**
     * The order to perform the payments for
     * @type {Order}
     */
    private order;
    /**
     * [constructor description]
     * @param  {Order}  order The order to perform the payments for
     * @return {void}         This method doesn't return anything
     */
    constructor(order: Order | ShoppingCart);
    /**
     * Returns the list of available payment methods
     * @return {Promise<IEVAServiceResponse<EVA.Core.GetAvailablePaymentMethods, EVA.Core.GetAvailablePaymentMethodsResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    getAvailableMethods(): Promise<IEVAServiceResponse<EVA.Core.Services.GetAvailablePaymentMethods, EVA.Core.Services.GetAvailablePaymentMethodsResponse>>;
    /**
     * Retuns the list of banks available for iDeal payments
     * @return {Promise<IEVAServiceResponse<EVA.Payment.Acapture.ListAcaptureIdealBanks, EVA.Payment.Acapture.ListAcaptureIdealBanksResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    getIdealBanks(): Promise<IEVAServiceResponse<EVA.Payment.Acapture.ListAcaptureIdealBanks, EVA.Payment.Acapture.ListAcaptureIdealBanksResponse>>;
    getMultiSafePaymentGateWays(orderId: string | number): Promise<IEVAServiceResponse<EVA.Payment.MultiSafepay.GetMultiSafepayGateways, EVA.Payment.MultiSafepay.GetMultiSafepayGatewaysResponse>>;
    /**
     * Retuns the list of user cards available
     * @return {Promise<IEVAServiceResponse<EVA.Payment.UserCard.ListUserCardsForUser, EVA.Payment.UserCard.ListUserCardsForUserResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    listUserCards(userID: number): Promise<IEVAServiceResponse<EVA.Payment.UserCard.ListUserCardsForUser, EVA.Payment.UserCard.ListUserCardsForUserResponse>>;
    /**
     * Pay for the order using a PIN terminal from ADYEN (shop/employee only)
     * @param  {string}  merchantCode     The ADYEN merchant code to use
     * @param  {number}  amount           The amount to pay
     * @return {Promise<IEVAServiceResponse<EVA.Core.CreatePayment, EVA.Core.CreatePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    payWithAdyen(merchantCode: string, amount: number): Promise<IEVAServiceResponse<EVA.Core.Services.CreatePayment, EVA.Core.Services.CreatePaymentResponse>>;
    payWithMultiSafePay(method: string, returnUrl: string, amount?: number): any;
    /**
     * Pay for the order using iDEAL
     * @param  {string}  returnUrl     The URL to return to when the payment has completed
     * @param  {string}  idealBankName The name of the bank for the ideal payment
     * @return {Promise<IEVAServiceResponse<EVA.Core.CreatePayment, EVA.Core.CreatePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    payWithIdeal(returnUrl: string, idealBankName: string): Promise<IEVAServiceResponse<EVA.Core.Services.CreatePayment, EVA.Core.Services.CreatePaymentResponse>>;
    /**
     * Pay for the order using PayPal
     * @param  {string}  returnUrl     The URL to return to when the payment has completed
     * @return {Promise<IEVAServiceResponse<EVA.Core.CreatePayment, EVA.Core.CreatePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    payWithPaypal(returnUrl: string): Promise<IEVAServiceResponse<EVA.Core.Services.CreatePayment, EVA.Core.Services.CreatePaymentResponse>>;
    /**
     * Pay for the order using a MasterCard/Eurocard credit card
     * @param  {string}  returnUrl     The URL to return to when the payment has completed
     * @return {Promise<IEVAServiceResponse<EVA.Core.CreatePayment, EVA.Core.CreatePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    payWithMasterCard(returnUrl: string): Promise<IEVAServiceResponse<EVA.Core.Services.CreatePayment, EVA.Core.Services.CreatePaymentResponse>>;
    /**
     * Pay for the order using an American Express credit card
     * @param  {string}  returnUrl     The URL to return to when the payment has completed
     * @return {Promise<IEVAServiceResponse<EVA.Core.CreatePayment, EVA.Core.CreatePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    payWithAmericanExpress(returnUrl: string): Promise<IEVAServiceResponse<EVA.Core.Services.CreatePayment, EVA.Core.Services.CreatePaymentResponse>>;
    /**
     * Pay for the order using a Visa credit card
     * @param  {string}  returnUrl     The URL to return to when the payment has completed
     * @return {Promise<IEVAServiceResponse<EVA.Core.CreatePayment, EVA.Core.CreatePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    payWithVisa(returnUrl: string): Promise<IEVAServiceResponse<EVA.Core.Services.CreatePayment, EVA.Core.Services.CreatePaymentResponse>>;
    /**
     * Pay for an order with an insurance card (HFS Waardebrief)
     * @param {string}  referenceNumber The reference number for the credit payment
     * @param {string}  policyNumber    The policy number for the credit payment
     * @return {Promise<IEVAServiceResponse<EVA.Core.CreatePayment, EVA.Core.CreatePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    payWithInsuranceCreditPayment(referenceNumber: string, policyNumber: string): Promise<IEVAServiceResponse<EVA.Core.Services.CreatePayment, EVA.Core.Services.CreatePaymentResponse>>;
    /**
     * Pay for an order with a user card
     * @param {number}  userId          The id of the user
     * @param {number}  userCardId      The id of the userCardId
     * @param {amount}  amount          The amount that will be charged from the usercard
     * @return {Promise<IEVAServiceResponse<EVA.Core.CreatePayment, EVA.Core.CreatePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    payWithUserCard(userId: number, userCardId: number, amount: number): Promise<IEVAServiceResponse<EVA.Core.Services.CreatePayment, EVA.Core.Services.CreatePaymentResponse>>;
    /**
     * Pay for an order with Freebees
     * @param {number}  userId          The id of the user
     * @param {number}  userCardId      The id of the userCardId
     * @param {amount}  amount          The amount that will be charged from the usercard
     * @return {Promise<IEVAServiceResponse<EVA.Core.CreatePayment, EVA.Core.CreatePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    payWithFreebees(cardNumber: string, pin?: string, amount?: number, pinType?: "pin" | "password"): Promise<IEVAServiceResponse<EVA.Core.Services.CreatePayment, EVA.Core.Services.CreatePaymentResponse>>;
    /**
     * Pay for an order with Freebees
     * @param {number}  cardNumber      The freebeees card number of the user
     * @return {Promise<IEVAServiceResponse<EVA.Kijkshop.FreeBeesGetCardBalance, EVA.Kijkshop.FreeBeesGetCardBalanceResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    getFreebeesBalance(cardNumber: string): Promise<IEVAServiceResponse<EVA.Kijkshop.FreeBeesGetCardBalance, EVA.Kijkshop.FreeBeesGetCardBalanceResponse>>;
    /**
     * Get the balance of the usercard
     *
     * @param {number} cardId
     * @param {string} currencyId
     * @return {Promise<IEVAServiceResponse<EVA.Payment.UserCard.GetUserCardBalance, EVA.Payment.UserCard.GetUserCardBalanceResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    getUserCardBalance(cardId: number, currencyId?: string): Promise<IEVAServiceResponse<EVA.Payment.UserCard.GetUserCardBalance, EVA.Payment.UserCard.GetUserCardBalanceResponse>>;
    /**
     * Get the balance of a gifcard
     *
     * @param {string} cardId
     * @param {string} cardPin
     * @param {string} barcode
     * @param {string} cardType
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.CardBalanceCheck, EVA.EVA.Core.Services.CardBalanceCheckResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    cardBalanceCheck(cardId: string, cardPin?: string, barcode?: string, cardType?: string): Promise<IEVAServiceResponse<EVA.Core.Services.CardBalanceCheck, EVA.Core.Services.CardBalanceCheckResponse>>;
    cancelPendingUserCardPayment(paymentTransactionID: number): Promise<IEVAServiceResponse<EVA.Payment.UserCard.CancelPendingUserCardPayment, EVA.Core.EmptyResponseMessage>>;
    /**
     * Pay for an order with AfterPay
     * @param  {any}  properties     Optional additional properties to send
     * @return {Promise<IEVAServiceResponse<EVA.Core.CreatePayment, EVA.Core.CreatePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    payWithAfterPay(properties?: any): Promise<IEVAServiceResponse<EVA.Core.Services.CreatePayment, EVA.Core.Services.CreatePaymentResponse>>;
    /**
     * Finalize a payment (usercard needs this)
     * @param {string}  paymentTransactionID    The payment transaction that is to be finalized
     * @return {Promise<IEVAServiceResponse<EVA.Core.FinalizePayment, EVA.Core.FinalizePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    finalizePayment(paymentTransactionID: number, amount: number): Promise<IEVAServiceResponse<EVA.Core.Services.FinalizePayment, EVA.Core.Services.FinalizePaymentResponse>>;
    /**
     * Pay for an order with a manual payment
     * @return {Promise<IEVAServiceResponse<EVA.Core.CreatePayment, EVA.Core.CreatePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    payWithManualPayment(): Promise<IEVAServiceResponse<EVA.Core.Services.CreatePayment, EVA.Core.Services.CreatePaymentResponse>>;
    /**
     * Pay for the order using a PIN terminal (shop/employee only)
     * @param  {string}  stationId     The URL to return to when the payment has completed
     * @return {Promise<IEVAServiceResponse<EVA.Core.CreatePayment, EVA.Core.CreatePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    payWithPIN(stationId: string, amountGiven: number): Promise<IEVAServiceResponse<EVA.Pin.StartTransaction, EVA.Pin.StartTransactionResponse>>;
    /**
     * Abort a payment for the order using a PIN terminal (shop/employee only)
     * @param  {string}  stationId     The URL to return to when the payment has completed
     * @return {Promise<IEVAServiceResponse<EVA.Pin.CreatePinPayment, EVA.Pin.CreatePinPaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    abortPIN(stationId: string): Promise<IEVAServiceResponse<EVA.Pin.AbortTransaction, EVA.Pin.AbortTransactionResponse>>;
    /**
     * Pay for the order using a Cash drawer (shop/employee only)
     * @parm   {Order}   order         The order that is to be paid
     * @param  {string}  returnUrl     The URL to return to when the payment has completed
     * @return {Promise<IEVAServiceResponse<EVA.Payment.Cash.MakeCashPayment, EVA.Payment.Cash.MakeCashPaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    payWithCash(stationId: string, amountGiven: number): Promise<IEVAServiceResponse<EVA.Core.Services.CreatePayment, EVA.Core.Services.CreatePaymentResponse>>;
    /**
     * Pay for an order with a Media Markt method
     * @return {Promise<IEVAServiceResponse<EVA.Core.CreatePayment, EVA.Core.CreatePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    payWithMediaMarktPayment(): Promise<IEVAServiceResponse<EVA.Core.Services.CreatePayment, EVA.Core.Services.CreatePaymentResponse>>;
    /**
     * Pay for the order using Intersolve giftcards
     * @param  {string}  cardNumber     The number on the Intersolve giftcard
     * @param  {string}  [pin]          A giftcard optionally has a PIN, provide when on the card to use the card
     * @return {Promise<IEVAServiceResponse<EVA.Core.CreatePayment, EVA.Core.CreatePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    payWithIntersolve(returnUrl: string, cardNumber: string, pin?: string, amountGiven?: number): Promise<IEVAServiceResponse<EVA.Core.Services.CreatePayment, EVA.Core.Services.CreatePaymentResponse>>;
    /**
     * Create a payment for a certain paymentmethod. The available paymentmethods for this specific order can be retrieved using the getAvailableMethods()
     * @param {string}  paymentMethodCode   The code of the payment method to be used to pay for the order, valid values need to be retrieved using getAvailableMethods()
     * @param {any}     properties          Payment method specific properties
     * @param {number}  amount              Optional parameter to specify a specific amount for this payment. If not provided the EVA Core service will default to the open amount of the order
     */
    private createPayment(paymentMethodCode, properties, amount?);
    /**
     * Create a Acapture payment
     * @param {any}      properties         Payment method specific properties
     * @param {number}   amount             Optional parameter to specify a specific amount for this payment. If not provided the EVA Core service will default to the open amount of the order
     */
    private createAcapturePayment(properties);
    private createMultisafepayPayment(properties, amount?);
    /**
     * Start a PIN payment transaction
     * @param  {string}     stationId       The station identifier provided the PIN terminal
     * @param  {number}     transactionId   The transaction identifier for the PIN payment
     * @param  {boolean}    forceStart      Force starts the transaction. Defaults to true
     * @param  {number}     timeout         Change the default timeout of the XHR call, default is 30 seconds
     * @return {Promise<IEVAServiceResponse<EVA.Pin.StartTransaction, EVA.Pin.StartTransactionResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    private startTransaction(stationId, transactionId, forceStart?, timeout?);
}
