/**
 * The payments module provides the means to pay for an order using an online payment provider
 * @preferred
 */
import { EVAService, IEVAServiceResponse    } from "../service";
import { ShoppingCart                       } from "./shopping-cart";
import { Order                              } from "./orders";

import { XHR, EXMLHttpRequestMethod         } from "../xhr";
import * as _                                 from "lodash";

/**
 * Provides access to order payment methods available in EVA
 */
export class Payments
{
    /**
     * The order to perform the payments for
     * @type {Order}
     */
    private order: Order|ShoppingCart;

    /**
     * [constructor description]
     * @param  {Order}  order The order to perform the payments for
     * @return {void}         This method doesn't return anything
     */
    constructor( order: Order|ShoppingCart )
    {
        this.order = order;
    }

    /**
     * Returns the list of available payment methods
     * @return {Promise<IEVAServiceResponse<EVA.Core.GetAvailablePaymentMethods, EVA.Core.GetAvailablePaymentMethodsResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public getAvailableMethods(): Promise<IEVAServiceResponse<EVA.Core.Services.GetAvailablePaymentMethods, EVA.Core.Services.GetAvailablePaymentMethodsResponse>>
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Core.Services.GetAvailablePaymentMethods, EVA.Core.Services.GetAvailablePaymentMethodsResponse>( "GetAvailablePaymentMethods" );

        // Build the request data
        // The base class will add things like SessionID so to keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: any = {
            OrderID:         this.order.getId()
        };

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData );
    }

    /**
     * Retuns the list of banks available for iDeal payments
     * @return {Promise<IEVAServiceResponse<EVA.Payment.Acapture.ListAcaptureIdealBanks, EVA.Payment.Acapture.ListAcaptureIdealBanksResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public getIdealBanks(): Promise<IEVAServiceResponse<EVA.Payment.Acapture.ListAcaptureIdealBanks, EVA.Payment.Acapture.ListAcaptureIdealBanksResponse>>
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Payment.Acapture.ListAcaptureIdealBanks, EVA.Payment.Acapture.ListAcaptureIdealBanksResponse>( "ListAcaptureIdealBanks" );

        // Build the request data
        // The base class will add things like SessionID so to keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: any = {};

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData );
    }

    public getMultiSafePaymentGateWays( orderId: string|number): Promise<IEVAServiceResponse<EVA.Payment.MultiSafepay.GetMultiSafepayGateways, EVA.Payment.MultiSafepay.GetMultiSafepayGatewaysResponse>>
    {
        if ( orderId === undefined )
        {
            orderId = this.order.getId();
        }

        var service = new EVAService<EVA.Payment.MultiSafepay.GetMultiSafepayGateways, EVA.Payment.MultiSafepay.GetMultiSafepayGatewaysResponse>( "GetMultiSafepayGateways" );

        var requestData: any = {
            OrderID: orderId
        };

        return service.call( requestData );
    }

    /**
     * Retuns the list of user cards available
     * @return {Promise<IEVAServiceResponse<EVA.Payment.UserCard.ListUserCardsForUser, EVA.Payment.UserCard.ListUserCardsForUserResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public listUserCards( userID: number ): Promise<IEVAServiceResponse<EVA.Payment.UserCard.ListUserCardsForUser, EVA.Payment.UserCard.ListUserCardsForUserResponse>>
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Payment.UserCard.ListUserCardsForUser, EVA.Payment.UserCard.ListUserCardsForUserResponse>( "ListUserCardsForUser" );

        // Build the request data
        // The base class will add things like SessionID so to keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: any = {
            UserID:         userID
        };

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData );
    }

    /**
     * Pay for the order using a PIN terminal from ADYEN (shop/employee only)
     * @param  {string}  merchantCode     The ADYEN merchant code to use
     * @param  {number}  amount           The amount to pay
     * @return {Promise<IEVAServiceResponse<EVA.Core.CreatePayment, EVA.Core.CreatePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    public payWithAdyen( merchantCode: string, amount: number ): Promise<IEVAServiceResponse<EVA.Core.Services.CreatePayment, EVA.Core.Services.CreatePaymentResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Perform the call and capture the resulting promise
            //
            this.createPayment( "ADYEN", { "MerchantCode": merchantCode }, amount )
            .then( ( result ) =>
            {
                resolve( result.response );
            } )
            .catch( reject );
        } );
    }

    public payWithMultiSafePay( method: string, returnUrl: string, amount?: number ) {
        let payment: any;

        payment = this.createMultisafepayPayment(
        {
            GatewayID:      method,
            ReturnUrl:      returnUrl
        }, amount );

        return payment;
    }

    /**
     * Pay for the order using iDEAL
     * @param  {string}  returnUrl     The URL to return to when the payment has completed
     * @param  {string}  idealBankName The name of the bank for the ideal payment
     * @return {Promise<IEVAServiceResponse<EVA.Core.CreatePayment, EVA.Core.CreatePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    public payWithIdeal( returnUrl: string, idealBankName: string ): Promise<IEVAServiceResponse<EVA.Core.Services.CreatePayment, EVA.Core.Services.CreatePaymentResponse>>
    {
        let payment: any;

        payment = this.createAcapturePayment (
        {
            PaymentBrand:   "IDEAL",
            BankAccount:
            {
                BankName:   idealBankName,
                Country:     "NL"
            },
            ReturnUrl:      returnUrl
        });

        return payment;
    }

    /**
     * Pay for the order using PayPal
     * @param  {string}  returnUrl     The URL to return to when the payment has completed
     * @return {Promise<IEVAServiceResponse<EVA.Core.CreatePayment, EVA.Core.CreatePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    public payWithPaypal( returnUrl: string ): Promise<IEVAServiceResponse<EVA.Core.Services.CreatePayment, EVA.Core.Services.CreatePaymentResponse>>
    {
        return this.createAcapturePayment(
            {
                PaymentBrand:   "PAYPAL",
                ReturnUrl:      returnUrl
            }
        );
    }

    /**
     * Pay for the order using a MasterCard/Eurocard credit card
     * @param  {string}  returnUrl     The URL to return to when the payment has completed
     * @return {Promise<IEVAServiceResponse<EVA.Core.CreatePayment, EVA.Core.CreatePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    public payWithMasterCard( returnUrl: string ): Promise<IEVAServiceResponse<EVA.Core.Services.CreatePayment, EVA.Core.Services.CreatePaymentResponse>>
    {
        return this.createAcapturePayment(
            {
                PaymentBrand:   "MASTER",
                ReturnUrl:      returnUrl
            }
        );
    }

    /**
     * Pay for the order using an American Express credit card
     * @param  {string}  returnUrl     The URL to return to when the payment has completed
     * @return {Promise<IEVAServiceResponse<EVA.Core.CreatePayment, EVA.Core.CreatePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    public payWithAmericanExpress( returnUrl: string ): Promise<IEVAServiceResponse<EVA.Core.Services.CreatePayment, EVA.Core.Services.CreatePaymentResponse>>
    {
        return this.createAcapturePayment(
            {
                PaymentBrand:   "AMEX",
                ReturnUrl:      returnUrl
            }
        );
    }

    /**
     * Pay for the order using a Visa credit card
     * @param  {string}  returnUrl     The URL to return to when the payment has completed
     * @return {Promise<IEVAServiceResponse<EVA.Core.CreatePayment, EVA.Core.CreatePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    public payWithVisa( returnUrl: string ): Promise<IEVAServiceResponse<EVA.Core.Services.CreatePayment, EVA.Core.Services.CreatePaymentResponse>>
    {
        return this.createAcapturePayment(
            {
                PaymentBrand:   "VISA",
                ReturnUrl:      returnUrl
            }
        );
    }

    /**
     * Pay for an order with an insurance card (HFS Waardebrief)
     * @param {string}  referenceNumber The reference number for the credit payment
     * @param {string}  policyNumber    The policy number for the credit payment
     * @return {Promise<IEVAServiceResponse<EVA.Core.CreatePayment, EVA.Core.CreatePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    public payWithInsuranceCreditPayment( referenceNumber: string, policyNumber: string )
    {
        return this.createPayment(
            "HARMONY-INSURANCE",
            {
                ReferenceNumber:        referenceNumber,
                PolicyNumber:           policyNumber
            }
        );
    }

    /**
     * Pay for an order with a user card
     * @param {number}  userId          The id of the user
     * @param {number}  userCardId      The id of the userCardId
     * @param {amount}  amount          The amount that will be charged from the usercard
     * @return {Promise<IEVAServiceResponse<EVA.Core.CreatePayment, EVA.Core.CreatePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    public payWithUserCard( userId: number, userCardId: number, amount: number )
    {
        return this.createPayment(
            "USERCARD",
            {
                UserID: userId,
                UserCardID : userCardId,
                CardAmount: amount
            }
        );
    }

    /**
     * Pay for an order with Freebees
     * @param {number}  userId          The id of the user
     * @param {number}  userCardId      The id of the userCardId
     * @param {amount}  amount          The amount that will be charged from the usercard
     * @return {Promise<IEVAServiceResponse<EVA.Core.CreatePayment, EVA.Core.CreatePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    public payWithFreebees( cardNumber: string, pin?: string, amount?: number, pinType: "pin"|"password" = "pin" )
    {
        const properties: any = {
            CardNumber: cardNumber,
        };

        if ( pinType === "pin" ) {
            properties.Pin = pin;
        } else if ( pinType === "password" ) {
            properties.Password = pin;
        }

        return this.createPayment( "FREEBEES", properties, amount );
    }

    /**
     * Pay for an order with Freebees
     * @param {number}  cardNumber      The freebeees card number of the user
     * @return {Promise<IEVAServiceResponse<EVA.Kijkshop.FreeBeesGetCardBalance, EVA.Kijkshop.FreeBeesGetCardBalanceResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    public getFreebeesBalance( cardNumber: string )
    {
        var service = new EVAService<EVA.Kijkshop.FreeBeesGetCardBalance, EVA.Kijkshop.FreeBeesGetCardBalanceResponse>( "FreeBeesGetCardBalance" );

        // Build the service request descriptor
        //
        var requestData: EVA.Kijkshop.FreeBeesGetCardBalance = {
            CardNumber: cardNumber,
            __response: undefined
        };

        return service.call( requestData );
    }

    /**
     * Get the balance of the usercard
     *
     * @param {number} cardId
     * @param {string} currencyId
     * @return {Promise<IEVAServiceResponse<EVA.Payment.UserCard.GetUserCardBalance, EVA.Payment.UserCard.GetUserCardBalanceResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    public getUserCardBalance( cardId: number, currencyId?: string )
    {
        var service = new EVAService<EVA.Payment.UserCard.GetUserCardBalance, EVA.Payment.UserCard.GetUserCardBalanceResponse>( "GetCardBalance" );
        // Build the service request descriptor
        //
        var requestData: EVA.Payment.UserCard.GetUserCardBalance = {
            UserCardID: cardId,
            CurrencyID: currencyId,
            __response: undefined
        };

        return service.call( requestData );
    }

    /**
     * Get the balance of a gifcard
     *
     * @param {string} cardId
     * @param {string} cardPin
     * @param {string} barcode
     * @param {string} cardType
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.CardBalanceCheck, EVA.EVA.Core.Services.CardBalanceCheckResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    public cardBalanceCheck( cardId: string, cardPin?: string, barcode?: string, cardType = "INTERSOLVE" )
    {
        var service = new EVAService<EVA.Core.Services.CardBalanceCheck, EVA.Core.Services.CardBalanceCheckResponse>( "CardBalanceCheck" );

        // Build the service request descriptor
        //
        var request: EVA.Core.Services.CardBalanceCheck = {
            CardNumber: cardId,
            CardPin: cardPin,
            CardType: cardType,
            Barcode: barcode,
            __response: undefined
        };

        return service.call( request );
    }

    public cancelPendingUserCardPayment( paymentTransactionID: number )
    {
        var service = new EVAService<EVA.Payment.UserCard.CancelPendingUserCardPayment, EVA.Core.EmptyResponseMessage>( "CancelPendingUserCardPayment" );

        var requestData: EVA.Payment.UserCard.CancelPendingUserCardPayment = {
            PaymentTransactionID: paymentTransactionID,
            __response:           null
        };

        return service.call( requestData );
    }

    /**
     * Pay for an order with AfterPay
     * @param  {any}  properties     Optional additional properties to send
     * @return {Promise<IEVAServiceResponse<EVA.Core.CreatePayment, EVA.Core.CreatePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    public payWithAfterPay( properties?: any )
    {
        return this.createPayment( "AFTERPAY", properties );
    }

    /**
     * Finalize a payment (usercard needs this)
     * @param {string}  paymentTransactionID    The payment transaction that is to be finalized
     * @return {Promise<IEVAServiceResponse<EVA.Core.FinalizePayment, EVA.Core.FinalizePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    public finalizePayment( paymentTransactionID: number, amount: number ): Promise<IEVAServiceResponse<EVA.Core.Services.FinalizePayment, EVA.Core.Services.FinalizePaymentResponse>>
    {
        var service = new EVAService<EVA.Core.Services.FinalizePayment, EVA.Core.Services.FinalizePaymentResponse>( "FinalizePayment" );

        var requestData: any = {
            PaymentTransactionID: paymentTransactionID,
            Amount:               amount
        };

        return service.call( requestData );
    }

    /**
     * Pay for an order with a manual payment
     * @return {Promise<IEVAServiceResponse<EVA.Core.CreatePayment, EVA.Core.CreatePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    public payWithManualPayment()
    {
        return this.createPayment(
            "MANUAL",
            {
            }
        );
    }

    /**
     * Pay for the order using a PIN terminal (shop/employee only)
     * @param  {string}  stationId     The URL to return to when the payment has completed
     * @return {Promise<IEVAServiceResponse<EVA.Core.CreatePayment, EVA.Core.CreatePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    public payWithPIN( stationId: string, amountGiven: number ): Promise<IEVAServiceResponse<EVA.Pin.StartTransaction, EVA.Pin.StartTransactionResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Perform the call and capture the resulting promise
            //
            this.createPayment(
                "PIN", {},
                // This is the amount we want to PIN
                //
                Math.max( 0, amountGiven ) )
            .then( ( result ) =>
            {
                var transaction = result.response.PaymentTransaction;
                // Normal transaction take 30 seconds. For PIN we have to wait for the customer to perform it's action
                // So we increase the wait time to three minutes (180000 milliseconds)
                //
                this.startTransaction( stationId, transaction.ID, true, 180000 ).then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }

    /**
     * Abort a payment for the order using a PIN terminal (shop/employee only)
     * @param  {string}  stationId     The URL to return to when the payment has completed
     * @return {Promise<IEVAServiceResponse<EVA.Pin.CreatePinPayment, EVA.Pin.CreatePinPaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    public abortPIN( stationId: string ): Promise<IEVAServiceResponse<EVA.Pin.AbortTransaction, EVA.Pin.AbortTransactionResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create our service instance
            //
            var service = new EVAService<EVA.Pin.AbortTransaction, EVA.Pin.AbortTransactionResponse>( "AbortTransaction" );

            // Build the request data
            // The base class will add things like SessionID so to keep typescript
            // happy we're using an 'any' type object here
            // We need to explicitly set UseHardwareID to false because it defaults to true
            //
            var requestData: any = {
                UseHardwareID:         false,
                StationID:             stationId
            };

            // Perform the call and capture the resulting promise
            //
            return service.call( requestData );
        } );
    }

    /**
     * Pay for the order using a Cash drawer (shop/employee only)
     * @parm   {Order}   order         The order that is to be paid
     * @param  {string}  returnUrl     The URL to return to when the payment has completed
     * @return {Promise<IEVAServiceResponse<EVA.Payment.Cash.MakeCashPayment, EVA.Payment.Cash.MakeCashPaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    public payWithCash( stationId: string, amountGiven: number ): Promise<IEVAServiceResponse<EVA.Core.Services.CreatePayment, EVA.Core.Services.CreatePaymentResponse>>
    {
        return this.createPayment(
            "CASH",
            {
                AmountGiven:        Math.max( 0, amountGiven ),
                StationID:          stationId
            }
        );
    }

    /**
     * Pay for an order with a Media Markt method
     * @return {Promise<IEVAServiceResponse<EVA.Core.CreatePayment, EVA.Core.CreatePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    public payWithMediaMarktPayment()
    {
        return this.createPayment(
            "MEDIAMARKT",
            {
            }
        );
    }

    /**
     * Pay for the order using Intersolve giftcards
     * @param  {string}  cardNumber     The number on the Intersolve giftcard
     * @param  {string}  [pin]          A giftcard optionally has a PIN, provide when on the card to use the card
     * @return {Promise<IEVAServiceResponse<EVA.Core.CreatePayment, EVA.Core.CreatePaymentResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    public payWithIntersolve( returnUrl: string, cardNumber: string, pin?: string, amountGiven?: number ): Promise<IEVAServiceResponse<EVA.Core.Services.CreatePayment, EVA.Core.Services.CreatePaymentResponse>>
    {
        return this.createPayment(
            "INTERSOLVE",
            {
                CardNumber:     cardNumber,
                Pin:            pin,
                ReturnUrl:      returnUrl
            },
            amountGiven
        );
    }

    /**
     * Create a payment for a certain paymentmethod. The available paymentmethods for this specific order can be retrieved using the getAvailableMethods()
     * @param {string}  paymentMethodCode   The code of the payment method to be used to pay for the order, valid values need to be retrieved using getAvailableMethods()
     * @param {any}     properties          Payment method specific properties
     * @param {number}  amount              Optional parameter to specify a specific amount for this payment. If not provided the EVA Core service will default to the open amount of the order
     */
    private createPayment( paymentMethodCode: string, properties: any, amount?: number ): Promise<IEVAServiceResponse<EVA.Core.Services.CreatePayment, EVA.Core.Services.CreatePaymentResponse>>
    {
        var service = new EVAService<EVA.Core.Services.CreatePayment, EVA.Core.Services.CreatePaymentResponse>( "CreatePayment" );

        var requestData: any = {
            Amount:         amount,
            Code:           paymentMethodCode,
            OrderID:        this.order.getId(),
            Properties:     properties
        };

        return service.call( requestData );
    }

    /**
     * Create a Acapture payment
     * @param {any}      properties         Payment method specific properties
     * @param {number}   amount             Optional parameter to specify a specific amount for this payment. If not provided the EVA Core service will default to the open amount of the order
     */
    private createAcapturePayment( properties: any ): Promise<IEVAServiceResponse<EVA.Core.Services.CreatePayment, EVA.Core.Services.CreatePaymentResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            this.createPayment(
                "ACAPTURE",
                properties
            )
            .then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.CreatePayment, EVA.Core.Services.CreatePaymentResponse> ) =>
            {
                let success = _.get( serviceResult, "response.Properties.Success" );

                if ( success )
                {
                    resolve( serviceResult );
                }
                else
                {
                    reject( serviceResult );
                }
            } )
            .catch( reject );
        } );
    }

    private createMultisafepayPayment( properties: any, amount?: number ): Promise<IEVAServiceResponse<EVA.Core.Services.CreatePayment, EVA.Core.Services.CreatePaymentResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            this.createPayment(
                "MULTISAFEPAY",
                properties,
                amount
            )
            .then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.CreatePayment, EVA.Core.Services.CreatePaymentResponse> ) =>
            {
                let success = _.get( serviceResult, "response.Properties.Success" );

                if ( success )
                {
                    resolve( serviceResult );
                }
                else
                {
                    reject( serviceResult );
                }
            } )
            .catch( reject );
        } );
    }

    /**
     * Start a PIN payment transaction
     * @param  {string}     stationId       The station identifier provided the PIN terminal
     * @param  {number}     transactionId   The transaction identifier for the PIN payment
     * @param  {boolean}    forceStart      Force starts the transaction. Defaults to true
     * @param  {number}     timeout         Change the default timeout of the XHR call, default is 30 seconds
     * @return {Promise<IEVAServiceResponse<EVA.Pin.StartTransaction, EVA.Pin.StartTransactionResponse>>}   A promise resolving when the fetch from the back-end has completed
     */
    private startTransaction( stationId: string, transactionId: number, forceStart: boolean = true, timeout = 30000 ): Promise<IEVAServiceResponse<EVA.Pin.StartTransaction, EVA.Pin.StartTransactionResponse>>
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Pin.StartTransaction, EVA.Pin.StartTransactionResponse>( "StartTransaction" );

        // Build the request data
        // The base class will add things like SessionID so to keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: any = {
            PaymentTransactionID:  transactionId,
            ForceStart:            forceStart,
            UseHardwareID:         false,
            StationID:             stationId
        };

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData, true, timeout )
            .then(response => {
                switch (response.response.ResultStatus)
                {
                    // Only code that indicates that the transaction was successful
                    case 2: // Success
                        return response;

                    case 17: // ConnectToTerminal
                        var xhr     = new XHR();
                        xhr.call( {
                            method:         EXMLHttpRequestMethod.GET,
                            // This may potentially be running in a browser. Needs to use https:
                            // to fool the browser that it is safe to connect to the given url
                            //
                            url:            "https://" + response.response.LocalTerminalAddress,
                            headers:        {}
                        } )
                        .catch(() => {
                            // Ignore whatever happens, we just want to attempt to connect
                            // to the terminal; that's the best we can do
                            //
                        } );

                        // Go at it again because we've done our job
                        //
                        return this.startTransaction( stationId, transactionId, forceStart );

                    default:
                        // Reject the promise because we got a none success code
                        //
                        throw response;
                }
            });
    }
}
