import { FetchableItem, FetchablePagedList } from "./base";
import { IEVAServiceResponse } from "../service";
export declare namespace products {
    function getProductPrices(productIds: number[]): Promise<IEVAServiceResponse<EVA.Core.Services.GetProductPrices, EVA.Core.Services.GetProductPricesResponse>>;
    function getConfigurableProductsDetails(productIds: number[], includes?: string[]): Promise<IEVAServiceResponse<EVA.Core.Services.GetConfigurableProductsDetail, EVA.Core.Services.GetConfigurableProductsDetailResponse>>;
    function getProductsDetails(productIds: number[], pageConfig?: EVA.Core.PageConfig, additionalFilters?: any): Promise<IEVAServiceResponse<EVA.Core.Services.SearchProducts, EVA.Core.Services.SearchProductsResponse>>;
}
/**
 * Provides access to a product available in EVA
 */
export declare class Product extends FetchableItem<EVA.Core.ProductDto, EVA.Core.Services.GetProductDetail, EVA.Core.Services.GetProductDetailResponse> {
    /**
     * Fetches the product details
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.Products.GetProductDetail, EVA.Core.Services.Products.GetProductDetailResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    fetch(): Promise<IEVAServiceResponse<EVA.Core.Services.GetProductDetail, EVA.Core.Services.GetProductDetailResponse>>;
    /**
     * Fetches the product configuration details
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetConfigurableProductDetail, EVA.Core.Services.GetConfigurableProductDetailResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    getConfigurationDetails(): Promise<IEVAServiceResponse<EVA.Core.Services.GetConfigurableProductDetail, EVA.Core.Services.GetConfigurableProductDetailResponse>>;
    /**
     * Retrieves the stock details for the product
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetStockDetailsForProduct, EVA.Core.Services.GetStockDetailsForProductResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    getStockDetails(): Promise<IEVAServiceResponse<EVA.Core.Services.GetStockDetailsForProduct, EVA.Core.Services.GetStockDetailsForProductResponse>>;
    /**
     * Retrieves the availability indication for the product
     * @param  {number}         quantityRequested           Quantity to check availability for. Defaults to 1
     * @param  {Array<number>}  organizationUnitIDs         An array of organisation id's to limit the availability check to
     * @param  {boolean}        fastestPickupShopOnly       Only return the fastest pickup option. Defaults to false
     * @param  {boolean}        includeAvailabilityTexts    Add availability text messages to the response. Defaults to false
     * @param  {boolean}        detailedShopInformation     Add detailed shop information to the response. Defaults to false
     * @return {Promise<Array<EVA.Core.ProductAvailabilityIndicationDto>>} A promise resolving when the fetch from the back-end has completed
     */
    getAvailability(quantityRequested?: number, organizationUnitIDs?: Array<number>, fastestPickupShopOnly?: boolean, includeAvailabilityTexts?: boolean, detailedShopInformation?: boolean): Promise<Array<EVA.Core.ProductAvailabilityIndication>>;
    /**
     * Retrieves the available variants (color, size, etc.) for a product
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.SearchProducts, EVA.Core.Services.SearchProductsResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    getVariants(limit?: number, start?: number, sortDirection?: EVA.Core.SortDirection, sortProperty?: string, additionalFilters?: any): Promise<IEVAServiceResponse<EVA.Core.Services.SearchProducts, EVA.Core.Services.SearchProductsResponse>>;
}
/**
 * Provides access to a fashion product available in EVA
 */
export declare class FashionProduct extends Product {
    /**
     * Convenience method to build a product image URL
     * @param  {number} productID   The product identifier
     * @param  {number} width       The output image width
     * @param  {number} height      The output image height
     * @param  {string} type        The image type (ie. png, jpg, etc)
     *
     * @return {string}             An image URL
     */
    static getProductImageUrl(productID: number, width?: number, height?: number, format?: string): string;
    create(product: EVA.Babylon.CreateFashionProduct): Promise<IEVAServiceResponse<EVA.Core.Services.GetProductDetail, EVA.Core.Services.GetProductDetailResponse>>;
    update(productDetailsToUpdate: EVA.Babylon.UpdateFashionProduct): Promise<IEVAServiceResponse<EVA.Core.Services.GetProductDetail, EVA.Core.Services.GetProductDetailResponse>>;
    delete(): Promise<IEVAServiceResponse<EVA.Babylon.DeleteFashionProduct, EVA.Core.EmptyResponseMessage>>;
    getCoverSheetImages(): Promise<IEVAServiceResponse<EVA.Babylon.GetProductImagesForCoverSheet, EVA.Babylon.GetProductImagesForCoverSheetResponse>>;
    getCreator(): Promise<IEVAServiceResponse<EVA.Babylon.GetFashionProductCreator, EVA.Babylon.GetFashionProductCreatorResponse>>;
    countCollections(): Promise<IEVAServiceResponse<EVA.Babylon.CountFashionCollectionsForProduct, EVA.Babylon.CountFashionCollectionsForProductResponse>>;
}
/**
 * Provides access to a list of products available in EVA
 */
export declare class ProductList extends FetchablePagedList<EVA.Core.ProductDto, EVA.Core.Services.SearchProducts, EVA.Core.Services.SearchProductsResponse> {
    /**
     * The return aggregations flag
     * @type {any}
     */
    includeAggregations: boolean;
    /**
     * The return suggestions flag
     * @type {any}
     */
    includeSuggestions: boolean;
    /**
     * The query string to search for in the products catalog
     * @type {string}
     */
    private query;
    /**
     * The list of fields to exclude from the products in the list response
     * @type {Array<string>}
     */
    private excludedFields;
    /**
     * The list of fields to include in the products in the list response
     * @type {Array<string>}
     */
    private includedFields;
    /**
     * The options for the list of fields to exclude from the products in the list response
     * @type {Array<string>}
     */
    private excludedFieldsOption;
    /**
     * The collection of aggregation filters to search for in the products catalog
     * @type {any}
     */
    private aggregationFilters;
    /**
     * The collection of aggregation options to apply to the search in the products catalog
     * @type {any}
     */
    private aggregationOptions;
    /**
     * The aggregation state
     * @type {string}
     */
    private aggregationState;
    /**
     * The returned aggregations
     * @type {any}
     */
    private aggregations;
    /**
     * Returns the current list of excluded fields
     * @return {Array<string>} The configured list of excluded product fields
     */
    getExcludedFields(): Array<string>;
    /**
     * Returns the current list of excluded fields
     * @return {Array<string>} The configured list of excluded product fields
     */
    addExcludedField(fieldName: string): Array<string>;
    /**
     * Clears the configured list of excluded fields
     * @return {Array<string>} The configured list of excluded product fields
     */
    clearExcludedFields(): Array<string>;
    /**
     * Sets the excluded fields option
     * @return {EVA.Core.ExcludedFieldOptions} The aggregation state
     */
    setExcludedFieldsOptions(option: EVA.Core.ExcludedFieldOptions): any;
    /**
     * Return the currently set excluded field option
     * @return {EVA.Core.ExcludedFieldOptions} The excluded field option value
     */
    getExcludedFieldsOption(): EVA.Core.ExcludedFieldOptions;
    /**
     * Returns the current list of included fields
     * @return {Array<string>} The configured list of included product fields
     */
    getIncludedFields(): Array<string>;
    /**
     * Returns the current list of included fields
     * @return {Array<string>} The configured list of included product fields
     */
    addIncludedField(fieldName: string): Array<string>;
    /**
     * Clears the configured list of included fields
     * @return {Array<string>} The configured list of included product fields
     */
    clearIncludedFields(): Array<string>;
    /**
     * Sets the aggregation state
     * @return {any} The aggregation state
     */
    setAggregationState(aggregationState: string): any;
    /**
     * Return the currently set aggregation state
     * @return {string} The aggregation state
     */
    getAggregationState(): string;
    /**
     * Sets the aggregation options
     * @return {any} The aggregation options
     */
    setAggregationOptions(aggregationOptions: any): any;
    /**
     * Return the currently set aggregation options
     * @return {any} The aggregation options
     */
    getAggregationOptions(): any;
    /**
     * Returns the current query string
     * @return {string}       The configured query string
     */
    getQuery(): string;
    /**
     * Set the query to search for
     * @param  {string} query The new query string
     * @return {string}       The configured query string
     */
    setQuery(query: string): string;
    /**
     * Return the aggregations that were returned with the product list
     * @return {any} The aggreations
     */
    getAggregations(): any;
    /**
     * Sets the current item details using the fetched service response.
     * Extending classes should override this method to pull the details from response
     * @param  {EVA.Core.Services.Products.SearchProductsResponse}  response    The response from the service call
     * @return {Array<EVA.Core.ProductDto>}                      The product items for the current page
     */
    setData(response: any): EVA.Core.ProductDto[];
    /**
     * Fetches the product list item details
     * @return {Promise<IEVAServiceResponse<EVA.Core.ProductDto, EVA.Core.Services.SearchProducts, EVA.Core.Services.SearchProductsResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    fetch(): Promise<IEVAServiceResponse<EVA.Core.Services.SearchProducts, EVA.Core.Services.SearchProductsResponse>>;
}
/**
 * Provides access to a list of recently viewed (fashion) products available in EVA
 */
export declare class RecentlyViewedProductList extends FetchablePagedList<EVA.Babylon.RecentlyViewedProduct, EVA.Babylon.ListRecentlyViewedProducts, EVA.Babylon.ListRecentlyViewedProductsResponse> {
    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Babylon.ListRecentlyViewedProducts, EVA.Babylon.ListRecentlyViewedProductsResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    fetch(): Promise<IEVAServiceResponse<EVA.Babylon.ListRecentlyViewedProducts, EVA.Babylon.ListRecentlyViewedProductsResponse>>;
}
export declare function getFashionProductCategories(): Promise<IEVAServiceResponse<EVA.Babylon.GetFashionProductCategories, EVA.Babylon.GetFashionProductCategoriesResponse>>;
