/**
 * The product module provides access to the product data from the EVA back-end
 * @preferred
 */

import { SettingsManager                   } from "../settings";
import { FetchableItem, FetchablePagedList } from "./base";
import { EVAService, IEVAServiceResponse   } from "../service";
import { EXMLHttpRequestMethod             } from "../xhr";
import * as _                                from "lodash";

export namespace products {
    "use strict";

    export function getProductPrices( productIds: number[] ) {
        // Create the service instance
        //
        var service = new EVAService<EVA.Core.Services.GetProductPrices, EVA.Core.Services.GetProductPricesResponse>( "GetProductPrices" );

        // Build the request data
        // The base class will add things like SessionID so keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: EVA.Core.Services.GetProductPrices  = <EVA.Core.Services.GetProductPrices> {
            ProductIDs: productIds
        };

        // Perform the call and return resulting promise
        //
        return service.call( requestData );
    }

    export function getConfigurableProductsDetails( productIds: number[], includes?: string[] ) {
        // Create the service instance
        //
        var service = new EVAService<EVA.Core.Services.GetConfigurableProductsDetail, EVA.Core.Services.GetConfigurableProductsDetailResponse>( "GetConfigurableProductsDetail" );

        // Build the request data
        // The base class will add things like SessionID so keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: EVA.Core.Services.GetConfigurableProductsDetail  = <EVA.Core.Services.GetConfigurableProductsDetail> {
            ProductIDs: productIds,
            Includes:   includes
        };

        // Perform the call and return resulting promise
        //
        return service.call( requestData );
    }

    export function getProductsDetails( productIds: number[], pageConfig?: EVA.Core.PageConfig, additionalFilters: any = {} ) {

        const productList = new ProductList();

        if ( pageConfig )
        {
          productList.setPageConfiguration( pageConfig );
        }
        else
        {
            productList.setPageConfiguration({
                Start:          0,
                Limit:          9999,
                SortDirection:  0,
                SortProperty:   "product_id",
                Filter:         {}
             } );
        }

        productList.setFilter( _.extend( additionalFilters || {}, {
            "product_id": {
                "Values": productIds
            }
        } ) );

        return productList.fetch();
    }
}

/**
 * Provides access to a product available in EVA
 */
export class Product extends FetchableItem<EVA.Core.ProductDto, EVA.Core.Services.GetProductDetail, EVA.Core.Services.GetProductDetailResponse>
{
    /**
     * Fetches the product details
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.Products.GetProductDetail, EVA.Core.Services.Products.GetProductDetailResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Core.Services.GetProductDetail, EVA.Core.Services.GetProductDetailResponse>>
    {
        if ( this.getId() === undefined )
        {
            return Promise.reject( {
                status:     404,
                response:   "Not found"
            } );
        }
        else
        {
            // Build the request data
            //
            var requestData: EVA.Core.Services.GetProductDetail = <EVA.Core.Services.GetProductDetail>{
                ID: this.getId()
            };

            return super.fetch( "GetProductDetail", requestData );
        }
    }

    /**
     * Fetches the product configuration details
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetConfigurableProductDetail, EVA.Core.Services.GetConfigurableProductDetailResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public getConfigurationDetails(): Promise<IEVAServiceResponse<EVA.Core.Services.GetConfigurableProductDetail, EVA.Core.Services.GetConfigurableProductDetailResponse>>
    {
        if ( this.getId() === undefined )
        {
            return Promise.reject( {
                status:     404,
                response:   "Not found"
            } );
        }
        else
        {
            // Create our service instance
            //
            var service = new EVAService<EVA.Core.Services.GetConfigurableProductDetail, EVA.Core.Services.GetConfigurableProductDetailResponse>( "GetConfigurableProductDetail" );

            // Build the request data
            //
            var requestData: EVA.Core.Services.GetConfigurableProductDetail = <EVA.Core.Services.GetConfigurableProductDetail>{
                ID: this.getId()
            };

            return service.call( requestData );
        }
    }

    /**
     * Retrieves the stock details for the product
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetStockDetailsForProduct, EVA.Core.Services.GetStockDetailsForProductResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public getStockDetails(): Promise<IEVAServiceResponse<EVA.Core.Services.GetStockDetailsForProduct, EVA.Core.Services.GetStockDetailsForProductResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create our service instance
            //
            var service = new EVAService<EVA.Core.Services.GetStockDetailsForProduct, EVA.Core.Services.GetStockDetailsForProductResponse>( "GetStockDetailsForProduct" );

            // Build the request data
            // The base class will add things like SessionID so to keep typescript
            // happy we're using an 'any' type object here
            //
            var requestData: any = {
                ProductID: this.getId()
            };

            // Perform the call and capture the resulting promise
            //
            service.call( requestData )
            .then( () =>
            {
                this.fetch().then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }

    /**
     * Retrieves the availability indication for the product
     * @param  {number}         quantityRequested           Quantity to check availability for. Defaults to 1
     * @param  {Array<number>}  organizationUnitIDs         An array of organisation id's to limit the availability check to
     * @param  {boolean}        fastestPickupShopOnly       Only return the fastest pickup option. Defaults to false
     * @param  {boolean}        includeAvailabilityTexts    Add availability text messages to the response. Defaults to false
     * @param  {boolean}        detailedShopInformation     Add detailed shop information to the response. Defaults to false
     * @return {Promise<Array<EVA.Core.ProductAvailabilityIndicationDto>>} A promise resolving when the fetch from the back-end has completed
     */
    public getAvailability( quantityRequested: number = 1, organizationUnitIDs?: Array<number>, fastestPickupShopOnly: boolean = false, includeAvailabilityTexts: boolean = false, detailedShopInformation: boolean = false ): Promise<Array<EVA.Core.ProductAvailabilityIndication>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create our service instance
            //
            var service = new EVAService<EVA.Core.Services.GetAvailabilityIndication, EVA.Core.Services.GetAvailabilityIndicationResponse>( "GetAvailabilityIndication" );

            // Build the request data
            // The base class will add things like SessionID so to keep typescript
            // happy we're using an 'any' type object here
            //
            var requestData: any = {
                Products: [ {
                    ProductID:            this.getId(),
                    QuantityRequested:    quantityRequested
                } ],
                FastestPickupShopOnly:    fastestPickupShopOnly,
                OrganizationUnitIDs:      organizationUnitIDs,
                DetailedShopInformation:  detailedShopInformation,
                IncludeAvailabilityTexts: includeAvailabilityTexts
            };

            // Perform the call and capture the resulting promise
            //
            service.call( requestData )
            .then( ( result ) =>
            {
                resolve( _.get( result, "response.Products" ) || [] );
            } )
            .catch( reject );
        } );
    }

    /**
     * Retrieves the available variants (color, size, etc.) for a product
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.SearchProducts, EVA.Core.Services.SearchProductsResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public getVariants( limit: number = 15, start: number = 0, sortDirection: EVA.Core.SortDirection = 0, sortProperty: string = "product_id", additionalFilters: any = {} ): Promise<IEVAServiceResponse<EVA.Core.Services.SearchProducts, EVA.Core.Services.SearchProductsResponse>>
    {
        // We will use the product search to find the child products for this product
        //
        var productVariants = new ProductList( limit, start, sortDirection, sortProperty );

        // Set this product as the parent
        //
        productVariants.setFilter( _.extend( additionalFilters || {}, {
            "parent_hierarchy": {
                "Values": [ this.getId() ]
            }
        } ) );

        // Optimize the request by limiting fieldset and
        // not adding aggregations
        //
        productVariants.includeAggregations = false;

        productVariants.addIncludedField( "product_id"              );
        productVariants.addIncludedField( "clothing_primary_color"  );
        productVariants.addIncludedField( "logical_level"           );
        productVariants.addIncludedField( "clothing_size"           );
        productVariants.addIncludedField( "display_price"           );
        productVariants.addIncludedField( "product_media"           );
        productVariants.addIncludedField( "is_sellable"             );

        return productVariants.fetch();
    }

}

/**
 * Provides access to a fashion product available in EVA
 */
export class FashionProduct extends Product
{
    /**
     * Convenience method to build a product image URL
     * @param  {number} productID   The product identifier
     * @param  {number} width       The output image width
     * @param  {number} height      The output image height
     * @param  {string} type        The image type (ie. png, jpg, etc)
     *
     * @return {string}             An image URL
     */
    public static getProductImageUrl( productID: number, width = 100, height = 100, format = "png" )
    {
        return SettingsManager.getEndPointURL() + `/productimage/${width}/${height}/${productID}.${format}`;
    }

    public create( product: EVA.Babylon.CreateFashionProduct ): Promise<IEVAServiceResponse<EVA.Core.Services.GetProductDetail, EVA.Core.Services.GetProductDetailResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create the service instance
            //
            var service = new EVAService<EVA.Babylon.CreateFashionProduct, EVA.Babylon.CreateFashionProductResponse>( "CreateFashionProduct", "/api/v1/fashion/product", EXMLHttpRequestMethod.POST );

            // Perform the call and capture the resulting promise
            //
            service.call( product )
            .then( ( createResult: IEVAServiceResponse<EVA.Babylon.CreateFashionProduct, EVA.Babylon.CreateFashionProductResponse> ) =>
            {
                // Collect the ID from the response
                //
                this.id = createResult.response.ProductID;

                // Return the fetch of the newly created collection
                //
                this.fetch().then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }

    public update( productDetailsToUpdate: EVA.Babylon.UpdateFashionProduct ): Promise<IEVAServiceResponse<EVA.Core.Services.GetProductDetail, EVA.Core.Services.GetProductDetailResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            if ( this.getId() === undefined )
            {
                return Promise.reject( {
                    status:     404,
                    response:   "Not found"
                } );
            }
            else
            {
                // Update the user profile
                //
                // var service = new EVAService<EVA.Babylon.UpdateFashionProduct, EVA.Core.EmptyResponseMessage>( "UpdateFashionProduct", "/api/v1/fashion/product/" + this.getId(), EXMLHttpRequestMethod.PATCH );
                var service = new EVAService<EVA.Babylon.UpdateFashionProduct, EVA.Core.EmptyResponseMessage>( "UpdateFashionProduct", "/api/v1/fashion/product/", EXMLHttpRequestMethod.PATCH );

                // Ensure collectionID is set
                //
                productDetailsToUpdate.ProductID = <number>this.getId();

                // Perform the call and capture the resulting promise
                //
                service.call( productDetailsToUpdate )
                .then( ( updateResult: IEVAServiceResponse<EVA.Babylon.UpdateFashionProduct, EVA.Core.EmptyResponseMessage> ) =>
                {
                    // Return the fetch of the updated user profile
                    //
                    this.fetch().then( resolve ).catch( reject );
                } )
                .catch( reject );
            }
        } );
    }

    public delete(): Promise<IEVAServiceResponse<EVA.Babylon.DeleteFashionProduct, EVA.Core.EmptyResponseMessage>>
    {
        if ( this.getId() === undefined )
        {
            return Promise.reject( {
                status:     404,
                response:   "Not found"
            } );
        }
        else
        {
            // Create the service instance
            //
            var callURL: string = "/api/v1/fashion/product/" + this.getId();
            var service = new EVAService<EVA.Babylon.DeleteFashionProduct, EVA.Core.EmptyResponseMessage>( "DeleteFashionProduct", callURL, EXMLHttpRequestMethod.DELETE );

            // Build the request data
            // The base class will add things like SessionID so keep typescript
            // happy we're using an 'any' type object here
            //
            var requestData: any = {
            };

            // Perform the call and capture the resulting promise
            //
            return service.call( requestData );
        }
    }

    public getCoverSheetImages(): Promise<IEVAServiceResponse<EVA.Babylon.GetProductImagesForCoverSheet, EVA.Babylon.GetProductImagesForCoverSheetResponse>>
    {
        if ( this.getId() === undefined )
        {
            return Promise.reject( {
                status:     404,
                response:   "Not found"
            } );
        }
        else
        {
            // Fetch the product cover sheet images
            //
            var service = new EVAService<EVA.Babylon.GetProductImagesForCoverSheet, EVA.Babylon.GetProductImagesForCoverSheetResponse>( "GetProductImagesForCoverSheet", "/api/v1/fashion/product/" + this.getId() + "/cover_sheet_images", EXMLHttpRequestMethod.GET );

            // Perform the call and capture the resulting promise
            //
            return service.call();
        }
    }

    public getCreator(): Promise<IEVAServiceResponse<EVA.Babylon.GetFashionProductCreator, EVA.Babylon.GetFashionProductCreatorResponse>>
    {
        if ( this.getId() === undefined )
        {
            return Promise.reject( {
                status:     404,
                response:   "Not found"
            } );
        }
        else
        {
            // Fetch the product cover sheet images
            //
            var service = new EVAService<EVA.Babylon.GetFashionProductCreator, EVA.Babylon.GetFashionProductCreatorResponse>( "GetFashionProductCreator", "/api/v1/fashion/product/" + this.getId() + "/creator", EXMLHttpRequestMethod.GET );

            // Perform the call and capture the resulting promise
            //
            return service.call();
        }
    }

    public countCollections(): Promise<IEVAServiceResponse<EVA.Babylon.CountFashionCollectionsForProduct, EVA.Babylon.CountFashionCollectionsForProductResponse>>
    {
        if ( this.getId() === undefined )
        {
            return Promise.reject( {
                status:     404,
                response:   "Not found"
            } );
        }
        else
        {
            // Fetch the product cover sheet images
            //
            var service = new EVAService<EVA.Babylon.CountFashionCollectionsForProduct, EVA.Babylon.CountFashionCollectionsForProductResponse>( "CountFashionCollectionsForProduct", "/api/v1/fashion/product/" + this.getId() + "/collection/count", EXMLHttpRequestMethod.GET );

            // Perform the call and capture the resulting promise
            //
            return service.call();
        }
    }
}

/**
 * Provides access to a list of products available in EVA
 */
export class ProductList extends FetchablePagedList<EVA.Core.ProductDto, EVA.Core.Services.SearchProducts, EVA.Core.Services.SearchProductsResponse>
{
    /**
     * The return aggregations flag
     * @type {any}
     */
    public includeAggregations: boolean = true;

    /**
     * The return suggestions flag
     * @type {any}
     */
    public includeSuggestions: boolean = false;

    /**
     * The query string to search for in the products catalog
     * @type {string}
     */
    private query: string = "";

    /**
     * The list of fields to exclude from the products in the list response
     * @type {Array<string>}
     */
    private excludedFields: Array<string> = [];

    /**
     * The list of fields to include in the products in the list response
     * @type {Array<string>}
     */
    private includedFields: Array<string>;

    /**
     * The options for the list of fields to exclude from the products in the list response
     * @type {Array<string>}
     */
    private excludedFieldsOption: EVA.Core.ExcludedFieldOptions;

    /**
     * The collection of aggregation filters to search for in the products catalog
     * @type {any}
     */
    private aggregationFilters: any;

    /**
     * The collection of aggregation options to apply to the search in the products catalog
     * @type {any}
     */
    private aggregationOptions: any;

    /**
     * The aggregation state
     * @type {string}
     */
    private aggregationState: string;

    /**
     * The returned aggregations
     * @type {any}
     */
    private aggregations: any;


    /**
     * Returns the current list of excluded fields
     * @return {Array<string>} The configured list of excluded product fields
     */
    public getExcludedFields(): Array<string>
    {
        return this.excludedFields;
    }

    /**
     * Returns the current list of excluded fields
     * @return {Array<string>} The configured list of excluded product fields
     */
    public addExcludedField( fieldName: string ): Array<string>
    {
        this.excludedFields.push( fieldName );
        return this.excludedFields;
    }

    /**
     * Clears the configured list of excluded fields
     * @return {Array<string>} The configured list of excluded product fields
     */
    public clearExcludedFields(): Array<string>
    {
        return this.excludedFields = [];
    }

    /**
     * Sets the excluded fields option
     * @return {EVA.Core.ExcludedFieldOptions} The aggregation state
     */
    public setExcludedFieldsOptions( option: EVA.Core.ExcludedFieldOptions ): any
    {
        return this.excludedFieldsOption = option;
    }

    /**
     * Return the currently set excluded field option
     * @return {EVA.Core.ExcludedFieldOptions} The excluded field option value
     */
    public getExcludedFieldsOption(): EVA.Core.ExcludedFieldOptions
    {
        return this.excludedFieldsOption;
    }

    /**
     * Returns the current list of included fields
     * @return {Array<string>} The configured list of included product fields
     */
    public getIncludedFields(): Array<string>
    {
        return this.includedFields;
    }

    /**
     * Returns the current list of included fields
     * @return {Array<string>} The configured list of included product fields
     */
    public addIncludedField( fieldName: string ): Array<string>
    {
        if ( !_.isArray( this.includedFields) )
        {
            this.includedFields = [];
        }
        this.includedFields.push( fieldName );
        return this.includedFields;
    }

    /**
     * Clears the configured list of included fields
     * @return {Array<string>} The configured list of included product fields
     */
    public clearIncludedFields(): Array<string>
    {
        return this.includedFields = [];
    }

    /**
     * Sets the aggregation state
     * @return {any} The aggregation state
     */
    public setAggregationState( aggregationState: string ): any
    {
        return this.aggregationState = aggregationState;
    }

    /**
     * Return the currently set aggregation state
     * @return {string} The aggregation state
     */
    public getAggregationState(): string
    {
        return this.aggregationState;
    }

    /**
     * Sets the aggregation options
     * @return {any} The aggregation options
     */
    public setAggregationOptions( aggregationOptions: any ): any
    {
        return this.aggregationOptions = aggregationOptions;
    }

    /**
     * Return the currently set aggregation options
     * @return {any} The aggregation options
     */
    public getAggregationOptions(): any
    {
        return this.aggregationOptions;
    }

    /**
     * Returns the current query string
     * @return {string}       The configured query string
     */
    public getQuery(): string
    {
        return this.query;
    }

    /**
     * Set the query to search for
     * @param  {string} query The new query string
     * @return {string}       The configured query string
     */
    public setQuery( query: string ): string
    {
        return this.query = query;
    }

    /**
     * Return the aggregations that were returned with the product list
     * @return {any} The aggreations
     */
    public getAggregations(): any
    {
        return this.aggregations;
    }

    /**
     * Sets the current item details using the fetched service response.
     * Extending classes should override this method to pull the details from response
     * @param  {EVA.Core.Services.Products.SearchProductsResponse}  response    The response from the service call
     * @return {Array<EVA.Core.ProductDto>}                      The product items for the current page
     */
    public setData( response: any ): EVA.Core.ProductDto[]
    {
        // The default response is usually EVA.Core.PagedResultGeneric
        // An extending class will know exactly what is returned and where
        // the data for <T> can be found
        //
        // Store the paging response information
        //
        this.currentPageNumber  = response.Page;
        this.totalNumberOfPages = Math.ceil( response.Total / response.PageSize );
        this.totalNumberOfItems = response.Total;
        this.appliedFilter      = response.Filters;

        // Store the aggregations
        //
        this.aggregations     = response.Aggregations;
        this.aggregationState = response.AggregationState;
        this.aggregationOptions = response.AggregationOptions;

        // Set and return the page items
        //
        return this.data = response.Products;
    }

    /**
     * Fetches the product list item details
     * @return {Promise<IEVAServiceResponse<EVA.Core.ProductDto, EVA.Core.Services.SearchProducts, EVA.Core.Services.SearchProductsResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Core.Services.SearchProducts, EVA.Core.Services.SearchProductsResponse>> {

        // Ensure we have a sane page configuration
        //
        if ( _.isEmpty( this.pageConfig ) )
        {
            this.pageConfig = {
                Start:          0,
                Limit:          10,
                Filter:         {},
                SortProperty:   undefined,
                SortDirection:  undefined
            };
        }

        // The SearchProduct calls works similar to a paged request but the page
        // configuration details are placed in seperate fields
        //
        var sortField:  EVA.Core.SortFieldDescriptor = <EVA.Core.SortFieldDescriptor>{
            FieldName:  this.pageConfig.SortProperty,
            Direction:  this.pageConfig.SortDirection
        };

        // Build the request data
        //
        var requestData: EVA.Core.Services.SearchProducts = <EVA.Core.Services.SearchProducts> {
            Page:                   Math.floor( this.pageConfig.Start / this.pageConfig.Limit ) + 1,
            PageSize:               this.pageConfig.Limit,
            Filters:                this.pageConfig.Filter as any,
            Sort:                   [ sortField ],
            IncludedFields:         this.includedFields,
            ExcludedFields:         this.excludedFields,
            ExcludedFieldsOption:   this.excludedFieldsOption,
            IncludeAggregations:    this.includeAggregations,
            IncludeSuggestions:     this.includeSuggestions,
            AggregationState:       this.aggregationState,
            AggregationOptions:     this.aggregationOptions,
            Query:                  this.query || "",
            __response:             undefined
        };

        return super.fetch( "SearchProducts", requestData );
    }
}



/**
 * Provides access to a list of recently viewed (fashion) products available in EVA
 */
export class RecentlyViewedProductList extends FetchablePagedList<EVA.Babylon.RecentlyViewedProduct, EVA.Babylon.ListRecentlyViewedProducts, EVA.Babylon.ListRecentlyViewedProductsResponse>
{
    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Babylon.ListRecentlyViewedProducts, EVA.Babylon.ListRecentlyViewedProductsResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Babylon.ListRecentlyViewedProducts, EVA.Babylon.ListRecentlyViewedProductsResponse>>
    {
        // Build the request data
        //
        var requestData: any = {};

        // Apply PageConfig parameters directly
        //
        _.assign( requestData, _.omit( this.getPageConfiguration(), [ "Filter" ] ) );

        return super.fetch( "ListRecentlyViewedProducts", requestData );
    }
}

export function getFashionProductCategories(): Promise<IEVAServiceResponse<EVA.Babylon.GetFashionProductCategories, EVA.Babylon.GetFashionProductCategoriesResponse>>
{
    "use strict";

    // Create the service instance
    //
    var callURL: string = "/api/v1/fashion/product/category";
    var service = new EVAService<EVA.Babylon.GetFashionProductCategories, EVA.Babylon.GetFashionProductCategoriesResponse>( "GetFashionProductCategories", callURL, EXMLHttpRequestMethod.GET );

    // Build the request data
    //
    var requestData: EVA.Babylon.GetFashionProductCategories  = <EVA.Babylon.GetFashionProductCategories> {
    };

    // Perform the call and return resulting promise
    //
    return service.call( requestData );
}
