/**
 * The applications module provides access to the applications data from the EVA back-end
 * @preferred
 */

import { FetchablePagedList                 } from "./base";
import { EVAService, IEVAServiceResponse    } from "../service";
import { EXMLHttpRequestMethod              } from "../xhr";
import * as _                                 from "lodash";

/**
 * Provides access to a babylon social functions in EVA
 * These are the by identifier versions that can be reused by EVA SDK objects
 */
export namespace Search
{
    "use strict";

    export function quickFashionSearch( query: string, collectionSize?: number, productSize?: number, actorSize?: number, onlyAutocompleteProducts: boolean = false ): Promise<IEVAServiceResponse<EVA.Babylon.QuickSearchFashionItems, EVA.Babylon.QuickSearchFashionItemsResponse>>
    {
        // Create the service instance
        //
        var callURL: string = `/api/v1/fashion/quicksearch`;
        var service = new EVAService<EVA.Babylon.QuickSearchFashionItems, EVA.Babylon.QuickSearchFashionItemsResponse>( "QuickSearchFashionItems", callURL, EXMLHttpRequestMethod.POST );

        var requestData: EVA.Babylon.QuickSearchFashionItems = <EVA.Babylon.QuickSearchFashionItems>{
            FashionCollectionSize:    collectionSize,
            ProductSize:              productSize,
            ActorSize:                actorSize,
            Query:                    query,
            OnlyAutocompleteProducts: onlyAutocompleteProducts
        };

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData );
    }

    export function socialUserSearch( query: string, pageSize?: number, pageNr?: number )
    {
        // Create the service instance
        //
        var callURL: string = `/api/v1/social/user/search`;
        var service = new EVAService<EVA.Babylon.SocialUserSearch, EVA.Babylon.SocialUserSearchResponse>( "SocialUserSearch", callURL, EXMLHttpRequestMethod.POST );

        var requestData: EVA.Babylon.SocialUserSearch = <EVA.Babylon.SocialUserSearch>{
            Page:                   pageNr,
            PageSize:               pageSize,
            Query:                  query
        };

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData );
    }

    export function socialFriendsSearch( query: string, pageSize?: number, pageNr?: number )
    {
        // Create the service instance
        //
        var callURL: string = `/api/v1/social/user/friends/search`;
        var service = new EVAService<EVA.Babylon.SocialSearchFriends, EVA.Babylon.SocialSearchFriendsResponse>( "SocialSearchFriends", callURL, EXMLHttpRequestMethod.POST );

        var requestData: EVA.Babylon.SocialSearchFriends = <EVA.Babylon.SocialSearchFriends>{
            Page:                   pageNr,
            PageSize:               pageSize,
            Query:                  query
        };

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData );
    }

    export function fashionCollectionSearch( query: string, pageSize?: number, pageNr?: number, supplierName?: string, sortOption?: EVA.Babylon.FashionCollectionSearchModelSortOptions )
    {
        // Create the service instance
        //
        var callURL: string = `/api/v1/fashion/collection/search`;
        var service = new EVAService<EVA.Babylon.SearchFashionCollections, EVA.Babylon.SearchFashionCollectionsResponse>( "SearchFashionCollections", callURL, EXMLHttpRequestMethod.POST );

        var requestData: EVA.Babylon.SearchFashionCollections = <EVA.Babylon.SearchFashionCollections>{
            Page:                   pageNr,
            PageSize:               pageSize,
            Query:                  query,
            SupplierName:           supplierName,
            SortOptions:            sortOption
        };

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData );
    }
}

/**
 * Provides access to a list of products available in EVA
 */
export class FashionProductList extends FetchablePagedList<EVA.Core.ProductDto, EVA.Babylon.SearchFashionProducts, EVA.Core.Services.SearchProductsResponse>
{
    /**
     * The return aggregations flag
     * @type {any}
     */
    public includeAggregations: boolean = true;

    /**
     * The return suggestions flag
     * @type {any}
     */
    public includeSuggestions: boolean = false;

    /**
     * The query string to search for in the products catalog
     * @type {string}
     */
    private query: string = "";

    /**
     * The list of fields to exclude from the products in the list response
     * @type {Array<string>}
     */
    private excludedFields: Array<string> = [];

    /**
     * The options for the list of fields to exclude from the products in the list response
     * @type {Array<string>}
     */
    private excludedFieldsOption: EVA.Core.ExcludedFieldOptions;

    /**
     * The list of fields to include in the products in the list response
     * @type {Array<string>}
     */
    private includedFields: Array<string> = [];

    /**
     * The collection of aggregation filters to search for in the products catalog
     * @type {any}
     */
    private aggregationFilters: any;

    /**
     * The aggregation guid
     * @type {string}
     */
    private aggregationState: string;

    /**
     * The returned aggregations
     * @type {any}
     */
    private aggregations: any;

    /**
     * The collection of aggregation options to apply to the search in the products catalog
     * @type {any}
     */
    private aggregationOptions: any;

    /**
     * Returns the current list of excluded fields
     * @return {Array<string>} The configured list of excluded product fields
     */
    public getExcludedFields(): Array<string>
    {
        return this.excludedFields;
    }

    /**
     * Returns the current list of excluded fields
     * @return {Array<string>} The configured list of excluded product fields
     */
    public addExcludedField( fieldName: string ): Array<string>
    {
        this.excludedFields.push( fieldName );
        return this.excludedFields;
    }

    /**
     * Clears the configured list of excluded fields
     * @return {Array<string>} The configured list of excluded product fields
     */
    public clearExcludedFields(): Array<string>
    {
        return this.excludedFields = [];
    }

    /**
     * Sets the excluded fields option
     * @return {EVA.Core.ExcludedFieldOptions} The aggregation state
     */
    public setExcludedFieldsOptions( option: EVA.Core.ExcludedFieldOptions ): any
    {
        return this.excludedFieldsOption = option;
    }

    /**
     * Return the currently set excluded field option
     * @return {EVA.Core.ExcludedFieldOptions} The excluded field option value
     */
    public getExcludedFieldsOption(): EVA.Core.ExcludedFieldOptions
    {
        return this.excludedFieldsOption;
    }

    /**
     * Returns the current list of included fields
     * @return {Array<string>} The configured list of included product fields
     */
    public getIncludedFields(): Array<string>
    {
        return this.includedFields;
    }

    /**
     * Returns the current list of included fields
     * @return {Array<string>} The configured list of included product fields
     */
    public addIncludedField( fieldName: string ): Array<string>
    {
        if ( !_.isArray( this.includedFields) )
        {
            this.includedFields = [];
        }
        this.includedFields.push( fieldName );
        return this.includedFields;
    }

    /**
     * Clears the configured list of included fields
     * @return {Array<string>} The configured list of included product fields
     */
    public clearIncludedFields(): Array<string>
    {
        return this.includedFields = [];
    }

    /**
     * Sets the aggregation filters
     * @return {any} The aggregation filters
     */
    public setAggregationFilters( aggregationFilters: any ): any
    {
        return this.aggregationFilters = aggregationFilters;
    }

    /**
     * Return the currently set aggregation filters
     * @return {any} The aggregation filters
     */
    public getAggregationFilters(): any
    {
        return this.aggregationFilters;
    }

    /**
     * Sets the aggregation options
     * @return {any} The aggregation options
     */
    public setAggregationOptions( aggregationOptions: any ): any
    {
        return this.aggregationOptions = aggregationOptions;
    }

    /**
     * Return the currently set aggregation options
     * @return {any} The aggregation options
     */
    public getAggregationOptions(): any
    {
        return this.aggregationOptions;
    }

    /**
     * Returns the current query string
     * @return {string}       The configured query string
     */
    public getQuery(): string
    {
        return this.query;
    }

    /**
     * Set the query to search for
     * @param  {string} query The new query string
     * @return {string}       The configured query string
     */
    public setQuery( query: string ): string
    {
        return this.query = query;
    }

    /**
     * Return the aggregations that were returned with the product list
     * @return {any} The aggreations
     */
    public getAggregations(): any
    {
        return this.aggregations;
    }

    /**
     * Sets the current item details using the fetched service response.
     * Extending classes should override this method to pull the details from response
     * @param  {EVA.Core.Services.Products.SearchProductsResponse}  response    The response from the service call
     * @return {Array<EVA.Core.ProductDto>}                                     The product items for the current page
     */
    public setData( response: any ): EVA.Core.ProductDto[]
    {
        // The default response is usually EVA.Core.PagedResultGeneric
        // An extending class will know exactly what is returned and where
        // the data for <T> can be found
        //
        // Store the paging response information
        //
        this.currentPageNumber  = response.Page;
        this.totalNumberOfPages = Math.ceil( response.Total / response.PageSize );
        this.totalNumberOfItems = response.Total;
        this.appliedFilter      = response.Filters;

        // Store the aggregations
        //
        this.aggregations       = response.Aggregations;
        this.aggregationState   = response.AggregationState;
        this.aggregationFilters = response.AggregationFilters;

        // Set and return the page items
        //
        return this.data = response.Products;
    }

    /**
     * Fetches the product list item details
     * @return {Promise<IEVAServiceResponse<EVA.Core.ProductDto, EVA.Babylon.SearchFashionProducts, EVA.Core.Services.SearchProductsResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Babylon.SearchFashionProducts, EVA.Core.Services.SearchProductsResponse>>
    {
        // Ensure we have a sane page configuration
        //
        if ( _.isEmpty( this.pageConfig ) )
        {
            this.pageConfig = {
                Start:          0,
                Limit:          10,
                Filter:         {},
                SortProperty:   undefined,
                SortDirection:  undefined
            };
        }

        // The SearchProduct calls works similar to a paged request but the page
        // configuration details are placed in seperate fields
        //
        var sortField:  EVA.Core.SortFieldDescriptor = <EVA.Core.SortFieldDescriptor>{
            FieldName:  this.pageConfig.SortProperty,
            Direction:  this.pageConfig.SortDirection
        };

        // Build the request data
        //
        var requestData: EVA.Babylon.SearchFashionProducts = <EVA.Babylon.SearchFashionProducts> {
            Page:                   Math.floor( this.pageConfig.Start / this.pageConfig.Limit ) + 1,
            PageSize:               this.pageConfig.Limit,
            Filters:                this.pageConfig.Filter as any,
            Sort:                   [ sortField ],
            IncludedFields:         this.includedFields,
            ExcludedFields:         this.excludedFields,
            IncludeAggregations:    this.includeAggregations,
            IncludeSuggestions:     this.includeSuggestions,
            ExcludedFieldsOption:   this.excludedFieldsOption,
            AggregationFilters:     this.aggregationFilters,
            AggregationState:       this.aggregationState,
            AggregationOptions:     this.aggregationOptions,
            Query:                  this.query || "",
            __response:             undefined
        };

        return super.fetch( "SearchFashionProducts", requestData );
    }
}
