/**
 * The shopping cart module provides access to shopping cart data from the EVA back-end
 * Note that the shopping cart is bound to the current session so there is only 1 of them
 * You can regard it as the active order
 * @preferred
 */
import { FetchableItem } from "./base";
import { IEVAServiceResponse } from "../service";
import { LoggedInUser, User } from "./user";
import { Payments } from "./payments";
import { Product } from "./products";
import { Order } from "./orders";
/**
 * Provides access to the shopping cart available in EVA
 */
export declare class ShoppingCart extends FetchableItem<EVA.Core.OrderDto, EVA.Core.Services.GetShoppingCart, EVA.Core.ShoppingCartResponse> {
    /**
     * The payment options for the order
     * @type {Payment}
     */
    payment: Payments;
    /**
     * Indicates the order/cart is a pickup order (for a single shop)
     * @type {boolean}
     */
    isPickupOrder: boolean;
    /**
     * The current open amount still to be paid for the shopping cart.
     * @type {number}
     */
    private openAmount;
    /**
     * The current open foreign amount still to be paid for the shopping cart converted to the currency set on order.
     * @type {number}
     */
    private foreignOpenAmount;
    /**
     * The value of the items in the shopping cart with VAT
     * @type {number}
     */
    private totalAmountInTax;
    /**
     * The value of the items in the shopping cart without VAT
     * @type {number}
     */
    private totalAmount;
    /**
     * Order state indicating the order is completed
     * @type {boolean}
     */
    private _isCompleted;
    /**
     * Order state indicating the order is shipped
     * @type {boolean}
     */
    private _isShipped;
    /**
     * Order state indicating the order is paid
     * @type {boolean}
     */
    private _isPaid;
    /**
     * Order state indicating the order is invoiced
     * @type {boolean}
     */
    private _isInvoiced;
    /**
     * Order state indicating the order is cancelled
     * @type {boolean}
     */
    private _isCancelled;
    /**
     * The list of organizations where orderline items are reserved for pickup
     * @type {Array<any>}
     */
    private _stockOrganizationUnits;
    /**
     * The list of requested dates for orderline item deliveries
     * @type {Array<any>}
     */
    private _requestedDates;
    /**
     * Returned messages to check discount invalid reasons
     * @type {EVA.Core.DiscountInvalidReasons[]}
     */
    private _messages;
    /**
     * This class wraps the current shopping cart from EVA and the operations you can perform on it.
     *
     * @return {void} This method doesn't return anything
     */
    constructor();
    /**
     * Returns if the order state 'completed' applies to the order
     * @return {boolean} Indicates the state is active
     */
    isCompleted(): boolean;
    /**
     * Returns if the order state 'shipped' applies to the order
     * @return {boolean} Indicates the state is active
     */
    isShipped(): boolean;
    /**
     * Returns if the order state 'paid' applies to the order
     * @return {boolean} Indicates the state is active
     */
    isPaid(): boolean;
    /**
     * Returns if the order state 'invoiced' applies to the order
     * @return {boolean} Indicates the state is active
     */
    isInvoiced(): boolean;
    /**
     * Returns if the order state 'cancelled' applies to the order
     * @return {boolean} Indicates the state is active
     */
    isCancelled(): boolean;
    /**
     * Returns the current open amount still to be paid for the shopping cart
     * @return {number}
     */
    getOpenAmount(): number;
    /**
     * Returns the total value of all the items in the shopping cart with VAT
     * @return {number} The value of the items in the shopping cart with VAT
     */
    getTotalAmountInTax(): number;
    /**
     * Returns the total value of all the items in the shopping cart without VAT
     * @return {number} The value of the items in the shopping cart without VAT
     */
    getTotalAmount(): number;
    /**
     * Return order lines by type
     * @return {Array<EVA.Core.OrderLineDto>} Array of order lines
     */
    getOrderLinesByType(orderLineType: number | EVA.Core.OrderLineTypes): Array<EVA.Core.OrderLineDto>;
    /**
     * Return the total amount of shipping costs
     * @return {number} The total value of the shipping costs
     */
    getTotalShippingAmount(inTax?: boolean): number;
    /**
     * Return the total amount of ShippingCosts
     * @return {number} The total value of the shipping costs
     */
    getTotalDiscountAmount(inTax?: boolean): number;
    /**
     * Returns the list of different organizations where orderline items are te be picked up
     * @return {number} The list of organizations
     */
    readonly stockOrganizationUnits: Array<any>;
    /**
     * Returns the list of different organizations where orderline items are te be picked up
     * @return {number} The list of organizations
     */
    readonly requestedDates: Array<any>;
    /**
     * Returns the discount invalid reasons
     * @return {number}
     */
    getMessages(): EVA.Core.DiscountInvalidReasons[];
    /**
     * Fetches the shopping cart details
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetShoppingCart, EVA.Core.ShoppingCartResponse>>} A promise resolving when the call to the back-end has completed
     */
    fetch(): Promise<IEVAServiceResponse<EVA.Core.Services.GetShoppingCart, EVA.Core.ShoppingCartResponse>>;
    /**
     * Sets the current item details using the fetched service response.
     * Extending classes should override this method to pull the details from response
     * @param  {EVA.Core.ShoppingCartResponse}  response    The response from the service call
     * @return {EVA.Core.OrderDto}              The order details
     */
    setData(response: any): EVA.Core.OrderDto;
    /**
     * Adds a product to the shopping cart
     * @param  {number|Product}                 product         Either a product id or a product class to add to the shopping cart
     * @param  {number}                         quantity        Quantity to add
     * @param  {EVA.Core.LineActionTypes}       lineActionType  Line action type. Default is 4 (delivery), 1 is for reservation
     * @return {Promise}                                        A promise resolving when the call to the back-end has completed
     */
    addProduct(product: number | Product, quantity?: number, lineActionType?: EVA.Core.LineActionTypes): Promise<IEVAServiceResponse<EVA.Core.Services.AddProductToOrder, EVA.Core.Services.AddProductToOrderResponse>>;
    /**
     * Changes an order line to a stock reservation
     * @param  {number}     orderLineId     The order line id that contains the product(s) that is/are to be reserverd
     * @param  {number}     organizationID  The shop to reserve the product at
     * @return {Promise}                    A promise resolving when the call to the back-end has completed
     */
    reserveOrderLine(orderLineId: number, organizationID: number): Promise<{}>;
    /**
     * Changes an order line to be shipped
     * @param  {number}     orderLineId     The order line id that contains the product(s) that is/are to be reserverd
     * @return {Promise}                    A promise resolving when the call to the back-end has completed
     */
    shipOrderLine(orderLineId: number): Promise<{}>;
    /**
     * Changes an order line to be delivered
     * @param  {number}     orderLineId     The order line id that contains the product(s) that is/are to be reserverd
     * @return {Promise}                    A promise resolving when the call to the back-end has completed
     */
    deliverOrderLine(orderLineId: number): Promise<{}>;
    /**
     * Removes order lines from the shopping cart
     * @param  {number|Array<number>}       orderLineId     The identifier or array of identifiers of the order line(s) to remove from the shopping cart
     * @return {Promise}                                    A promise resolving when the call to the back-end has completed
     */
    cancelOrderLine(orderLineId: number | Array<number>): Promise<IEVAServiceResponse<EVA.Core.Services.CancelOrderLine, EVA.Core.Services.CancelOrderLineResponse>>;
    /**
     * Convenience methods to remove all instances of a product from the shopping  This is the more direct mirror call of addProduct
     * @param  {number|Product}     product     Either a product id or a product class to remove from the shopping cart
     * @return {Promise}                        A promise resolving when the call to the back-end has completed
     */
    removeProduct(product: number | Product): Promise<IEVAServiceResponse<EVA.Core.Services.CancelOrderLine, EVA.Core.ShoppingCartResponse>>;
    /**
     * Modify the number of items for an order line
     * @param  {number}         orderLineId     The identifier of the order line to modify from the shopping cart
     * @param  {number}         quantity        The new quantity
     * @return {Promise}                        A promise resolving when the call to the back-end has completed
     */
    modifyQuantity(orderLineId: number, quantity: number): Promise<IEVAServiceResponse<EVA.Core.Services.ModifyQuantityOrdered, EVA.Core.ShoppingCartResponse>>;
    /**
     * Clears the entire shopping basket
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetShoppingCart, EVA.Core.ShoppingCartResponse>>} A promise resolving when the call to the back-end has completed
     */
    empty(): Promise<IEVAServiceResponse<EVA.Core.Services.GetShoppingCart, EVA.Core.ShoppingCartResponse>>;
    /**
     * List the available shipping methods for the current cart
     * @return {Promise}                        A promise resolving when the call to the back-end has completed
     */
    listShippingMethods(orderId: number, orderlineIds: Array<number>): Promise<IEVAServiceResponse<EVA.Core.Services.ListAvailableShippingMethods, EVA.Core.Services.ListAvailableShippingMethodsResponse>>;
    /**
     * Gets the shipping methods currently set to the cart
     * @return {Promise}                        A promise resolving when the call to the back-end has completed
     */
    getShippingMethods(orderId: number): Promise<IEVAServiceResponse<EVA.Core.Services.GetShippingMethodsForOrder, EVA.Core.Services.GetShippingMethodsForOrderResponse>>;
    /**
     * Gets the available shipping dates of a shipping method
     * @return {Promise}                        A promise resolving when the call to the back-end has completed
     */
    getShippingDates(): Promise<IEVAServiceResponse<EVA.Core.Services.ListAvailableShippingDates, EVA.Core.Services.ListAvailableShippingDatesResponse>>;
    /**
     * Sets the pickup organiszation unit to the cart.
     * We don't know the orderId, but the session Id will do the trick
     * @return {Promise} A promise resolving when the call to the back-end has completed
     */
    setPickupOrganizationUnit(organizationUnitID: number): Promise<IEVAServiceResponse<EVA.Core.Services.SetPickupOrganizationUnit, EVA.Core.ShoppingCartResponse>>;
    /**
     * Gets the shipping methods currently set to the cart
     * @return {Promise}                        A promise resolving when the call to the back-end has completed
     */
    setShippingMethod(orderId: number, orderLineId: number, shippingMethodId: number, requestDeliveryDate: Date): Promise<IEVAServiceResponse<EVA.Core.Services.SetShippingMethod, EVA.Core.ShoppingCartResponse>>;
    /**
     * Modify the number of items for an order line
     * @param  {string}         couponCode      The coupon code that needs to be added to the cart
     * @return {Promise}                        A promise resolving when the call to the back-end has completed
     */
    addCouponCode(couponCode: string): Promise<IEVAServiceResponse<EVA.Core.Services.AddDiscountToOrder, EVA.Core.ShoppingCartResponse>>;
    /**
     * Retrieves the shopping cart summary information
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetShoppingCartInfo, EVA.Core.Services.ShoppingCartInfoResponse>>} A promise resolving when the call to the back-end has completed
     */
    getInfo(): Promise<IEVAServiceResponse<EVA.Core.Services.GetShoppingCartInfo, EVA.Core.Services.GetShoppingCartInfoResponse>>;
    /**
     * Attaches a customer to the shopping cart
     * @param  {LoggedInUser}       customer       The current user class that is to be set as the customer for the shopping cart
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    setCustomer(customer: LoggedInUser | User): Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>;
    /**
     * Places an order for the current shopping cart contents
     * @return {Promise<Order>} A promise resolving when the call to the back-end has completed
     */
    createOrder(): Promise<Order>;
    /**
     * Set the shopping basket to an existing order. Removes any existing items in the shopping cart
     * @param  {number|Order}       order       Either an order id or an order class to set as the shopping cart
     * @return {Promise}                        A promise resolving when the call to the back-end has completed
     */
    setFromOrder(order: number | Order): Promise<IEVAServiceResponse<EVA.Core.Services.GetShoppingCart, EVA.Core.ShoppingCartResponse>>;
    /**
     * Attaches a fashion collection to an order line
     * @param  {number}         orderLineId     The identifier of the order line to modify from the shopping cart
     * @param  {number}         collectionId    The fashion collection identifier
     * @return {Promise}                        A promise resolving when the call to the back-end has completed
     */
    setFashionCollectionOriginForOrderLine(orderLineID: number, collectionID: number): Promise<{}>;
}
