/**
 * The shopping cart module provides access to shopping cart data from the EVA back-end
 * Note that the shopping cart is bound to the current session so there is only 1 of them
 * You can regard it as the active order
 * @preferred
 */

import { FetchableItem                   } from "./base";
import { EVAService, IEVAServiceResponse } from "../service";
import { LoggedInUser, User              } from "./user";
import { Payments                        } from "./payments";
import { Product                         } from "./products";
import { Order                           } from "./orders";
import * as _                              from "lodash";

/**
 * Provides access to the shopping cart available in EVA
 */
export class ShoppingCart extends FetchableItem<EVA.Core.OrderDto, EVA.Core.Services.GetShoppingCart, EVA.Core.ShoppingCartResponse> {

    /**
     * The payment options for the order
     * @type {Payment}
     */
    public payment: Payments;

    /**
     * Indicates the order/cart is a pickup order (for a single shop)
     * @type {boolean}
     */
    public isPickupOrder: boolean;

    /**
     * The current open amount still to be paid for the shopping cart.
     * @type {number}
     */
    private openAmount: number;

    /**
     * The current open foreign amount still to be paid for the shopping cart converted to the currency set on order.
     * @type {number}
     */
    private foreignOpenAmount: number;

    /**
     * The value of the items in the shopping cart with VAT
     * @type {number}
     */
    private totalAmountInTax: number;

    /**
     * The value of the items in the shopping cart without VAT
     * @type {number}
     */
    private totalAmount: number;

    /**
     * Order state indicating the order is completed
     * @type {boolean}
     */
    private _isCompleted: boolean;

    /**
     * Order state indicating the order is shipped
     * @type {boolean}
     */
    private _isShipped: boolean;

    /**
     * Order state indicating the order is paid
     * @type {boolean}
     */
    private _isPaid: boolean;

    /**
     * Order state indicating the order is invoiced
     * @type {boolean}
     */
    private _isInvoiced: boolean;

    /**
     * Order state indicating the order is cancelled
     * @type {boolean}
     */
    private _isCancelled: boolean;

    /**
     * The list of organizations where orderline items are reserved for pickup
     * @type {Array<any>}
     */
    private _stockOrganizationUnits: Array<any>;

    /**
     * The list of requested dates for orderline item deliveries
     * @type {Array<any>}
     */
    private _requestedDates: Array<any>;

    /**
     * Returned messages to check discount invalid reasons
     * @type {EVA.Core.DiscountInvalidReasons[]}
     */
    private _messages: EVA.Core.DiscountInvalidReasons[];

    /**
     * This class wraps the current shopping cart from EVA and the operations you can perform on it.
     *
     * @return {void} This method doesn't return anything
     */
    constructor()
    {
        super();

        this.payment = new Payments( this );
    }

    /**
     * Returns if the order state 'completed' applies to the order
     * @return {boolean} Indicates the state is active
     */
    public isCompleted(): boolean
    {
        return this._isCompleted === true;
    }

    /**
     * Returns if the order state 'shipped' applies to the order
     * @return {boolean} Indicates the state is active
     */
    public isShipped(): boolean
    {
        return this._isShipped === true;
    }

    /**
     * Returns if the order state 'paid' applies to the order
     * @return {boolean} Indicates the state is active
     */
    public isPaid(): boolean
    {
        return this._isPaid === true;
    }

    /**
     * Returns if the order state 'invoiced' applies to the order
     * @return {boolean} Indicates the state is active
     */
    public isInvoiced(): boolean
    {
        return this._isInvoiced === true;
    }

    /**
     * Returns if the order state 'cancelled' applies to the order
     * @return {boolean} Indicates the state is active
     */
    public isCancelled(): boolean
    {
        return this._isCancelled === true;
    }

    /**
     * Returns the current open amount still to be paid for the shopping cart
     * @return {number}
     */
    public getOpenAmount(): number
    {
        return this.openAmount;
    }

    /**
     * Returns the total value of all the items in the shopping cart with VAT
     * @return {number} The value of the items in the shopping cart with VAT
     */
    public getTotalAmountInTax(): number
    {
        return this.totalAmountInTax;
    }

    /**
     * Returns the total value of all the items in the shopping cart without VAT
     * @return {number} The value of the items in the shopping cart without VAT
     */
    public getTotalAmount(): number
    {
        return this.totalAmount;
    }

    /**
     * Return order lines by type
     * @return {Array<EVA.Core.OrderLineDto>} Array of order lines
     */
    public getOrderLinesByType( orderLineType: number|EVA.Core.OrderLineTypes ): Array<EVA.Core.OrderLineDto>
    {
        return _.filter( this.data.Lines, ( line ) =>
        {
            return line.Type === orderLineType;
        } );
    }

    /**
     * Return the total amount of shipping costs
     * @return {number} The total value of the shipping costs
     */
    public getTotalShippingAmount( inTax: boolean = true ): number
    {
        let targetField: string;
        if ( inTax )
        {
            targetField = "TotalAmountInTax";
        }
        else
        {
            targetField = "TotalAmount";
        }
        return _.sumBy( this.getOrderLinesByType( 5 ), targetField );
    }

    /**
     * Return the total amount of ShippingCosts
     * @return {number} The total value of the shipping costs
     */
    public getTotalDiscountAmount( inTax: boolean = true ): number
    {
        let targetField: string;
        if ( inTax )
        {
            targetField = "TotalAmountInTax";
        }
        else
        {
            targetField = "TotalAmount";
        }
        return _.sumBy( this.getOrderLinesByType( 1 ), targetField );
    }

    /**
     * Returns the list of different organizations where orderline items are te be picked up
     * @return {number} The list of organizations
     */
    get stockOrganizationUnits(): Array<any>
    {
        return this._stockOrganizationUnits;
    }

    /**
     * Returns the list of different organizations where orderline items are te be picked up
     * @return {number} The list of organizations
     */
    get requestedDates(): Array<any>
    {
        return this._requestedDates;
    }

    /**
     * Returns the discount invalid reasons
     * @return {number}
     */
    public getMessages() {
        return this._messages;
    }

    /**
     * Fetches the shopping cart details
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetShoppingCart, EVA.Core.ShoppingCartResponse>>} A promise resolving when the call to the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Core.Services.GetShoppingCart, EVA.Core.ShoppingCartResponse>>
    {
        // Build the request data
        //
        var requestCartData: EVA.Core.Services.GetShoppingCart = <EVA.Core.Services.GetShoppingCart>{};

        return super.fetch( "GetShoppingCart", requestCartData );
    }

    /**
     * Sets the current item details using the fetched service response.
     * Extending classes should override this method to pull the details from response
     * @param  {EVA.Core.ShoppingCartResponse}  response    The response from the service call
     * @return {EVA.Core.OrderDto}              The order details
     */
    public setData( response: any ): EVA.Core.OrderDto
    {
        // Collect the data exposed outside of the OrderDto
        //
        this.openAmount         = response.OpenAmount             	      || 0;
        this.totalAmount        = response.TotalAmount                    || 0;
        this.totalAmountInTax   = response.TotalAmountInTax               || 0;
        this.foreignOpenAmount  = response.ForeignOpenAmount              || 0;
        this._isCompleted       = response.ShoppingCart.IsCompleted   === true;
        this._isShipped         = response.ShoppingCart.IsShipped     === true;
        this._isPaid            = response.ShoppingCart.IsPaid        === true;
        this._isInvoiced        = response.ShoppingCart.IsInvoiced    === true;
        this._isCancelled       = response.ShoppingCart.IsCancelled   === true;
        this._messages          = response.ShoppingCart.Messages         || [];

        // Set the shopping cart data
        //
        this.data = response.ShoppingCart;

        // Set or update the ID
        //
        this.id = this.data.ID;

        // Collect the stock organizations from the order lines
        // This used to be per line but is now centralised on the order
        //
        this._stockOrganizationUnits = [ this.data.OrganizationUnitID ];

        // Collect the requested delivery dates from the order lines
        //
        this._requestedDates = _.without( _.uniq( _.map( this.data.Lines, ( line ) =>
        {
            if ( line.LineActionType === 4 && line.RequestedDate )
            {
                return line.RequestedDate;
            }
        } ) ), undefined );

        // Check if the order is a pickup order
        //
        let pickupOrder = this._stockOrganizationUnits.length < 2;
        if ( pickupOrder )
        {
            _.forEach( this.data.Lines, ( line ) =>
            {
                pickupOrder = pickupOrder && line.LineActionType === 1;
            } );
        }
        this.isPickupOrder = pickupOrder;

        return this.data;
    }

    /**
     * Adds a product to the shopping cart
     * @param  {number|Product}                 product         Either a product id or a product class to add to the shopping cart
     * @param  {number}                         quantity        Quantity to add
     * @param  {EVA.Core.LineActionTypes}       lineActionType  Line action type. Default is 4 (delivery), 1 is for reservation
     * @return {Promise}                                        A promise resolving when the call to the back-end has completed
     */
    public addProduct( product: number|Product, quantity: number = 1, lineActionType: EVA.Core.LineActionTypes = 4 ): Promise<IEVAServiceResponse<EVA.Core.Services.AddProductToOrder, EVA.Core.Services.AddProductToOrderResponse>>
    {
        // You add a product to the shopping cart either by its id or by feeding us an
        // instance of the SDK product class
        //
        var productId: number|string;
        if ( product instanceof Product )
        {
            productId = product.getId();
        }
        else
        {
            productId = <number>product;
        }

        // Create our service instance
        //
        var service = new EVAService<EVA.Core.Services.AddProductToOrder, EVA.Core.Services.AddProductToOrderResponse>( "AddProductToOrder" );

        // Build the request data
        // The base class will add things like SessionID so to keep typescript
        // happy we're using an 'any' type object here
        //
        // Note that webshop can only reserve or ask for delivery of an items
        // So only the following LineActionTypes are allowed:
        //
        // EVA.Core.LineActionTypes.ReserveLine: 1
        // EVA.Core.LineActionTypes.Delivery:    4
        //
        var requestData: any = { // EVA.Core.Services.AddProductToOrder = {
            ProductID:              productId,
            QuantityOrdered:        quantity,
            LineActionType:         lineActionType,
        };

        // Perform the call and capture the resulting promise
        //
        return new Promise( ( resolve, reject ) => {
            service.call( requestData )
            .then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.AddProductToOrder, EVA.Core.Services.AddProductToOrderResponse> ) =>
            {
                this.setData( serviceResult.response );
                resolve( serviceResult );
            } )
            .catch( reject );
        } );
    }

    /**
     * Changes an order line to a stock reservation
     * @param  {number}     orderLineId     The order line id that contains the product(s) that is/are to be reserverd
     * @param  {number}     organizationID  The shop to reserve the product at
     * @return {Promise}                    A promise resolving when the call to the back-end has completed
     */
    public reserveOrderLine( orderLineId: number, organizationID: number  )
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Core.Services.ModifyStockOrganizationUnit, EVA.Core.ShoppingCartResponse>( "ModifyStockOrganizationUnit" );

        // Build the request data
        // The base class will add things like SessionID so to keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: any = {
            OrderLineID:             orderLineId,
            StockOrganizationUnitID: organizationID,
            LineActionType:          1
        };

        // Perform the call and capture the resulting promise
        //
        return new Promise( ( resolve, reject ) => {
            service.call( requestData )
            .then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.ModifyStockOrganizationUnit, EVA.Core.ShoppingCartResponse> ) =>
            {
                this.setData( serviceResult.response );
                resolve( serviceResult );
            } )
            .catch( reject );
        } );
    }

    /**
     * Changes an order line to be shipped
     * @param  {number}     orderLineId     The order line id that contains the product(s) that is/are to be reserverd
     * @return {Promise}                    A promise resolving when the call to the back-end has completed
     */
    public shipOrderLine( orderLineId: number  )
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Core.Services.ModifyLineActionType, EVA.Core.ShoppingCartResponse>( "ModifyLineActionType" );

        // Build the request data
        // The base class will add things like SessionID so to keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: any = {
            OrderLineID:             orderLineId,
            LineActionType:          3
        };

        // Perform the call and capture the resulting promise
        //
        return new Promise( ( resolve, reject ) => {
            service.call( requestData )
            .then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.ModifyLineActionType, EVA.Core.ShoppingCartResponse> ) =>
            {
                this.setData( serviceResult.response );
                resolve( serviceResult );
            } )
            .catch( reject );
        } );
    }

    /**
     * Changes an order line to be delivered
     * @param  {number}     orderLineId     The order line id that contains the product(s) that is/are to be reserverd
     * @return {Promise}                    A promise resolving when the call to the back-end has completed
     */
    public deliverOrderLine( orderLineId: number  )
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Core.Services.ModifyLineActionType, EVA.Core.ShoppingCartResponse>( "ModifyLineActionType" );

        // Build the request data
        // The base class will add things like SessionID so to keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: any = {
            OrderLineID:             orderLineId,
            LineActionType:          4
        };

        // Perform the call and capture the resulting promise
        //
        return new Promise( ( resolve, reject ) => {
            service.call( requestData )
            .then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.ModifyLineActionType, EVA.Core.ShoppingCartResponse> ) =>
            {
                this.setData( serviceResult.response );
                resolve( serviceResult );
            } )
            .catch( reject );
        } );
    }

    /**
     * Removes order lines from the shopping cart
     * @param  {number|Array<number>}       orderLineId     The identifier or array of identifiers of the order line(s) to remove from the shopping cart
     * @return {Promise}                                    A promise resolving when the call to the back-end has completed
     */
    public cancelOrderLine( orderLineId: number|Array<number> ): Promise<IEVAServiceResponse<EVA.Core.Services.CancelOrderLine, EVA.Core.Services.CancelOrderLineResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create our service instance
            //
            var service = new EVAService<EVA.Core.Services.CancelOrderLine, EVA.Core.Services.CancelOrderLineResponse>( "CancelOrderLine" );

            // Build the request data
            // The base class will add things like SessionID so to keep typescript
            // happy we're using an 'any' type object here
            //
            var requestData: any = {
                OrderLineID: orderLineId
            };

            // Perform the call and capture the resulting promise
            //
            service.call( requestData )
            .then( () =>
            {
                this.fetch().then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }

    /**
     * Convenience methods to remove all instances of a product from the shopping  This is the more direct mirror call of addProduct
     * @param  {number|Product}     product     Either a product id or a product class to remove from the shopping cart
     * @return {Promise}                        A promise resolving when the call to the back-end has completed
     */
    public removeProduct( product: number|Product ): Promise<IEVAServiceResponse<EVA.Core.Services.CancelOrderLine, EVA.Core.ShoppingCartResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // You remove a product from the shopping cart either by its id or by feeding us an
            // instance of the SDK product class
            //
            var productId: number|string;
            if ( product instanceof Product )
            {
                productId = product.getId();
            }
            else
            {
                productId = <number>product;
            }

            // Collect the order line id's where this product is present
            //
            var orderLineIds: Array<Promise<IEVAServiceResponse<EVA.Core.Services.CancelOrderLine, EVA.Core.Services.CancelOrderLineResponse>>> = [];
            if ( _.isArray( this.data.Lines ) )
            {
                _.forEach( this.data.Lines, ( orderLine: any ) =>
                {
                    if ( orderLine.ProductID === productId )
                    {
                        // Create an inline service call because the local method will refresh the cart each time
                        //
                        var service = new EVAService<EVA.Core.Services.CancelOrderLine, EVA.Core.Services.CancelOrderLineResponse>( "CancelOrderLine" );
                        var requestData: any = {
                            OrderLineID: orderLine.ID
                        };
                        orderLineIds.push( service.call( requestData ) );
                    }
                } );
            }

            (<any>Promise).all( orderLineIds )
            .then( () =>
            {
                this.fetch().then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }

    /**
     * Modify the number of items for an order line
     * @param  {number}         orderLineId     The identifier of the order line to modify from the shopping cart
     * @param  {number}         quantity        The new quantity
     * @return {Promise}                        A promise resolving when the call to the back-end has completed
     */
    public modifyQuantity( orderLineId: number, quantity: number ): Promise<IEVAServiceResponse<EVA.Core.Services.ModifyQuantityOrdered, EVA.Core.ShoppingCartResponse>>
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Core.Services.ModifyQuantityOrdered, EVA.Core.ShoppingCartResponse>( "ModifyQuantityOrdered" );

        // Build the request data
        // The base class will add things like SessionID so to keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: any = {
            OrderLineID:        orderLineId,
            NewQuantityOrdered: quantity
        };

        // Perform the call and capture the resulting promise
        //
        return new Promise( ( resolve, reject ) => {
            service.call( requestData )
            .then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.ModifyQuantityOrdered, EVA.Core.ShoppingCartResponse> ) =>
            {
                this.setData( serviceResult.response );
                resolve( serviceResult );
            } )
            .catch( reject );
        } );
    }

    /**
     * Clears the entire shopping basket
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetShoppingCart, EVA.Core.ShoppingCartResponse>>} A promise resolving when the call to the back-end has completed
     */
    public empty(): Promise<IEVAServiceResponse<EVA.Core.Services.GetShoppingCart, EVA.Core.ShoppingCartResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create our service instance
            //
            var service = new EVAService<EVA.Core.Services.DetachOrderFromSession, EVA.Core.EmptyResponseMessage>( "DetachOrderFromSession" );

            // Perform the call and capture the resulting promise
            //
            service.call()
            .then( () =>
            {
                this.fetch().then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }

    /**
     * List the available shipping methods for the current cart
     * @return {Promise}                        A promise resolving when the call to the back-end has completed
     */
    public listShippingMethods( orderId: number, orderlineIds: Array<number> ): Promise<IEVAServiceResponse<EVA.Core.Services.ListAvailableShippingMethods, EVA.Core.Services.ListAvailableShippingMethodsResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create our service instance
            //
            var service = new EVAService<EVA.Core.Services.ListAvailableShippingMethods, EVA.Core.Services.ListAvailableShippingMethodsResponse>( "ListAvailableShippingMethods" );

            var requestData: any = {
                OrderID:            orderId,
                OrderlineIDs:       orderlineIds
            };

            // Perform the call and capture the resulting promise
            //
            service.call( requestData )
            .then( ( result: any ) =>
            {
                resolve( result.response );
            } )
            .catch( reject );
        } );
    }

    /**
     * Gets the shipping methods currently set to the cart
     * @return {Promise}                        A promise resolving when the call to the back-end has completed
     */
    public getShippingMethods( orderId: number ): Promise<IEVAServiceResponse<EVA.Core.Services.GetShippingMethodsForOrder, EVA.Core.Services.GetShippingMethodsForOrderResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create our service instance
            //
            var service = new EVAService<EVA.Core.Services.GetShippingMethodsForOrder, EVA.Core.Services.GetShippingMethodsForOrderResponse>( "GetShippingMethodsForOrder" );

            var requestData: any = {
                OrderID:            orderId
            };

            // Perform the call and capture the resulting promise
            //
            service.call( requestData )
            .then( ( result: any ) =>
            {
                resolve( result.response );
            } )
            .catch( reject );
        } );
    }

    /**
     * Gets the available shipping dates of a shipping method
     * @return {Promise}                        A promise resolving when the call to the back-end has completed
     */
    public getShippingDates(): Promise<IEVAServiceResponse<EVA.Core.Services.ListAvailableShippingDates, EVA.Core.Services.ListAvailableShippingDatesResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create our service instance
            //
            var service = new EVAService<EVA.Core.Services.ListAvailableShippingDates, EVA.Core.Services.ListAvailableShippingDatesResponse>( "ListAvailableShippingDates" );

            // Perform the call and capture the resulting promise
            //
            service.call()
            .then( () =>
            {
                this.fetch().then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }

    /**
     * Sets the pickup organiszation unit to the cart.
     * We don't know the orderId, but the session Id will do the trick
     * @return {Promise} A promise resolving when the call to the back-end has completed
     */
    public setPickupOrganizationUnit( organizationUnitID: number ): Promise<IEVAServiceResponse<EVA.Core.Services.SetPickupOrganizationUnit, EVA.Core.ShoppingCartResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create our service instance
            //
            var service = new EVAService<EVA.Core.Services.SetPickupOrganizationUnit, EVA.Core.ShoppingCartResponse>( "SetPickupOrganizationUnit" );

            var requestData: EVA.Core.Services.SetPickupOrganizationUnit = {
                OrganizationUnitID: organizationUnitID,
                OrderID:            undefined,
                SessionID:          undefined,
                __response:         undefined
            };

            // Perform the call and capture the resulting promise
            //
            service.call( requestData )
            .then( () =>
            {
                this.fetch().then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }

    /**
     * Gets the shipping methods currently set to the cart
     * @return {Promise}                        A promise resolving when the call to the back-end has completed
     */
    public setShippingMethod( orderId: number, orderLineId: number, shippingMethodId: number, requestDeliveryDate: Date ): Promise<IEVAServiceResponse<EVA.Core.Services.SetShippingMethod, EVA.Core.ShoppingCartResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create our service instance
            //
            var service = new EVAService<EVA.Core.Services.SetShippingMethod, EVA.Core.ShoppingCartResponse>( "SetShippingMethod" );

            var requestData: any = {
                OrderID:                orderId,
                // OrderLineID:            orderLineId,
                shippingMethodId:       shippingMethodId,
                // RequestDeliveryDate:    requestDeliveryDate
            };

            if (orderLineId) {
                requestData.OrderLineID = orderLineId;
            }

            if (requestDeliveryDate) {
                requestData.RequestDeliveryDate = requestDeliveryDate;
            }

            // Perform the call and capture the resulting promise
            //
            service.call( requestData )
            .then( () =>
            {
                this.fetch().then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }

    /**
     * Modify the number of items for an order line
     * @param  {string}         couponCode      The coupon code that needs to be added to the cart
     * @return {Promise}                        A promise resolving when the call to the back-end has completed
     */
    public addCouponCode( couponCode: string ): Promise<IEVAServiceResponse<EVA.Core.Services.AddDiscountToOrder, EVA.Core.ShoppingCartResponse>>
    {
        // Create our service instance
        //
        var service = new EVAService<EVA.Core.Services.AddDiscountToOrder, EVA.Core.ShoppingCartResponse>( "AddDiscountToOrder" );

        // Build the request data
        // The base class will add things like SessionID so to keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: any = {
            CouponCode: couponCode
        };

        // Perform the call and capture the resulting promise
        //
        return new Promise( ( resolve, reject ) => {
            service.call( requestData )
            .then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.AddDiscountToOrder, EVA.Core.ShoppingCartResponse> ) =>
            {
                this.setData( serviceResult.response );
                resolve( serviceResult );
            } )
            .catch( reject );
        } );
    }

    /**
     * Retrieves the shopping cart summary information
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetShoppingCartInfo, EVA.Core.Services.ShoppingCartInfoResponse>>} A promise resolving when the call to the back-end has completed
     */
    public getInfo(): Promise<IEVAServiceResponse<EVA.Core.Services.GetShoppingCartInfo, EVA.Core.Services.GetShoppingCartInfoResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create our service instance
            //
            var service = new EVAService<EVA.Core.Services.GetShoppingCartInfo, EVA.Core.Services.GetShoppingCartInfoResponse>( "GetShoppingCartInfo" );

            // Perform the call
            //
            service.call().then( ( result ) =>
            {
                resolve( result.response );
            } ).catch( reject );
        } );
    }

    /**
     * Attaches a customer to the shopping cart
     * @param  {LoggedInUser}       customer       The current user class that is to be set as the customer for the shopping cart
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public setCustomer( customer: LoggedInUser|User ): Promise<IEVAServiceResponse<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create our service instance
            //
            var service = new EVAService<EVA.Core.Services.AttachCustomerToOrder, EVA.Core.EmptyResponseMessage>( "AttachCustomerToOrder" );

            // Build the request data
            // The base class will add things like SessionID so to keep typescript
            // happy we're using an 'any' type object here
            //
            var requestData: any = {
                UserID: customer.data.ID,
                OrderID: this.data.ID
            };

            // Perform the call and capture the resulting promise
            //
            service.call( requestData )
            .then( () =>
            {
                this.fetch().then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }

    /**
     * Places an order for the current shopping cart contents
     * @return {Promise<Order>} A promise resolving when the call to the back-end has completed
     */
    public createOrder(): Promise<Order>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create our service instance
            //
            var service = new EVAService<EVA.Core.Services.CreateOrderFromShoppingCart, EVA.Core.Services.CreateOrderFromShoppingCartResponse>( "CreateOrderFromShoppingCart" );

            // Build the request data
            // The base class will add things like SessionID so to keep typescript
            // happy we're using an 'any' type object here
            //
            var requestData: any = {
                CartID:         this.getId(),
                CustomerID:     _.get( this.data, "Customer.ID" )
            };

            // Perform the call and capture the resulting promise
            //
            service.call( requestData )
            .then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.CreateOrderFromShoppingCart, EVA.Core.Services.CreateOrderFromShoppingCartResponse> ) =>
            {
                var order = new Order();
                order.setData( serviceResult.response );

                resolve( order );
            } )
            .catch( reject );
        } );
    }

    /**
     * Set the shopping basket to an existing order. Removes any existing items in the shopping cart
     * @param  {number|Order}       order       Either an order id or an order class to set as the shopping cart
     * @return {Promise}                        A promise resolving when the call to the back-end has completed
     */
    public setFromOrder( order: number|Order): Promise<IEVAServiceResponse<EVA.Core.Services.GetShoppingCart, EVA.Core.ShoppingCartResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create our service instance
            //
            var service = new EVAService<EVA.Core.Services.AttachOrderToSession, EVA.Core.EmptyResponseMessage>( "AttachOrderToSession" );

            var orderId: number|string;
            if ( order instanceof Order )
            {
                orderId = order.getId();
            }
            else
            {
                orderId = <number>order;
            }

            // Build the request data
            // The base class will add things like SessionID so to keep typescript
            // happy we're using an 'any' type object here
            //
            var requestData: any = {
                OrderID: orderId,
            };

            // Perform the call and capture the resulting promise
            //
            service.call( requestData )
            .then( () =>
            {
                this.fetch().then( resolve ).catch( reject );
            } )
            .catch( reject );
        } );
    }

    /**
     * Attaches a fashion collection to an order line
     * @param  {number}         orderLineId     The identifier of the order line to modify from the shopping cart
     * @param  {number}         collectionId    The fashion collection identifier
     * @return {Promise}                        A promise resolving when the call to the back-end has completed
     */
   public setFashionCollectionOriginForOrderLine( orderLineID: number, collectionID: number )
   {
       return new Promise( ( resolve: any, reject: any ) =>
       {
           var service = new EVAService<EVA.Babylon.SetFashionCollectionOriginForOrderLine, EVA.Core.EmptyResponseMessage>( "SetFashionCollectionOriginForOrderLine" );

           var requestData: EVA.Babylon.SetFashionCollectionOriginForOrderLine = {
               OrderLineID:  orderLineID,
               CollectionID: collectionID,
               __response:   undefined
           };

           // Perform the call and capture the resulting promise
           //
           service.call( requestData )
           .then( () =>
           {
               this.fetch().then( resolve ).catch( reject );
           } )
           .catch( reject );
        } );
   }
}
