/**
 * The shops module provides access to the shops (organization unit type) data from the EVA back-end
 * @preferred
 */

import { FetchableItem, FetchablePagedList  } from "./base";
import { IEVAServiceResponse                } from "../service";

/**
 * Provides access to a content page available in EVA
 */
export class Shop extends FetchableItem<EVA.Core.OrganizationUnitDto, EVA.Core.Services.GetOrganizationUnit, EVA.Core.Services.GetOrganizationUnitResponse>
{

    /**
     * Fetches the page content
     * @return {Promise<IEVAServiceResponse<EVA.Core.Services.GetOrganizationUnit, EVA.Core.Services.GetOrganizationUnitResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Core.Services.GetOrganizationUnit, EVA.Core.Services.GetOrganizationUnitResponse>>
    {
        // Build the request data
        //
        var requestData: EVA.Core.Services.GetOrganizationUnit = <EVA.Core.Services.GetOrganizationUnit>{
            OrganizationUnitID: this.getId()
        };

        return super.fetch( "GetOrganizationUnit", requestData );
    }
}

/**
 * Provides access to a list of shops available in EVA
 */
export class ShopList extends FetchablePagedList<EVA.Core.OrganizationUnitDto, EVA.Core.Services.ListOrganizationUnitsByType, EVA.Core.Services.ListOrganizationUnitsByTypeResponse>
{

    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.CMS.Dto.OrganizationUnitDto, EVA.Core.Services.ListOrganizationUnitsByType, EVA.Core.Services.ListOrganizationUnitsByTypeResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Core.Services.ListOrganizationUnitsByType, EVA.Core.Services.ListOrganizationUnitsByTypeResponse>>
    {
        // Build the request data
        //
        var requestData: EVA.Core.Services.ListOrganizationUnitsByType = <EVA.Core.Services.ListOrganizationUnitsByType> {
            PageConfig:                this.getPageConfiguration(),
            OrganizationUnitTypes:     1, // Shop
            IncludeParentApplications: undefined,
            __response:                undefined
        };

        return super.fetch( "ListOrganizationUnitsByType", requestData );
    }
}

/**
 * Provides access to a list of shops available in EVA
 */
export class ShopProximityList extends FetchablePagedList<EVA.Core.OrganizationUnitDto, EVA.Core.Services.ListOrganizationUnitByProximity, EVA.Core.Services.ListOrganizationUnitByProximityResponse>
{

    /**
     * Optional location to use for the shop list (proximity search)
     * @type {string}
     */
    private location: any = {
        lat: null,
        lng: null
    };

    /**
     * Changes to list of shops returned to those in the closest proximity of the given location
     * @param  {number} lat The locations latitude
     * @param  {number} lng The locations longitude
     * @return {object}     The chosen location
     */
    public setLocation( lat: number, lng: number )
    {
        this.location.lat = lat;
        this.location.lng = lng;

        return this.location;
    }

    /**
     * Returns the currently chosen location to use for a proximity based shop list
     * @return {object} The currently chosen location
     */
    public getLocation()
    {
        return this.location;
    }

    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.CMS.Dto.OrganizationUnitDto, EVA.Core.Services.ListOrganizationUnitByProximity, EVA.Core.Services.ListOrganizationUnitByProximityResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Core.Services.ListOrganizationUnitByProximity, EVA.Core.Services.ListOrganizationUnitByProximityResponse>>
    {
        // Build the request data
        //
        var requestData: EVA.Core.Services.ListOrganizationUnitByProximity = <EVA.Core.Services.ListOrganizationUnitByProximity> {
            PageConfig:     this.getPageConfiguration(),
            Latitude:       this.location.lat,
            Longitude:      this.location.lng
        };

        return super.fetch( "ListOrganizationUnitByProximity", requestData );
    }
}
