/**
 * The applications module provides access to the applications data from the EVA back-end
 * @preferred
 */

import { SettingsManager                   } from "../settings";
import { FetchableItem, FetchablePagedList } from "./base";
import { EVAService, IEVAServiceResponse   } from "../service";
import { EXMLHttpRequestMethod             } from "../xhr";
import * as _                                from "lodash";

/**
 * Provides access to a babylon social functions in EVA
 * These are the by identifier versions that can be reused by EVA SDK objects
 */
export namespace Social
{
    "use strict";

    // Reusable social action method
    //
    function socialAction( itemID: number|string, type: string, action: string, requestData: any = {}, requestMethod: EXMLHttpRequestMethod = EXMLHttpRequestMethod.POST )
    {
        // Create the service instance
        //
        var callURL: string = `/api/v1/social/${type}/${itemID}/${action}`;
        var service = new EVAService<any, EVA.Core.EmptyResponseMessage>( "SocialAction", callURL, requestMethod );

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData );
    }

    function socialRequest( itemID: number|string, type: string, action: string, requestData: any = {} )
    {
        // Create the service instance
        //
        var callURL: string = `/api/v1/social/${type}/${itemID}/${action}`;
        var service = new EVAService<any, EVA.Core.EmptyResponseMessage>( "SocialAction", callURL, EXMLHttpRequestMethod.GET );

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData );
    }

    export function getListOfLikedObjects( offset?: string, limit: number = 10 ): Promise<IEVAServiceResponse<EVA.Babylon.SocialListLikedObjects, EVA.Babylon.SocialListLikedObjectsResponse>>
    {
        // Create the service instance
        //
        // var callURL: string = `/api/v1/social/user/profile/liked`;
        // var service = new EVAService<EVA.Babylon.SocialListLikedObjects, EVA.Babylon.SocialListLikedObjectsResponse>( "SocialListLikedObjects", callURL, EXMLHttpRequestMethod.GET );

        var service = new EVAService<EVA.Babylon.SocialListLikedObjects, EVA.Babylon.SocialListLikedObjectsResponse>( "SocialListLikedObjects" );

        // Build the request data
        // The base class will add things like SessionID so keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: any = {
            PageConfig: {}
        };

        if ( offset )
        {
            requestData.PageConfig.Offset = offset;
        }
        if ( limit )
        {
            requestData.PageConfig.Limit = limit;
        }

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData );
    }

    export function getListOfProgressions( userID: number, type: EVA.Social.SocialProgressionType ): Promise<IEVAServiceResponse<EVA.Social.SocialGetProgressions, EVA.Social.SocialGetProgressionsResponse>>
    {
        // Create the service instance
        //
        var service = new EVAService<EVA.Social.SocialGetProgressions, EVA.Social.SocialGetProgressionsResponse>( "SocialGetProgressions" );
        let requestData: EVA.Social.SocialGetProgressions = {
            Type: type,
            UserID: userID,
            __response: undefined
        };

        return service.call( requestData );
    }

    ////////////////////////////////////////////////////////////////////////////
    // Product
    ////////////////////////////////////////////////////////////////////////////
    //
    export function likeProductByID( productID: number ): Promise<IEVAServiceResponse<EVA.Babylon.SocialLikeProduct, EVA.Core.EmptyResponseMessage>>
    {
        return socialAction( productID, "product", "like" );
    }

    export function unlikeProductByID( productID: number ): Promise<IEVAServiceResponse<EVA.Babylon.SocialUnlikeProduct, EVA.Core.EmptyResponseMessage>>
    {
        return socialAction( productID, "product", "unlike" );
    }

    export function shareProductWithUsers( productID: number, userIDs: Array<number>, message: string )
    {
        var requestData: EVA.Babylon.ShareProductWithUsers = <EVA.Babylon.ShareProductWithUsers>{
            ProductID:  productID,
            UserIDs:    userIDs,
            Message:    message
        };

        return socialAction( productID, "product", "share", requestData );
    }

    export function productSharedExternally( productID: number, platform: EVA.Babylon.SocialMediaPlatform )
    {
        var requestData: EVA.Babylon.SocialProductSharedExternally = <EVA.Babylon.SocialProductSharedExternally>{
            ProductID:  productID,
            Platform:   platform
        };

        return socialAction( productID, "product", "share/" + platform, requestData );
    }

    export function listProductInteractionSummary( productID: number )
    {
        return socialRequest( productID, "product", "interaction/summary" );
    }

    export function listProductInteractionDetails( productID: number, limit?: number, offset?: string, type?: string )
    {
        var requestData: any = {};
        var interactionTypeFilter = "";
        if ( limit  !== undefined ) { requestData.limit  = limit;  }
        if ( offset !== undefined ) { requestData.offset = offset; }
        if ( type   !== undefined ) { interactionTypeFilter = "/" + type; }

        return socialRequest( productID, "product", `interaction/detail${interactionTypeFilter}`, requestData );
    }

    export function getProductStats( productIDs: Array<number>|number )
    {
        var service = new EVAService<EVA.Babylon.SocialGetProductsStats, EVA.Babylon.SocialStatsResponse>( "SocialGetProductsStats" );

        if ( _.isNumber( productIDs ) )
        {
            productIDs = <Array<number>>[ productIDs ];
        }

        var requestData: EVA.Babylon.SocialGetProductsStats = <EVA.Babylon.SocialGetProductsStats>{
            ProductIDs: productIDs
        };

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData );
    }

    ////////////////////////////////////////////////////////////////////////////
    // Collection
    ////////////////////////////////////////////////////////////////////////////
    //
    export function likeCollectionByID( collectionID: number ): Promise<IEVAServiceResponse<EVA.Babylon.SocialLikeFashionCollection, EVA.Core.EmptyResponseMessage>>
    {
        return socialAction( collectionID, "collection", "like" );
    }

    export function unlikeCollectionByID( collectionID: number ): Promise<IEVAServiceResponse<EVA.Babylon.SocialUnlikeFashionCollection, EVA.Core.EmptyResponseMessage>>
    {
        return socialAction( collectionID, "collection", "unlike" );
    }

    export function shareCollectionWithUsers( collectionID: number, userIDs: Array<number>, message: string )
    {
        var requestData: EVA.Babylon.ShareFashionCollectionWithUsers = <EVA.Babylon.ShareFashionCollectionWithUsers>{
            CollectionID:   collectionID,
            UserIDs:        userIDs,
            Message:        message
        };

        return socialAction( collectionID, "collection", "share", requestData );
    }

    export function collectionSharedExternally( collectionID: number, platform: EVA.Babylon.SocialMediaPlatform )
    {
        var requestData: EVA.Babylon.SocialFashionCollectionSharedExternally = <EVA.Babylon.SocialFashionCollectionSharedExternally>{
            CollectionID: collectionID,
            Platform:     platform
        };

        return socialAction( collectionID, "collection", "share/" + platform, requestData );
    }

    export function inviteFriends( invites: Array<EVA.Babylon.InviteFriendsDto>|EVA.Babylon.InviteFriendsDto ): Promise<IEVAServiceResponse<EVA.Babylon.SocialInviteFriends, EVA.Core.EmptyResponseMessage>>
    {
        var service = new EVAService<EVA.Babylon.SocialInviteFriends, EVA.Core.EmptyResponseMessage>( "SocialInviteFriends" );

        if ( !_.isArray( invites ) )
        {
            invites = <Array<EVA.Babylon.InviteFriendsDto>>[ invites ];
        }

        var requestData: EVA.Babylon.SocialInviteFriends = <EVA.Babylon.SocialInviteFriends>{
            Invites: invites
        };

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData );
    }

    export function listCollectionInteractionSummary( collectionID: number )
    {
        return socialRequest( collectionID, "collection", "interaction/summary" );
    }

    export function listCollectionInteractionDetails( collectionID: number, limit?: number, offset?: string, filter?: string|number )
    {
        let requestData: any = {};
        if ( limit  !== undefined ) { requestData.limit  = limit;  }
        if ( offset !== undefined ) { requestData.offset = offset; }

        let interactionType = "interaction/detail";
        if ( filter )
        {
            switch ( filter )
            {
                case "like":
                case "likes":
                case "liked":
                {
                    filter = 1;
                } break;

                case "comment":
                case "comments":
                case "commented":
                {
                    filter = 2;
                } break;

                case "clip":
                case "clips":
                case "clipped":
                {
                    filter = 3;
                } break;

                default:
                {
                    // NOOP
                } break;
            }
            interactionType += `/${filter}`;
        }

        return socialRequest( collectionID, "collection", interactionType, requestData );
    }

    export function getCollectionStats( collectionIDs: Array<number>|number )
    {
        var service = new EVAService<EVA.Babylon.SocialGetFashionCollectionsStats, EVA.Babylon.SocialStatsResponse>( "SocialGetFashionCollectionsStats" );

        if ( _.isNumber( collectionIDs ) )
        {
            collectionIDs = <Array<number>>[ collectionIDs ];
        }

        var requestData: EVA.Babylon.SocialGetFashionCollectionsStats = <EVA.Babylon.SocialGetFashionCollectionsStats>{
            CollectionIDs: collectionIDs
        };

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData );
    }

    export function getCollectionStatsOverTime( collectionID: number, granularity: EVA.Babylon.CollectionStatsGranularity )
    {
        var service = new EVAService<EVA.Babylon.GetFashionCollectionStatsOverTime, EVA.Babylon.SocialStatsResponse>( "GetFashionCollectionStatsOverTime" );

        var requestData: EVA.Babylon.GetFashionCollectionStatsOverTime = <EVA.Babylon.GetFashionCollectionStatsOverTime>{
            CollectionID: collectionID,
            Granularity:  granularity
        };

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData );
    }

    ////////////////////////////////////////////////////////////////////////////
    // User
    ////////////////////////////////////////////////////////////////////////////
    //
    export function likeUserByID( userID: number ): Promise<IEVAServiceResponse<EVA.Babylon.SocialLikeUser, EVA.Core.EmptyResponseMessage>>
    {
        return socialAction( userID, "user", "like" );
    }

    export function unlikeUserByID( userID: number ): Promise<IEVAServiceResponse<EVA.Babylon.SocialUnlikeUser, EVA.Core.EmptyResponseMessage>>
    {
        return socialAction( userID, "user", "unlike" );
    }

    export function followUserByID( userID: number ): Promise<IEVAServiceResponse<EVA.Babylon.SocialFollowUser, EVA.Core.EmptyResponseMessage>>
    {
        return socialAction( userID, "user", "follow" );
    }

    export function unfollowUserByID( userID: number ): Promise<IEVAServiceResponse<EVA.Babylon.SocialUnfollowUser, EVA.Core.EmptyResponseMessage>>
    {
        return socialAction( userID, "user", "unfollow" );
    }

    export function getAvatarURL( userID: number, size = 100, format = "png" )
    {
        return SettingsManager.getEndPointURL() + `/user/avatar/${size}/${userID}.${format}`;
    }

    export function isFollowingUsers( userIDs: Array<number> )
    {
        // Create the service instance
        //
        var callURL: string = "/api/v1/social/user/following";
        var service = new EVAService<EVA.Babylon.SocialIsFollowingUsers, EVA.Babylon.SocialIsFollowingUsersResponse>( "SocialIsFollowingUsers", callURL, EXMLHttpRequestMethod.POST );

        // Build the request data
        // The base class will add things like SessionID so keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: EVA.Babylon.SocialIsFollowingUsers  = <EVA.Babylon.SocialIsFollowingUsers> {
            UserIDs: userIDs
        };

        // Perform the call and return resulting promise
        //
        return service.call( requestData );
    }

    ////////////////////////////////////////////////////////////////////////////
    // UserMedia
    ////////////////////////////////////////////////////////////////////////////
    //
    export function likeUserMediaByID( userMediaID: number ): Promise<IEVAServiceResponse<EVA.Babylon.SocialLikeFashionUserMedia, EVA.Core.EmptyResponseMessage>>
    {
        return socialAction( userMediaID, "user_media", "like" );
    }

    export function unlikeUserMediaByID( userMediaID: number ): Promise<IEVAServiceResponse<EVA.Babylon.SocialUnlikeFashionUserMedia, EVA.Core.EmptyResponseMessage>>
    {
        return socialAction( userMediaID, "user_media", "unlike" );
    }

    ////////////////////////////////////////////////////////////////////////////
    // Comments
    ////////////////////////////////////////////////////////////////////////////
    //
    export function removeCommentByID( commentID: number )
    {
        // Create the service instance
        //
        var callURL: string = `/api/v1/social/comment/${commentID}`;
        var service = new EVAService<EVA.Babylon.SocialDeleteComment, EVA.Core.EmptyResponseMessage>( "SocialDeleteComment", callURL, EXMLHttpRequestMethod.DELETE );

        // Perform the call and capture the resulting promise
        //
        return service.call();
    }

    export function addCommentForProduct( productID: number, comment: string, mentionedUserIDs: Array<number> )
    {
        return socialAction( productID, "product", "comment", { CommentText: comment, MentionedUserIDs: mentionedUserIDs } );
    }

    export function addCommentForCollection( collectionID: number, comment: string )
    {
        return socialAction( collectionID, "collection", "comment", { CommentText: comment } );
    }

    export function addCommentForUserMedia( userMediaID: number, comment: string )
    {
        return socialAction( userMediaID, "user_media", "comment", { CommentText: comment } );
    }

    export function likeCommentByID( commentID: number ): Promise<IEVAServiceResponse<EVA.Babylon.SocialLikeComment, EVA.Core.EmptyResponseMessage>>
    {
        return socialAction( commentID, "comment", "like" );
    }

    export function unlikeCommentByID( commentID: number ): Promise<IEVAServiceResponse<EVA.Babylon.SocialUnlikeComment, EVA.Core.EmptyResponseMessage>>
    {
        return socialAction( commentID, "comment", "unlike" );
    }

    ////////////////////////////////////////////////////////////////////////////
    // Stats
    ////////////////////////////////////////////////////////////////////////////
    //
    export function getStatsForFeedObjects( objects: Array<EVA.Babylon.FeedObjectDto> )
    {
        // Create the service instance
        //
        var callURL: string = "/api/v1/social/feed/stats";
        var service = new EVAService<EVA.Babylon.SocialGetStatsForFeedObjects, EVA.Babylon.SocialObjectStatsResponse>( "SocialGetStatsForFeedObjects", callURL, EXMLHttpRequestMethod.POST );

        // Build the request data
        // The base class will add things like SessionID so keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: EVA.Babylon.SocialGetStatsForFeedObjects  = <EVA.Babylon.SocialGetStatsForFeedObjects> {
            ObjectIDs: objects
        };

        // Perform the call and return resulting promise
        //
        return service.call( requestData );
    }

    ////////////////////////////////////////////////////////////////////////////
    // Abuse
    ////////////////////////////////////////////////////////////////////////////
    //
    export function blockUserByID( userID: number): Promise<IEVAServiceResponse<EVA.Babylon.BlockUser, EVA.Core.EmptyResponseMessage>>
    {
        // Create the service instance
        //
        var callURL: string = "/api/v1/social/user/block_list";
        var service = new EVAService<EVA.Babylon.BlockUser, EVA.Core.EmptyResponseMessage>( "BlockUser", callURL, EXMLHttpRequestMethod.POST );

        // Build the request data
        // The base class will add things like SessionID so keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: EVA.Babylon.BlockUser  = <EVA.Babylon.BlockUser> {
            UserID: userID
        };

        // Perform the call and return resulting promise
        //
        return service.call( requestData );
    }

    export function unblockUserByID( userID: number): Promise<IEVAServiceResponse<EVA.Babylon.UnblockUser, EVA.Core.EmptyResponseMessage>>
    {
        // Create the service instance
        //
        var callURL: string = "/api/v1/social/user/block_list/" + userID;
        var service = new EVAService<EVA.Babylon.UnblockUser, EVA.Core.EmptyResponseMessage>( "UnblockUser", callURL, EXMLHttpRequestMethod.DELETE );

        // Build the request data
        // The base class will add things like SessionID so keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: EVA.Babylon.UnblockUser  = <EVA.Babylon.UnblockUser> {
        };

        // Perform the call and return resulting promise
        //
        return service.call( requestData );
    }

    export function reportContent( contentType: EVA.Babylon.SocialReportContentTypes, contentID: string, reason: EVA.Babylon.SocialReportReasons, description?: string )
    {
        // Create the service instance
        //
        var service = new EVAService<EVA.Babylon.SocialReportContent, EVA.Core.EmptyResponseMessage>( "SocialReportContent" );

        // Build the request data
        // The base class will add things like SessionID so keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: EVA.Babylon.SocialReportContent  = <EVA.Babylon.SocialReportContent> {
            Type:        contentType
        ,   Reason:      reason
        ,   ID:          contentID
        ,   Description: description
        };

        // Perform the call and return resulting promise
        //
        return service.call( requestData );
    }

    ////////////////////////////////////////////////////////////////////////////
    // Testing
    ////////////////////////////////////////////////////////////////////////////
    //
    export function sendPushNotificationTest( userId: number, title: string, body: string, data?: any )
    {
        // Create the service instance
        //
        var service = new EVAService<EVA.Babylon.BabylonPushNotificationTest, EVA.Core.EmptyResponseMessage>( "BabylonPushNotificationTest" );

        // Build the request data
        // The base class will add things like SessionID so keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: EVA.Babylon.BabylonPushNotificationTest  = <EVA.Babylon.BabylonPushNotificationTest> {
            UserID:      userId
        ,   Title:       title
        ,   Body:        body
        ,   Data:        data
        };

        // Perform the call and return resulting promise
        //
        return service.call( requestData );
    }
}

export class SocialProfile extends FetchableItem<EVA.Babylon.SocialUserProfile, EVA.Babylon.SocialGetUserProfile, EVA.Babylon.SocialGetUserProfileResponse>
{
    /**
     * Sets the current item details using the fetched service response.
     * Extending classes should override this method to pull the details from response
     * @param  {RES} response The response from the service call
     * @return {T}            The item details
     */
    public setData( response: any ): EVA.Babylon.SocialUserProfile
    {
        // The default response is usually EVA.Core.GetResponse
        // An extending class will know exactly what is returned and where
        // the data for <T> can be found
        //
        return this.data = <EVA.Babylon.SocialUserProfile>response.Profile;
    }

    /**
     * Fetches the social user profile details
     * @return {Promise<IEVAServiceResponse<EVA.Babylon.SocialUserProfile, EVA.Babylon.SocialGetUserProfile, EVA.Babylon.SocialGetUserProfileResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Babylon.SocialGetUserProfile, EVA.Babylon.SocialGetUserProfileResponse>>
    {
        if ( this.getId() === undefined )
        {
            return Promise.reject( {
                status:     404,
                response:   "Not found"
            } );
        }
        else
        {
            // Build the request data
            //
            var requestData: EVA.Babylon.SocialGetUserProfile = <EVA.Babylon.SocialGetUserProfile>{
                UserID: this.getId()
            };

            return super.fetch( "SocialGetUserProfile", requestData, "/api/v1/social/user/" + this.getId() + "/profile", EXMLHttpRequestMethod.GET );
        }
    }

    /**
     * Fetches the social user profile details by nickname
     * @return {Promise<IEVAServiceResponse<EVA.Babylon.SocialUserProfile, EVA.Babylon.SocialGetUserProfile, EVA.Babylon.SocialGetUserProfileResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetchByNickname( nickname: string ): Promise<IEVAServiceResponse<EVA.Babylon.SocialGetUserProfileByNickname, EVA.Babylon.SocialGetUserProfileResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Build the request data
            //
            var requestData: EVA.Babylon.SocialGetUserProfileByNickname = <EVA.Babylon.SocialGetUserProfileByNickname>{
                Nickname: nickname
            };

            // Perform the call and capture the resulting promise
            //
            var service = new EVAService<EVA.Babylon.SocialGetUserProfileByNickname, EVA.Babylon.SocialGetUserProfileResponse>( "SocialGetUserProfileByNickname" );
            service.call( requestData )
            .then( ( fetchResult: IEVAServiceResponse<EVA.Babylon.SocialGetUserProfileByNickname, EVA.Babylon.SocialGetUserProfileResponse> ) =>
            {
                this.id = fetchResult.response.Profile.UserID;
                this.setData( fetchResult.response );

                resolve( fetchResult );
            } )
            .catch( reject );
        } );
    }

    public update( userProfile: EVA.Babylon.SocialUpdateUserProfileModel ): Promise<IEVAServiceResponse<EVA.Babylon.SocialGetUserProfile, EVA.Babylon.SocialGetUserProfileResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            if ( this.getId() === undefined )
            {
                reject( {
                    status:     404,
                    response:   "Not found"
                } );
            }
            else
            {
                // Update the user profile
                //
                var service = new EVAService<EVA.Babylon.SocialUpdateUserProfile, EVA.Core.EmptyResponseMessage>( "SocialUpdateUserProfile", "/api/v1/social/user/" + this.getId() + "/profile", EXMLHttpRequestMethod.PATCH );

                // Build the request data
                // The base class will add things like SessionID so keep typescript
                // happy we're using an 'any' type object here
                //
                var requestData: any = {
                    Profile: userProfile
                };

                // Perform the call and capture the resulting promise
                //
                service.call( requestData )
                .then( ( updateResult: IEVAServiceResponse<EVA.Babylon.SocialUpdateUserProfile, EVA.Core.EmptyResponseMessage> ) =>
                {
                    // Return the fetch of the updated user profile
                    //
                    this.fetch().then( resolve ).catch( reject );
                } )
                .catch( reject );
            }
        } );
    }
}

export class SocialFollowers extends FetchablePagedList<EVA.Babylon.Follower, EVA.Babylon.SocialListFollowers, EVA.Babylon.SocialListFollowersResponse>
{
    /**
     * The user to retrieve the follower list for
     * @type {number}
     */
    private userID: number;

    /**
     * Sets the user id to retrieve the follower list for
     * @param  {number} userID      The user identifier
     * @return {void}               This method doesn't return anything
     */
    public setUserID( userID: number )
    {
        this.userID = userID;
    }

    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Babylon.SocialListFollowers, EVA.Babylon.SocialListFollowersResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Babylon.SocialListFollowers, EVA.Babylon.SocialListFollowersResponse>> {

        // Build the request data
        //
        var requestData: EVA.Babylon.SocialListFollowers = <EVA.Babylon.SocialListFollowers>{};

        // Apply PageConfig parameters directly
        //
        _.assign( requestData, _.omit( this.getPageConfiguration(), [ "Filter" ] ) );

        return super.fetch( "SocialListFollowers", requestData, `/api/v1/social/user/${this.userID}/followers`, EXMLHttpRequestMethod.GET );
    }
}

export class SocialFollowing extends FetchablePagedList<EVA.Babylon.Follower, EVA.Babylon.SocialListFollowing, EVA.Babylon.SocialListFollowingResponse>
{
    /**
     * The user to retrieve the following list for
     * @type {number}
     */
    private userID: number;

    /**
     * Sets the user id to retrieve the following list for
     * @param  {number} userID      The user identifier
     * @return {void}               This method doesn't return anything
     */
    public setUserID( userID: number )
    {
        this.userID = userID;
    }

    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Babylon.Social.Services.SocialListFollowing, EVA.Babylon.Social.Services.SocialListFollowingResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Babylon.SocialListFollowing, EVA.Babylon.SocialListFollowingResponse>> {

        // Build the request data
        //
        var requestData: EVA.Babylon.SocialListFollowing = <EVA.Babylon.SocialListFollowing>{};

        // Apply PageConfig parameters directly
        //
        _.assign( requestData, _.omit( this.getPageConfiguration(), [ "Filter" ] ) );

        return super.fetch( "SocialListFollowing", requestData, `/api/v1/social/user/${this.userID}/following`, EXMLHttpRequestMethod.GET );
    }
}

export class SocialBlockedUsers extends FetchablePagedList<EVA.Babylon.BlockUser, EVA.Babylon.SocialListBlockedFollowers, EVA.Babylon.SocialListBlockedFollowersResponse>
{
    /**
     * The user to retrieve the following list for
     * @type {number}
     */
    private userID: number;

    /**
     * Sets the user id to retrieve the following list for
     * @param  {number} userID      The user identifier
     * @return {void}               This method doesn't return anything
     */
    public setUserID( userID: number )
    {
        this.userID = userID;
    }

    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Babylon.Social.Services.SocialListFollowing, EVA.Babylon.Social.Services.SocialListFollowingResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Babylon.SocialListBlockedFollowers, EVA.Babylon.SocialListBlockedFollowersResponse>> {

        // Build the request data
        //
        var requestData: EVA.Babylon.SocialListBlockedFollowers = <EVA.Babylon.SocialListBlockedFollowers>{};

        // Apply PageConfig parameters directly
        //
        _.assign( requestData, _.omit( this.getPageConfiguration(), [ "Filter" ] ) );

        return super.fetch( "SocialListBlockedFollowers", requestData, `/api/v1/social/user/block_list`, EXMLHttpRequestMethod.GET );
    }
}

export class SocialCommentsForProduct extends FetchablePagedList<EVA.Babylon.Comment, EVA.Babylon.SocialListCommentsForProduct, EVA.Babylon.SocialListCommentsResponse>
{
    /**
     * The product to retrieve the comment list for
     * @type {number}
     */
    private productID: number;

    /**
     * Sets the product id to retrieve the follower list for
     * @param  {number} productID       The product identifier
     * @return {void}                   This method doesn't return anything
     */
    public setProductID( productID: number )
    {
        this.productID = productID;
    }

    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Babylon.SocialListCommentsForProduct, EVA.Babylon.SocialListCommentsResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Babylon.SocialListCommentsForProduct, EVA.Babylon.SocialListCommentsResponse>> {

        // Build the request data
        //
        var requestData: EVA.Babylon.SocialListCommentsForProduct = <EVA.Babylon.SocialListCommentsForProduct>{};

        // Apply PageConfig parameters directly
        //
        _.assign( requestData, _.omit( this.getPageConfiguration(), [ "Filter" ] ) );

        return super.fetch( "SocialListCommentsForProduct", requestData, `/api/v1/social/product/${this.productID}/comment`, EXMLHttpRequestMethod.GET );
    }
}

export class SocialCommentsForCollection extends FetchablePagedList<EVA.Babylon.Comment, EVA.Babylon.SocialListCommentsForFashionCollection, EVA.Babylon.SocialListCommentsResponse>
{
    /**
     * The collection to retrieve the comment list for
     * @type {number}
     */
    private collectionID: number;

    /**
     * Sets the collection id to retrieve the comment list for
     * @param  {number} collectionID        The collection identifier
     * @return {void}                       This method doesn't return anything
     */
    public setCollectionID( collectionID: number )
    {
        this.collectionID = collectionID;
    }

    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Babylon.SocialListCommentsForFashionCollection, EVA.Babylon.SocialListCommentsResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Babylon.SocialListCommentsForFashionCollection, EVA.Babylon.SocialListCommentsResponse>> {

        // Build the request data
        //
        var requestData: EVA.Babylon.SocialListCommentsForFashionCollection = <EVA.Babylon.SocialListCommentsForFashionCollection>{};

        // Apply PageConfig parameters directly
        //
        _.assign( requestData, _.omit( this.getPageConfiguration(), [ "Filter" ] ) );

        return super.fetch( "SocialListCommentsForFashionCollection", requestData, `/api/v1/social/collection/${this.collectionID}/comment`, EXMLHttpRequestMethod.GET );
    }
}

export class SocialCommentsForUserMedia extends FetchablePagedList<EVA.Babylon.Comment, EVA.Babylon.SocialListCommentsForFashionUserMedia, EVA.Babylon.SocialListCommentsResponse>
{
    /**
     * The user media to retrieve the comment list for
     * @type {number}
     */
    private userMediaID: number;

    /**
     * Sets the user media id to retrieve the follower list for
     * @param  {number} userMediaID      The user identifier
     * @return {void}               This method doesn't return anything
     */
    public setUserMediaID( userMediaID: number )
    {
        this.userMediaID = userMediaID;
    }

    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Babylon.SocialListCommentsForFashionUserMedia, EVA.Babylon.SocialListCommentsResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Babylon.SocialListCommentsForFashionUserMedia, EVA.Babylon.SocialListCommentsResponse>> {

        // Build the request data
        //
        var requestData: EVA.Babylon.SocialListCommentsForFashionUserMedia = <EVA.Babylon.SocialListCommentsForFashionUserMedia>{};

        // Apply PageConfig parameters directly
        //
        _.assign( requestData, _.omit( this.getPageConfiguration(), [ "Filter" ] ) );

        return super.fetch( "SocialListCommentsForFashionUserMedia", requestData, `/api/v1/social/user_media/${this.userMediaID}/comment`, EXMLHttpRequestMethod.GET );
    }
}

export class SocialProductsCreatedByUser extends FetchablePagedList<EVA.Babylon.ProductCreationByUserResult, EVA.Babylon.SocialListProductsCreationsByUser, EVA.Babylon.SocialListProductsCreationsByUserResponse>
{
    /**
     * The user to retrieve the creations list for
     * @type {number}
     */
    private userID: number;

    /**
     * Sets the user id to retrieve the creations list for
     * @param  {number} userID      The user identifier
     * @return {void}               This method doesn't return anything
     */
    public setUserID( userID: number )
    {
        this.userID = userID;
    }

    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Babylon.SocialListProductsCreationsByUser, EVA.Babylon.SocialListProductsCreationsByUserResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Babylon.SocialListProductsCreationsByUser, EVA.Babylon.SocialListProductsCreationsByUserResponse>> {

        // Build the request data
        //
        var requestData: EVA.Babylon.SocialListProductsCreationsByUser = <EVA.Babylon.SocialListProductsCreationsByUser>{
            PageConfig: this.getPageConfiguration()
        ,   UserID:     this.userID
        };

        return super.fetch( "SocialListProductsCreationsByUser", requestData );
    }
}

export class SocialFacebookFriends extends FetchablePagedList<EVA.Babylon.FacebookFriend, EVA.Babylon.SocialListFacebookFriends, EVA.Babylon.SocialListFacebookFriendsResponse>
{
    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Babylon.SocialListFacebookFriends, EVA.Babylon.SocialListFacebookFriendsResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Babylon.SocialListFacebookFriends, EVA.Babylon.SocialListFacebookFriendsResponse>> {

        // Build the request data
        //
        var requestData: EVA.Babylon.SocialListFacebookFriends = <EVA.Babylon.SocialListFacebookFriends>{
            PageConfig: this.getPageConfiguration()
        };

        return super.fetch( "SocialListFacebookFriends", requestData );
    }
}
