/**
 * The supplier module provides access to the supplier data from the EVA back-end
 * @preferred
 */
import { FetchableItem                    } from "./base";
import { EVAService, IEVAServiceResponse  } from "../service";
import * as _                               from "lodash";

/**
 * Provides access to an order available in EVA
 */
export class FashionSupplier extends FetchableItem<EVA.Babylon.FashionSupplierDataDto, EVA.Babylon.GetFashionSupplierData, EVA.Babylon.GetFashionSupplierDataResponse>
{
    /**
     * This class wraps an supplier from EVA and the operations you can perform on it.
     * You should supply the supplier identifier when creating an instance of this class
     *
     * @param  {number | string}    id      The identifier of the supplier to retrieve
     * @return {void}                       This method doesn't return anything
     */
    constructor( id?: number | string ) {
        super( id );
    }

    /**
     * Fetches the order details
     * @return {Promise<IEVAServiceResponse<EVA.Babylon.GetFashionSupplierData, EVA.Babylon.GetFashionSupplierDataResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Babylon.GetFashionSupplierData, EVA.Babylon.GetFashionSupplierDataResponse>> {
        // Build the request data
        //
        var requestOrderData: EVA.Babylon.GetFashionSupplierData = <EVA.Babylon.GetFashionSupplierData>{
            SupplierID: this.getId()
        };

        return super.fetch( "GetFashionSupplierData", requestOrderData );
    }

    public fetchForOrder( orderId: number ): Promise<IEVAServiceResponse<EVA.Babylon.GetFashionSupplierDataForOrder, EVA.Babylon.GetFashionSupplierDataForOrderResponse>> {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            var service = new EVAService<EVA.Babylon.GetFashionSupplierDataForOrder, EVA.Babylon.GetFashionSupplierDataForOrderResponse>( "GetFashionSupplierDataForOrder" );

            var requestData: any = {
                OrderID: orderId
            };

            service.call( requestData )
            .then( ( result ) => {

                // Populate supplier data from response
                //
                this.setData( result.response );

                resolve( result );
            } )
            .catch( reject );
        } );
     }

    /**
     * Sets the current item details using the fetched service response.
     * Extending classes should override this method to pull the details from response
     * @param  {EVA.Babylon.GetFashionSupplierDataResponse}  response    The response from the service call
     * @return {EVA.Babylon.FashionSupplierDataDto}                      The supplier details
     */
    public setData( response: any ): EVA.Babylon.FashionSupplierDataDto
    {
        // Set the supplier data
        //
        this.data = response.SupplierData;

        return this.data;
    }
}
