/**
 * The applications module provides access to the applications data from the EVA back-end
 * @preferred
 */

import { FetchablePagedList                } from "./base";
import { EVAService, IEVAServiceResponse   } from "../service";
import { EXMLHttpRequestMethod             } from "../xhr";
import * as _                                from "lodash";

/**
 * Provides access to user media (fashion) in EVA
 */
export namespace UserMedia
{
    "use strict";

    export function removeUserMediaByID( userMediaID: number )
    {
        // Create the service instance
        //
        var callURL: string = `/api/v1/fashion/user_media/${userMediaID}`;
        var service = new EVAService<EVA.Babylon.DeleteFashionUserMedia, EVA.Core.EmptyResponseMessage>( "DeleteFashionUserMedia", callURL, EXMLHttpRequestMethod.DELETE );

        // Perform the call and capture the resulting promise
        //
        return service.call();
    }

    export function addUserMediaToProduct( productID: number, blobID: string )
    {
        // Create the service instance
        //
        var callURL: string = `/api/v1/fashion/product/${productID}/user_media`;
        var service = new EVAService<any, EVA.Core.EmptyResponseMessage>( "SocialAction", callURL, EXMLHttpRequestMethod.POST );

        var requestData: EVA.Babylon.AddFashionUserMediaToProduct = <EVA.Babylon.AddFashionUserMediaToProduct>{
            ProductID:  productID,
            BlobID:     blobID
        };

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData );
    }

    export function addUserMediaToCollection( collectionID: number, blobID: string )
    {
        // Create the service instance
        //
        var callURL: string = `/api/v1/fashion/collection/${collectionID}/user_media`;
        var service = new EVAService<any, EVA.Core.EmptyResponseMessage>( "SocialAction", callURL, EXMLHttpRequestMethod.POST );

        var requestData: EVA.Babylon.AddFashionUserMediaToFashionCollection = <EVA.Babylon.AddFashionUserMediaToFashionCollection>{
            CollectionID:   collectionID,
            BlobID:         blobID
        };

        // Perform the call and capture the resulting promise
        //
        return service.call( requestData );
    }
}

export class UserMediaForProduct extends FetchablePagedList<EVA.Babylon.FashionUserMedia, EVA.Babylon.ListFashionUserMediaForProduct, EVA.Babylon.ListFashionUserMediaResponse>
{
    /**
     * The product to retrieve the user media list for
     * @type {number}
     */
    private productID: number;
    private userID: number;

    /**
     * Sets the product id to retrieve the user media list for
     * @param  {number} productID           The product identifier
     * @return {void}                       This method doesn't return anything
     */
    public setProductID( productID: number )
    {
        this.productID = productID;
    }

    /**
     * Sets the user id to retrieve the user media list for a specific user
     * @param  {number} userID           The user identifier
     * @return {void}                    This method doesn't return anything
     */
    public setUserID( userID: number )
    {
        this.userID = userID;
    }

    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Babylon.ListFashionUserMediaForProduct, EVA.Babylon.ListFashionUserMediaResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Babylon.ListFashionUserMediaForProduct, EVA.Babylon.ListFashionUserMediaResponse>> {

        // Build the request data
        //
        var requestData: EVA.Babylon.ListFashionUserMediaForProduct = <EVA.Babylon.ListFashionUserMediaForProduct>{};
        var queryString = this.userID ? `?UserID=${this.userID}` : "";

        // Apply PageConfig parameters directly
        //
        _.assign( requestData, _.omit( this.getPageConfiguration(), [ "Filter" ] ) );

        return super.fetch( "ListFashionUserMediaForProduct", requestData, `/api/v1/fashion/product/${this.productID}/user_media${queryString}`, EXMLHttpRequestMethod.GET );
    }
}

export class UserMediaForCollection extends FetchablePagedList<EVA.Babylon.FashionUserMedia, EVA.Babylon.ListFashionUserMediaForCollection, EVA.Babylon.ListFashionUserMediaResponse>
{
    /**
     * The collection to retrieve the user media list for
     * @type {number}
     */
    private collectionID: number;
    private userID: number;

    /**
     * Sets the collection id to retrieve the user media list for
     * @param  {number} collectionID        The collection identifier
     * @return {void}                       This method doesn't return anything
     */
    public setCollectionID( collectionID: number )
    {
        this.collectionID = collectionID;
    }

    /**
     * Sets the user id to retrieve the user media list for a specific user
     * @param  {number} userID           The user identifier
     * @return {void}                    This method doesn't return anything
     */
    public setUserID( userID: number )
    {
        this.userID = userID;
    }

    /**
     * Fetches the list of items
     * @return {Promise<IEVAServiceResponse<EVA.Babylon.ListFashionUserMediaForCollection, EVA.Babylon.ListFashionUserMediaResponse>>} A promise resolving when the fetch from the back-end has completed
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Babylon.ListFashionUserMediaForCollection, EVA.Babylon.ListFashionUserMediaResponse>> {

        // Build the request data
        //
        var requestData: EVA.Babylon.ListFashionUserMediaForCollection = <EVA.Babylon.ListFashionUserMediaForCollection>{};
        var queryString = this.userID ? `?UserID=${this.userID}` : "";

        // Apply PageConfig parameters directly
        //
        _.assign( requestData, _.omit( this.getPageConfiguration(), [ "Filter" ] ) );

        return super.fetch( "ListFashionUserMediaForCollection", requestData, `/api/v1/fashion/collection/${this.collectionID}/user_media${queryString}`, EXMLHttpRequestMethod.GET );
    }
}
