import { CustomerOrderList, Order } from "./orders";
import { FetchableItem } from "./base";
import { IEVAServiceResponse } from "../service";
export interface IEVALoginOptions {
    organizationUnitId?: number;
    confirmationToken?: string;
    identificationCode?: string;
    selectOrganizationByApplicationID?: boolean;
    publicLogin?: boolean;
    registerApiKey?: boolean;
    context?: string;
}
export interface IEVACreateCustomer {
    noAccount?: boolean;
    autoLogin?: boolean;
}
/**
 * General namespace for Users function
 */
export declare namespace Users {
    /**
     * Request a token (via e-mail) for a user
     *
     * @param  {string}  emailAddress the e-mail adres of the user to request the token for
     * @return {Promise}              A promise that is fulfilled when the EVA Core service responds
     */
    function requestResetPassword(emailAddress: string): Promise<IEVAServiceResponse<EVA.Core.Services.RequestPasswordResetToken, EVA.Core.EmptyResponseMessage>>;
    /**
     * Reset a user password using a token that was received in an e-mail
     *
     * @param  {string}  newPassword the new password
     * @param  {string}  token       the token to be used for this reset
     * @return {Promise}             A promise that is fulfilled when the EVA Core service responds
     */
    function resetPassword(newPassword: string, token: string): Promise<IEVAServiceResponse<EVA.Core.Services.ResetUserPassword, EVA.Core.Services.ResetUserPasswordResponse>>;
}
/**
 * Provides authentication to EVA
 *
 * id is authentication
 */
export declare class LoggedInUser extends FetchableItem<EVA.Core.LoggedInUserDto, EVA.Core.Services.GetCurrentUser, EVA.Core.Services.GetCurrentUserResponse> {
    orders: CustomerOrderList;
    authentication: number;
    organisationUnits: Array<EVA.Core.OrganizationUnitDto>;
    isAnonymous: boolean;
    isEmployee: boolean;
    isCustomer: boolean;
    private authenticationToken;
    static checkEmailAddressAvailability(emailAddress: string): Promise<any>;
    static checkNicknameAvailability(nickname: string): Promise<any>;
    /**
     * Register a new customer and autologin
     *
     * @param  {any}            customer EVA.Core.Services.CustomerDto but only EmailAddress
     * @return {Promise<any>}   A promise that is fulfilled when the EVA Core service responds
     */
    static register(customer: any, autoLogin?: boolean, noAccount?: boolean): Promise<any>;
    /**
     * Register a new customer with company
     *
     * @param  {EVA.Core.Services.CreateCustomerWithCompany} request    Create a customer with company request
     * @return {Promise<any>}                                           A promise that is fulfilled when the EVA Core service responds
     */
    static registerWithCompany(request: EVA.Core.Services.CreateCustomerWithCompany): Promise<boolean>;
    /**
     * Retrieve the company details for a user
     * @return {Promise<EVA.Core.CompanyDto>}
     */
    static getCompanyForUser(userId: number): Promise<EVA.Core.CompanyDto>;
    /**
     * Method to update a company that belongs to the logged-in user
     *
     * @param {EVA.Core.CompanyDto} company The company to update
     * @returns {Promise<boolean>} Promise that returns true if update was successful
     */
    static updateCompanyForUser(userId: number, company: EVA.Core.CompanyDto): Promise<boolean>;
    /**
     * Search for customers
     *
     * @param  {string}           query           The query to search on
     * @param  {number}           limit           The page size limit. Defaults to 15
     * @param  {number}           start           The item to start the list at. Defaults to 0 and is a 0-based index.
     * @return {Promise<any>}                     A promise that is fulfilled when the EVA Core service responds
     */
    static searchUsers(query: string, limit?: number, start?: number): Promise<any>;
    /**
     * Retrieves and updates current user data
     * @return {Promise} A promise that is fulfilled when the EVA Core service responds
     */
    fetch(): Promise<IEVAServiceResponse<EVA.Core.Services.GetCurrentUser, EVA.Core.Services.GetCurrentUserResponse>>;
    /**
     * @param  {any}                                  response Provide with body of the response to populate and set properly the data on CurrentUser
     * @return {EVA.Core.LoggedInUserDto}          The current LoggedInUserDto state
     */
    setData(response?: any): EVA.Core.LoggedInUserDto;
    /**
     * Return the current authentication token
     * @return {string} The current authentication token
     */
    getAuthenticationToken(): string;
    /**
     * Perform a login using email adress and password, and optional additional
     * data
     *
     * @param  {string}  login        The email address or username (nickname) to login with
     * @param  {string}  password     The password over the login
     * @param  {IEVALoginOptions?} options An optional structure with additional data for the login
     * @return {Promise}              Will resolve when succesfully logged in or rejected with the authentication result (EVA.Core.AuthenticationResults) or a general error
     */
    login(login: string, password: string, options?: IEVALoginOptions): Promise<IEVAServiceResponse<EVA.Core.Services.Login, EVA.Core.Services.LoginResponse>>;
    /**
     * Execute an explicit logout
     *
     * @return {Promise} A promise resolving when the call to the back-end has completed
     */
    logout(): Promise<IEVAServiceResponse<EVA.Core.Services.Logout, EVA.Core.Services.LogoutResponse>>;
    /**
     * Retrieves third party login options
     *
     * @return {Promise} A promise resolving when the call to the back-end has completed
     */
    getThirdPartyLoginOptions(): Promise<IEVAServiceResponse<EVA.Core.Services.GetAvailableThirdPartiesForLogin, EVA.Core.Services.GetAvailableThirdPartiesForLoginResponse>>;
    /**
     * Requests third party login.
     * Response will contain a redirect url that can be used to perform the login
     *
     * @return {Promise} A promise resolving when the call to the back-end has completed
     */
    requestThirdPartyLogin(loginRequest: EVA.Core.Services.RequestThirdPartyLogin): Promise<IEVAServiceResponse<EVA.Core.Services.RequestThirdPartyLogin, EVA.Core.Services.RequestThirdPartyLoginResponse>>;
    /**
     * Perform third party login with a token
     *
     * @return {Promise} A promise resolving when the call to the back-end has completed
     */
    authenticateWithThirdPartyLogin(loginRequest: EVA.Core.Services.AuthenticateWithThirdPartyLogin): Promise<IEVAServiceResponse<EVA.Core.Services.AuthenticateWithThirdPartyLogin, EVA.Core.Services.AuthenticateWithThirdPartyLoginResponse>>;
    /**
     * This call retrieves a paged list of orders belonging to the current user
     *
     * @param  {number}                         start           The item to start the list at. Defaults to 0 and is a 0-based index.
     * @param  {number}                         limit           The page size limit. Defaults to 15
     * @param  {EVA.Core.SortDirection}  sortDirection          The sorting direction for the list. Defaults to ascending
     * @return {Promise<customerOrderList>}                     A promise resolving when the call to the back-end has completed
     */
    getOrders(limit?: number, start?: number, sortDirection?: EVA.Core.SortDirection): Promise<CustomerOrderList>;
    /**
     * This call retrieves a single order
     * @param {number}              order_id                    The ID of the order
     * @param {boolean}             include_failed_payements    If set to true, return order even if the payment has not been completed
     * @return {Promies<Order>}                                 A promise resolving when the call to the back-end has completed
     */
    getOrder(order_id: number, include_failed_payements: boolean): Promise<Order>;
    /**
     * Checks if the user has a newsletter subscription
     * @return {boolean} Indicates if the user has a newsletter subscription
     */
    hasNewsletterSubscription(): boolean;
    /**
     * Subscribes to the Newsletter using the users current email address
     * @return {Promise}    A promise resolving when the call to the back-end has completed
     */
    subscribeToNewsletter(): Promise<IEVAServiceResponse<EVA.Core.Services.Subscribe, EVA.Core.Services.SubscribeResponse>>;
    /**
     * Unsubscribes from the Newsletter using the users current email address
     * @return {Promise}    A promise resolving when the call to the back-end has completed
     */
    unsubscribeFromNewsletter(): Promise<IEVAServiceResponse<EVA.Core.Services.Unsubscribe, EVA.Core.Services.UnsubscribeResponse>>;
    /**
     * Confirms a Newsletter subscription using the provided token
     * @param  {string}     token   The subscription confirmation token
     * @return {Promise}            A promise resolving when the call to the back-end has completed
     */
    confirmNewsletterSubscription(token: string): Promise<IEVAServiceResponse<EVA.Core.Services.ConfirmSubscribe, EVA.Core.Services.ConfirmSubscribeResponse>>;
    /**
     * Checks if the user has an SMS notification subscription
     * @return {boolean} Indicates if the user has an SMS notification subscription
     */
    hasSmsNotificationSubscription(): boolean;
    /**
     * Subscribes to the SMS Notifications using the users current email address
     * @return {Promise}    A promise resolving when the call to the back-end has completed
     */
    subscribeToSMSNotifications(): Promise<IEVAServiceResponse<EVA.Core.Services.Subscribe, EVA.Core.Services.SubscribeResponse>>;
    /**
     * Unsubscribes from the SMS Notifications using the users current email address
     * @return {Promise}    A promise resolving when the call to the back-end has completed
     */
    unsubscribeFromSMSNotifications(): Promise<IEVAServiceResponse<EVA.Core.Services.Unsubscribe, EVA.Core.Services.UnsubscribeResponse>>;
    /**
     * Update the user
     *
     * @return {Promise} A promise resolving when the call to the back-end has completed
     */
    update(): Promise<IEVAServiceResponse<EVA.Core.Services.UpdateUser, EVA.Core.Services.UpdateUserResponse>>;
    /**
     * Update the user's email address
     *
     * @return {Promise} A promise resolving when the call to the back-end has completed
     */
    updateUserEmailAddress(newEmailAddress: string): Promise<IEVAServiceResponse<EVA.Core.Services.UpdateUserEmailAddress, EVA.Core.RequestMessageWithEmptyResponse>>;
    /**
     * Change the passowrd of this user
     *
     * @param  {string}  newPassword The new password
     * @param  {string}  oldPassword The current / old password
     * @return {Promise}             A promise resolving when the call to the back-end has completed
     */
    changePassword(newPassword: string, oldPassword: string): Promise<IEVAServiceResponse<EVA.Core.Services.ChangeUserPassword, EVA.Core.Services.ChangeUserPasswordResponse>>;
}
export declare class User extends FetchableItem<EVA.Core.UserDto, EVA.Core.Services.GetUser, EVA.Core.Services.GetUserResponse> {
    isAnonymous: boolean;
    isEmployee: boolean;
    isCustomer: boolean;
    /**
     * Retrieves specific user data
     * @return {Promise} A promise that is fulfilled when the EVA Core service responds
     */
    fetch(): Promise<IEVAServiceResponse<EVA.Core.Services.GetUser, EVA.Core.Services.GetUserResponse>>;
    /**
     * @param  {any}                                  response  Service response containing the user data
     * @return {EVA.Core.UserDto}                   The user data
     */
    setData(response?: any): EVA.Core.UserDto;
}
