import { CustomerOrderList, Order           } from "./orders";
import { FetchableItem                      } from "./base";
import { EVAService, IEVAServiceResponse    } from "../service";
import { SettingsManager                    } from "../settings";
import * as _                                 from "lodash";

export interface IEVALoginOptions {
    organizationUnitId?: number;
    confirmationToken?: string;
    identificationCode?: string;
    selectOrganizationByApplicationID?: boolean;
    publicLogin?: boolean;
    registerApiKey?: boolean;
    context?: string;
}

export interface IEVACreateCustomer {
    noAccount?: boolean;
    autoLogin?: boolean;
}

/**
 * General namespace for Users function
 */
export namespace Users
{
    "use strict";

    /**
     * Request a token (via e-mail) for a user
     *
     * @param  {string}  emailAddress the e-mail adres of the user to request the token for
     * @return {Promise}              A promise that is fulfilled when the EVA Core service responds
     */
    export function requestResetPassword( emailAddress: string ): Promise<IEVAServiceResponse<EVA.Core.Services.RequestPasswordResetToken, EVA.Core.EmptyResponseMessage>>
    {
        var service = new EVAService<EVA.Core.Services.RequestPasswordResetToken, EVA.Core.EmptyResponseMessage>( "RequestPasswordResetToken" );

        // Build the request data
        // The base class will add things like SessionID so keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: any = {
            EmailAddress:   emailAddress
        };

        return service.call( requestData );
    }

    /**
     * Reset a user password using a token that was received in an e-mail
     *
     * @param  {string}  newPassword the new password
     * @param  {string}  token       the token to be used for this reset
     * @return {Promise}             A promise that is fulfilled when the EVA Core service responds
     */
    export function resetPassword( newPassword: string, token: string ): Promise<IEVAServiceResponse<EVA.Core.Services.ResetUserPassword, EVA.Core.Services.ResetUserPasswordResponse>>
    {
        var service = new EVAService<EVA.Core.Services.ResetUserPassword, EVA.Core.Services.ResetUserPasswordResponse>( "ResetUserPassword" );

        // Build the request data
        // The base class will add things like SessionID so keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: any = {
            NewPassword:    newPassword,
            Token:          token
        };

        return service.call( requestData );
    }
}

/**
 * Create a customer
 *
 * @param  {EVA.Core.Services.CustomerDto} customer A customer Dto object containing the customer details
 * @param  {IEVACreateCustomer}                  options  Creation options for the user account
 * @return {Promise<any>}                                 A promise that is fulfilled when the EVA Core service responds
 */
function createCustomer( customer: EVA.Core.Services.CustomerDto, options?: IEVACreateCustomer ): Promise<any>
{
    /* tslint:disable */
    "use strict";
    /* tslint:enable */

    return new Promise( ( resolve: any, reject: any ) =>
    {
        var service = new EVAService<EVA.Core.Services.CreateCustomer, EVA.Core.Services.CreateCustomerResponse>( "CreateCustomer" );

        // Build the request data
        // The base class will add things like SessionID so keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: any = {
            User:   customer
        };

        if ( options ) {
            requestData.NoAccount = options.noAccount;
            requestData.AutoLogin = options.autoLogin;
        }

        service.call( requestData )
        .then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.CreateCustomer, EVA.Core.Services.CreateCustomerResponse> ) =>
        {
            // console.log( "[CREATECUSTOMER] Response", serviceResult );

            // EVA.Core.Services.CreateCustomerResults
            //  * CreatedCustomer = 0
            //  * InvalidEmail = 1
            //  * UsedIncompleteAccount = 2
            //  * EmailAlreadyInUse = 3
            //
            if ( serviceResult.response.Result === 0 || serviceResult.response.Result === 2 )
            {
                resolve( serviceResult.response.User );
            }
            else
            {
                reject( serviceResult.response.Result );
            }
        } )
        .catch( ( error: any ) =>
        {
            // console.error( "[CREATECUSTOMER] Error", error );
            reject( error );
        } );
    } );
}

/**
 * Create a customer together with a company
 *
 * @param  {EVA.Core.Services.CreateCustomerWithCompany} request A create customer with company dto object
 * @return {Promise<any>}                                        A promise that is fulfilled when the EVA Core service responds
 */
function createCustomerWithCompany( requestData: EVA.Core.Services.CreateCustomerWithCompany ): Promise<boolean> {
    /* tslint:disable */
    "use strict";
    /* tslint:enable */

    return new Promise( ( resolve: any, reject: any ) =>
    {
        var service = new EVAService<EVA.Core.Services.CreateCustomerWithCompany, EVA.Core.RequestMessageWithEmptyResponse>( "CreateCustomerWithCompany" );

        service.call( requestData )
        .then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.CreateCustomerWithCompany, EVA.Core.RequestMessageWithEmptyResponse> ) =>
        {
            resolve( true );
        } )
        .catch( ( error: any ) =>
        {
            reject( error );
        } );
    } );
}

/**
 * Provides authentication to EVA
 *
 * id is authentication
 */
export class LoggedInUser extends FetchableItem<EVA.Core.LoggedInUserDto, EVA.Core.Services.GetCurrentUser, EVA.Core.Services.GetCurrentUserResponse>
{
    public orders:                          CustomerOrderList;
    public authentication:                  number;
    public organisationUnits:               Array<EVA.Core.OrganizationUnitDto>;
    public isAnonymous:                     boolean;
    public isEmployee:                      boolean;
    public isCustomer:                      boolean;

    private authenticationToken:            string;

    public static checkEmailAddressAvailability( emailAddress: string ): Promise<any>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            var service = new EVAService<EVA.Core.Services.CheckEmailAddressAvailability, EVA.Core.Services.CheckEmailAddressAvailabilityResponse>( "CheckEmailAddressAvailability" );

            // Build the request data
            // The base class will add things like SessionID so keep typescript
            // happy we're using an 'any' type object here
            //
            var requestData: any = {
                EmailAddress: emailAddress
            };

            service.call( requestData )
            .then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.CheckEmailAddressAvailability, EVA.Core.Services.CheckEmailAddressAvailabilityResponse> ) =>
            {
                if ( !serviceResult.response.IsAvailable )
                {
                    reject( serviceResult.response );
                }
                else
                {
                    resolve( serviceResult.response );
                }
            } )
            .catch( ( error: any ) =>
            {
                reject( error );
            } );
        } );
    }

    public static checkNicknameAvailability( nickname: string ): Promise<any>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            var service = new EVAService<EVA.Core.Services.CheckNicknameAvailability, EVA.Core.Services.CheckNicknameAvailabilityResponse>( "CheckNicknameAvailability" );

            // Build the request data
            // The base class will add things like SessionID so keep typescript
            // happy we're using an 'any' type object here
            //
            var requestData: any = {
                Nickname: nickname
            };

            service.call( requestData )
            .then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.CheckNicknameAvailability, EVA.Core.Services.CheckNicknameAvailabilityResponse> ) =>
            {
                if ( !serviceResult.response.IsAvailable )
                {
                    reject( serviceResult.response );
                }
                else
                {
                    resolve( serviceResult.response );
                }
            } )
            .catch( ( error: any ) =>
            {
                reject( error );
            } );
        } );
    }


    /**
     * Register a new customer and autologin
     *
     * @param  {any}            customer EVA.Core.Services.CustomerDto but only EmailAddress
     * @return {Promise<any>}   A promise that is fulfilled when the EVA Core service responds
     */
    public static register( customer: any, autoLogin: boolean = true, noAccount: boolean = false ): Promise<any>
    {
        /* tslint:disable */
        "use strict";
        /* tslint:enable */

        let options = {
            autoLogin: autoLogin,
            noAccount: noAccount
        };

        return new Promise( ( resolve: any, reject: any ) =>
        {
            createCustomer( customer, options )
                .then( ( user: EVA.Core.LoggedInUserDto ) =>
                {
                    resolve( {
                        AuthenticationToken:    user.AuthenticationToken
                    ,   User:                   user
                    } );
                } )
                .catch( reject );
        } );
    }


    /**
     * Register a new customer with company
     *
     * @param  {EVA.Core.Services.CreateCustomerWithCompany} request    Create a customer with company request
     * @return {Promise<any>}                                           A promise that is fulfilled when the EVA Core service responds
     */
    public static registerWithCompany( request: EVA.Core.Services.CreateCustomerWithCompany ): Promise<boolean>
    {
        /* tslint:disable */
        "use strict";
        /* tslint:enable */

        return new Promise( ( resolve: any, reject: any ) =>
        {
            createCustomerWithCompany( request )
                .then( ( success: boolean ) =>
                {
                    resolve( success );
                } )
                .catch( reject );
        } );
    }

    /**
     * Retrieve the company details for a user
     * @return {Promise<EVA.Core.CompanyDto>}
     */
    public static getCompanyForUser( userId: number ): Promise<EVA.Core.CompanyDto> {

        return new Promise( ( resolve: any, reject: any ) =>
        {
            var service = new EVAService<EVA.Core.Services.GetCompanyForUser, EVA.Core.Services.GetCompanyForUserResponse>( "GetCompanyForUser" );
            var requestData: EVA.Core.Services.GetCompanyForUser = {
                UserID: userId,
                __response: undefined
            };

            service.call( requestData )
            .then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.GetCompanyForUser, EVA.Core.Services.GetCompanyForUserResponse> ) =>
            {
                resolve( _.get( serviceResult, "response.Result" ) );
            } )
            .catch( ( error: any ) =>
            {
                reject( error );
            } );
        } );
    }

    /**
     * Method to update a company that belongs to the logged-in user
     *
     * @param {EVA.Core.CompanyDto} company The company to update
     * @returns {Promise<boolean>} Promise that returns true if update was successful
     */
    public static updateCompanyForUser( userId: number, company: EVA.Core.CompanyDto ): Promise<boolean> {

        return new Promise( ( resolve: any, reject: any ) =>
        {
            var service = new EVAService<EVA.Core.Services.UpdateCompanyForUser, EVA.Core.RequestMessageWithEmptyResponse>( "UpdateCompanyForUser" );
            var requestData: EVA.Core.Services.UpdateCompanyForUser = Object.assign( company, {
                UserID: userId
            ,   __response: undefined
            } );

            service.call( requestData )
            .then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.UpdateCompanyForUser, EVA.Core.RequestMessageWithEmptyResponse> ) =>
            {
                resolve( true );
            } )
            .catch( ( error: any ) =>
            {
                reject( error );
            } );
        } );
    }

    /**
     * Search for customers
     *
     * @param  {string}           query           The query to search on
     * @param  {number}           limit           The page size limit. Defaults to 15
     * @param  {number}           start           The item to start the list at. Defaults to 0 and is a 0-based index.
     * @return {Promise<any>}                     A promise that is fulfilled when the EVA Core service responds
     */
    public static searchUsers( query: string, limit: number = 25, start: number = 0 ): Promise<any>
    {
        /* tslint:disable */
        "use strict";
        /* tslint:enable */

        return new Promise( ( resolve: any, reject: any ) =>
        {
            var service = new EVAService<EVA.Core.Services.SearchUsers, EVA.Core.Services.SearchUsersResponse>( "SearchUsers" );

            // Build the request data
            // The base class will add things like SessionID so keep typescript
            // happy we're using an 'any' type object here
            //
            // EVA.Core.UserTypes
            //
            // None = 0,
            // Employee = 1,
            // Customer = 2,
            // Anonymous = 4,
            // Business = 8,
            // SuperUser = 17,
            // Migrated = 32,
            // Debtor = 64,
            // Incomplete = 128,
            // LimitedTrust = 256,
            // Tester = 512,
            //
            var requestData: any = {
                SearchQuery: query
            ,   Types: 2
            ,   PageConfig: {
                    Limit: limit
                ,   Start: start
                }
            };

            service.call( requestData )
            .then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.SearchUsers, EVA.Core.Services.SearchUsersResponse> ) =>
            {
                resolve( serviceResult.response );
            } )
            .catch( ( error: any ) =>
            {
                // console.error( "[SEARCH:USERS] Error", error );
                reject( error );
            } );
        } );
    }

    /**
     * Retrieves and updates current user data
     * @return {Promise} A promise that is fulfilled when the EVA Core service responds
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Core.Services.GetCurrentUser, EVA.Core.Services.GetCurrentUserResponse>>
    {
        // Build the request data
        //
        var requestData: EVA.Core.Services.GetCurrentUser = <EVA.Core.Services.GetCurrentUser>{
        };

        return super.fetch( "GetCurrentUser", requestData );
    }

    /**
     * @param  {any}                                  response Provide with body of the response to populate and set properly the data on CurrentUser
     * @return {EVA.Core.LoggedInUserDto}          The current LoggedInUserDto state
     */
    public setData( response: any = {} ): EVA.Core.LoggedInUserDto
    {
        // Set the current user data
        //
        this.data = response.User;

        if ( this.data )
        {
            this.id = this.data.ID;
        }
        else
        {
            this.id = undefined;
        }

        /**
         * EVA.Core.AuthenticationResults
         *  NotAuthenticated = 0
         *  NotAuthorized = 1
         *  Authenticated = 2
         *  NeedsTwoStepAuthentication = 3
         *  NeedsOrganizationUnitID = 4
         */
        this.authentication             = response.Authentication               !== undefined ? response.Authentication             : this.authentication;
        this.authenticationToken        = response.AuthenticationToken          !== undefined ? response.AuthenticationToken        : this.authenticationToken;
        this.organisationUnits          = response.OrganizationUnits            !== undefined ? response.OrganizationUnits          : this.organisationUnits;

        if ( response.Authentication === undefined || response.Authentication === 2 )
        {
            this.isEmployee             = this.data  && ( this.data.Type & 1 ) !== 0;
            this.isCustomer             = this.data  && ( this.data.Type & 2 ) !== 0;
            this.isAnonymous            = !this.data || ( this.data.Type & 4 ) !== 0;
        }
        else
        {
            this.isEmployee             = false;
            this.isCustomer             = false;
            this.isAnonymous            = true;
        }

        return this.data;
    }

    /**
     * Return the current authentication token
     * @return {string} The current authentication token
     */
    public getAuthenticationToken(): string
    {
        return this.authenticationToken;
    }

    /**
     * Perform a login using email adress and password, and optional additional
     * data
     *
     * @param  {string}  login        The email address or username (nickname) to login with
     * @param  {string}  password     The password over the login
     * @param  {IEVALoginOptions?} options An optional structure with additional data for the login
     * @return {Promise}              Will resolve when succesfully logged in or rejected with the authentication result (EVA.Core.AuthenticationResults) or a general error
     */
    public login( login: string, password: string, options?: IEVALoginOptions ): Promise<IEVAServiceResponse<EVA.Core.Services.Login, EVA.Core.Services.LoginResponse>>
    {
        // EVA.Core.Services.Login
        //
        var service = new EVAService<EVA.Core.Services.Login, EVA.Core.Services.LoginResponse>( "Login" );

        // Build the request data
        // The base class will add things like SessionID so keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: any = {
            ApplicationID:  SettingsManager.getApplicationId()
        };

        // EVA allows users to login using a QR code. This QR code contains
        // an identification code that is all you need to login
        //
        if ( options && options.identificationCode ) {
            requestData.IdentificationCode = options.identificationCode;
        } else {

            // Set the password
            //
            requestData.Password = password;

            // Since @'s are not allowed in usernames we can use that to distinguish
            // between email and usernames (nicknames)
            //
            if ( login.indexOf( "@" ) !== - 1 )
            {
                requestData.EmailAddress = login;
            }
            else
            {
                requestData.Nickname = login;
            }
        }


        if ( options )
        {
            if ( options.organizationUnitId )
            {
                requestData.OrganizationUnitID = options.organizationUnitId;
            }

            if ( options.confirmationToken ) {
                requestData.ConfirmationToken = options.confirmationToken;
            }

            if ( options.selectOrganizationByApplicationID ) {
                requestData.SelectOrganizationByApplicationID = options.selectOrganizationByApplicationID;
            }

            if ( options.publicLogin ) {
                requestData.PublicLogin = options.publicLogin;
            }

            if ( options.registerApiKey ) {
                requestData.RegisterApiKey = options.registerApiKey;
            }

            if ( options.context ) {
                requestData.Context = options.context;
            }
        }

        return new Promise( ( resolve: any, reject: any ) =>
        {
            service.call( requestData )
            .then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.Login, EVA.Core.Services.LoginResponse> ) =>
            {
                this.setData( serviceResult.response );

                // EVA.Core.AuthenticationResults.
                //  * NotAuthenticated = 0
                //  * NotAuthorized = 1
                //  * Authenticated = 2
                //  * NeedsTwoStepAuthentication = 3
                //  * NeedsOrganizationUnitID = 4
                //
                if ( this.authentication !== 2 )
                {
                    reject( this.authentication );
                }
                else
                {
                    resolve( serviceResult );
                }
            } )
            .catch( ( error ) =>
            {
                this.id = null;
                this.setData( {} );
                reject( error );
            } );
        } );
    }

    /**
     * Execute an explicit logout
     *
     * @return {Promise} A promise resolving when the call to the back-end has completed
     */
    public logout(): Promise<IEVAServiceResponse<EVA.Core.Services.Logout, EVA.Core.Services.LogoutResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // EVA.Core.Services.Logout
            //
            var service = new EVAService<EVA.Core.Services.Logout, EVA.Core.Services.LogoutResponse>( "Logout" );

            // Build the request data
            // The base class will add things like SessionID so keep typescript
            // happy we're using an 'any' type object here
            //
            var requestData: any = {};

            service.call( requestData )
            .then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.Logout, EVA.Core.Services.LogoutResponse> ) =>
            {
                // When the promise is successfully resolved we need to unset our details
                //
                if ( !serviceResult.response.LogoutSuccesful )
                {
                    if ( console ) { console.warn( "Server says logout wasn't successful. Proceeding to clear local session..." ); }
                }

                // Purge user data
                //
                this.data = null;

                resolve( serviceResult );
            } )
            .catch( reject );
        } );
    }

    /**
     * Retrieves third party login options
     *
     * @return {Promise} A promise resolving when the call to the back-end has completed
     */
    public getThirdPartyLoginOptions(): Promise<IEVAServiceResponse<EVA.Core.Services.GetAvailableThirdPartiesForLogin, EVA.Core.Services.GetAvailableThirdPartiesForLoginResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            var service = new EVAService<EVA.Core.Services.GetAvailableThirdPartiesForLogin, EVA.Core.Services.GetAvailableThirdPartiesForLoginResponse>( "GetAvailableThirdPartiesForLogin" );

            // Build the request data
            // The base class will add things like SessionID so keep typescript
            // happy we're using an 'any' type object here
            //
            var requestData: any = {};

            service.call( requestData )
            .then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.GetAvailableThirdPartiesForLogin, EVA.Core.Services.GetAvailableThirdPartiesForLoginResponse> ) =>
            {
                resolve( serviceResult.response );
            } )
            .catch( reject );
        } );
    }

    /**
     * Requests third party login.
     * Response will contain a redirect url that can be used to perform the login
     *
     * @return {Promise} A promise resolving when the call to the back-end has completed
     */
    public requestThirdPartyLogin( loginRequest: EVA.Core.Services.RequestThirdPartyLogin ): Promise<IEVAServiceResponse<EVA.Core.Services.RequestThirdPartyLogin, EVA.Core.Services.RequestThirdPartyLoginResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            var service = new EVAService<EVA.Core.Services.RequestThirdPartyLogin, EVA.Core.Services.RequestThirdPartyLoginResponse>( "RequestThirdPartyLogin" );

            service.call( loginRequest )
            .then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.RequestThirdPartyLogin, EVA.Core.Services.RequestThirdPartyLoginResponse> ) =>
            {
                resolve( serviceResult.response );
            } )
            .catch( reject );
        } );
    }

    /**
     * Perform third party login with a token
     *
     * @return {Promise} A promise resolving when the call to the back-end has completed
     */
    public authenticateWithThirdPartyLogin( loginRequest: EVA.Core.Services.AuthenticateWithThirdPartyLogin ): Promise<IEVAServiceResponse<EVA.Core.Services.AuthenticateWithThirdPartyLogin, EVA.Core.Services.AuthenticateWithThirdPartyLoginResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            var service = new EVAService<EVA.Core.Services.AuthenticateWithThirdPartyLogin, EVA.Core.Services.AuthenticateWithThirdPartyLoginResponse>( "AuthenticateWithThirdPartyLogin" );

            service.call( loginRequest )
            .then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.AuthenticateWithThirdPartyLogin, EVA.Core.Services.AuthenticateWithThirdPartyLoginResponse> ) =>
            {
                this.authenticationToken = serviceResult.response.AuthenticationToken !== undefined ? serviceResult.response.AuthenticationToken : this.authenticationToken;

                resolve( serviceResult.response );
            } )
            .catch( reject );
        } );
    }

    /**
     * This call retrieves a paged list of orders belonging to the current user
     *
     * @param  {number}                         start           The item to start the list at. Defaults to 0 and is a 0-based index.
     * @param  {number}                         limit           The page size limit. Defaults to 15
     * @param  {EVA.Core.SortDirection}  sortDirection          The sorting direction for the list. Defaults to ascending
     * @return {Promise<customerOrderList>}                     A promise resolving when the call to the back-end has completed
     */
    public getOrders( limit: number = 15, start: number = 0, sortDirection: EVA.Core.SortDirection = 0 ): Promise<CustomerOrderList>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            var customerOrderList = new CustomerOrderList( limit, start, sortDirection, "CreationTime" );

            customerOrderList.fetch()
            .then( () =>
            {
                this.orders = customerOrderList;

                resolve( customerOrderList );
            } )
            .catch( ( error ) =>
            {
                reject( error );
            } );
        } );
    }
    /**
     * This call retrieves a single order
     * @param {number}              order_id                    The ID of the order
     * @param {boolean}             include_failed_payements    If set to true, return order even if the payment has not been completed
     * @return {Promies<Order>}                                 A promise resolving when the call to the back-end has completed
     */
    public getOrder( order_id: number, include_failed_payements: boolean): Promise<Order> {
        return new Promise( ( resolve: any, reject: any) =>
        {
            var service = new EVAService<EVA.Core.Services.GetOrder, EVA.Core.Services.GetOrderResponse>( "GetOrder" );

            var requestData: any = {
                OrderID: order_id,
                IncludeFailedPayments: include_failed_payements
            };

            service.call( requestData )
            .then( (result) => {
                resolve(result.response.Result);
            })
            .catch( ( error ) =>
            {
                reject( error );
            } );
        });
    }

    /**
     * Checks if the user has a newsletter subscription
     * @return {boolean} Indicates if the user has a newsletter subscription
     */
    public hasNewsletterSubscription(): boolean
    {
        // SubscriptionType is a bitfield that we need to test
        // for value 1 for NewsLetter subscription
        //
        return this.data !== undefined && ( this.data.SubscriptionType & 1 ) !== 0;
    }

    /**
     * Subscribes to the Newsletter using the users current email address
     * @return {Promise}    A promise resolving when the call to the back-end has completed
     */
    public subscribeToNewsletter(): Promise<IEVAServiceResponse<EVA.Core.Services.Subscribe, EVA.Core.Services.SubscribeResponse>>
    {
        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Create the service instance
            //
            var service = new EVAService<EVA.Core.Services.Subscribe, EVA.Core.Services.SubscribeResponse>( "Subscribe" );

            // Build the request data
            // The base class will add things like SessionID so keep typescript
            // happy we're using an 'any' type object here
            //
            var requestData: any = {
                EmailAddress: this.data.EmailAddress,
                Type:         1
            };

            service.call( requestData )
            .then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.Subscribe, EVA.Core.Services.SubscribeResponse> ) =>
            {
                if ( serviceResult.response.Success === true )
                {
                    resolve();
                }
                else
                {
                    reject( serviceResult );
                }
            } )
            .catch( ( error ) =>
            {
                reject( error );
            } );
        } );
    }

    /**
     * Unsubscribes from the Newsletter using the users current email address
     * @return {Promise}    A promise resolving when the call to the back-end has completed
     */
    public unsubscribeFromNewsletter(): Promise<IEVAServiceResponse<EVA.Core.Services.Unsubscribe, EVA.Core.Services.UnsubscribeResponse>>
    {
        // Create the service instance
        //
        var service = new EVAService<EVA.Core.Services.Unsubscribe, EVA.Core.Services.UnsubscribeResponse>( "Unsubscribe" );

        // Build the request data
        // The base class will add things like SessionID so keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: any = {
            EmailAddress: this.data.EmailAddress,
            Type:         1
        };

        return service.call( requestData );
    }

    /**
     * Confirms a Newsletter subscription using the provided token
     * @param  {string}     token   The subscription confirmation token
     * @return {Promise}            A promise resolving when the call to the back-end has completed
     */
    public confirmNewsletterSubscription( token: string ): Promise<IEVAServiceResponse<EVA.Core.Services.ConfirmSubscribe, EVA.Core.Services.ConfirmSubscribeResponse>>
    {
        // Create the service instance
        //
        var service = new EVAService<EVA.Core.Services.ConfirmSubscribe, EVA.Core.Services.ConfirmSubscribeResponse>( "ConfirmSubscribe" );

        // Build the request data
        // The base class will add things like SessionID so keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: any = {
            Token:  token,
            Type:   1
        };

        return service.call( requestData );
    }

    /**
     * Checks if the user has an SMS notification subscription
     * @return {boolean} Indicates if the user has an SMS notification subscription
     */
    public hasSmsNotificationSubscription(): boolean
    {
        // SubscriptionType is a bitfield that we need to test
        // for value 2 for NewsLetter subscription
        //
        return this.data !== undefined && ( this.data.SubscriptionType & 2 ) !== 0;
    }

    /**
     * Subscribes to the SMS Notifications using the users current email address
     * @return {Promise}    A promise resolving when the call to the back-end has completed
     */
    public subscribeToSMSNotifications(): Promise<IEVAServiceResponse<EVA.Core.Services.Subscribe, EVA.Core.Services.SubscribeResponse>>
    {
        // Create the service instance
        //
        var service = new EVAService<EVA.Core.Services.Subscribe, EVA.Core.Services.SubscribeResponse>( "Subscribe" );

        // Build the request data
        // The base class will add things like SessionID so keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: any = {
            MobileNumber: this.data.PhoneNumber,
            Type:         2
        };

        return service.call( requestData );
    }

    /**
     * Unsubscribes from the SMS Notifications using the users current email address
     * @return {Promise}    A promise resolving when the call to the back-end has completed
     */
    public unsubscribeFromSMSNotifications(): Promise<IEVAServiceResponse<EVA.Core.Services.Unsubscribe, EVA.Core.Services.UnsubscribeResponse>>
    {
        // Create the service instance
        //
        var service = new EVAService<EVA.Core.Services.Unsubscribe, EVA.Core.Services.UnsubscribeResponse>( "Unsubscribe" );

        // Build the request data
        // The base class will add things like SessionID so keep typescript
        // happy we're using an 'any' type object here
        //
        var requestData: any = {
            MobileNumber: this.data.PhoneNumber,
            Type:         2
        };

        return service.call( requestData );
    }

    /**
     * Update the user
     *
     * @return {Promise} A promise resolving when the call to the back-end has completed
     */
    public update(): Promise<IEVAServiceResponse<EVA.Core.Services.UpdateUser, EVA.Core.Services.UpdateUserResponse>>
    {
        var service = new EVAService<EVA.Core.Services.UpdateUser, EVA.Core.Services.UpdateUserResponse>( "UpdateUser" );

        var requestData: any = {
            ToUpdate: this.data
        };

        return new Promise( ( resolve, reject ) => {
            service.call( requestData )
            .then( ( serviceResult ) =>
            {
                this.setData( serviceResult.response );
                resolve( serviceResult );
            } )
            .catch( reject );
        } );
    }

    /**
     * Update the user's email address
     *
     * @return {Promise} A promise resolving when the call to the back-end has completed
     */
    public updateUserEmailAddress( newEmailAddress: string ): Promise<IEVAServiceResponse<EVA.Core.Services.UpdateUserEmailAddress, EVA.Core.RequestMessageWithEmptyResponse>>
    {
        var service = new EVAService<EVA.Core.Services.UpdateUserEmailAddress, EVA.Core.RequestMessageWithEmptyResponse>( "UpdateUserEmailAddress" );

        var requestData: any = {
            ID:              this.data.ID,
            NewEmailAddress: newEmailAddress,
        };

        // Update data on success
        //
        return new Promise( ( resolve, reject ) => {
            service.call( requestData )
            .then( ( serviceResult ) =>
            {
                this.setData( serviceResult.response );
                resolve( serviceResult );
            } )
            .catch( reject );
        } );
    };

    /**
     * Change the passowrd of this user
     *
     * @param  {string}  newPassword The new password
     * @param  {string}  oldPassword The current / old password
     * @return {Promise}             A promise resolving when the call to the back-end has completed
     */
    public changePassword( newPassword: string, oldPassword: string ): Promise<IEVAServiceResponse<EVA.Core.Services.ChangeUserPassword, EVA.Core.Services.ChangeUserPasswordResponse>>
    {
        var service = new EVAService<EVA.Core.Services.ChangeUserPassword, EVA.Core.Services.ChangeUserPasswordResponse>( "ChangeUserPassword" );

        var requestData: any = {
            NewPassword:    newPassword
        ,   OldPassword:    oldPassword
        ,   UserID:         this.data.ID
        };

        return new Promise( ( resolve: any, reject: any ) =>
        {
            // Perform the call and capture the resulting promise
            //
            service.call( requestData ).then( ( serviceResult: IEVAServiceResponse<EVA.Core.Services.ChangeUserPassword, EVA.Core.Services.ChangeUserPasswordResponse> ) =>
            {
                if ( serviceResult.response.Success === true )
                {
                    resolve();
                }
                else
                {
                    reject( serviceResult.response.Result );
                }
            } )
            .catch( reject );
        } );
    }
}

export class User extends FetchableItem<EVA.Core.UserDto, EVA.Core.Services.GetUser, EVA.Core.Services.GetUserResponse>
{
    public isAnonymous:                     boolean;
    public isEmployee:                      boolean;
    public isCustomer:                      boolean;

    /**
     * Retrieves specific user data
     * @return {Promise} A promise that is fulfilled when the EVA Core service responds
     */
    public fetch(): Promise<IEVAServiceResponse<EVA.Core.Services.GetUser, EVA.Core.Services.GetUserResponse>>
    {
        // Build the request data
        //
        var requestData: EVA.Core.Services.GetUser = <EVA.Core.Services.GetUser> {
            ID: this.id
        };

        return super.fetch( "GetUser", requestData );
    }

    /**
     * @param  {any}                                  response  Service response containing the user data
     * @return {EVA.Core.UserDto}                   The user data
     */
    public setData( response: any = {} ): EVA.Core.UserDto
    {
        // Set the user data
        //
        this.data = response.Result;

        this.isEmployee             = this.data  && ( this.data.Type & 1 ) !== 0;
        this.isCustomer             = this.data  && ( this.data.Type & 2 ) !== 0;
        this.isAnonymous            = !this.data || ( this.data.Type & 4 ) !== 0;

        return this.data;
    }
}
