/**
 * This module provides access to an EVA Service call and ensures all required
 * settings, headers and call conditions are met
 *
 * @preferred
 */

import { SettingsManager                                        } from "./settings";
import { XHR, IXMLHttpRequestResponse, EXMLHttpRequestMethod    } from "./xhr";
import { serviceCache                                           } from "./service-cache";
import * as _                                                     from "lodash";

 /**
  * Any response is returned along with its HTTP responseStatus code
  * @type {interface}
  */
export interface IEVAServiceResponse<REQ, RES>
{
    request:         REQ;
    response:        RES;
    status:          number;
    xhr:             XMLHttpRequest;
}

/**
 * Wrapper class for a call to the EVA back-end
 * Ensure all proper settings are applied and handles all the type casting of
 * request and response for the XMLHttpRequest calls
 * @type {class}
 */
export class EVAService<REQ, RES>
{
    /**
     * Contains the name of the EVA service
     * @type {string}
     */
    private name: string;

    /**
     * Contains the path of the EVA service
     * @type {string}
     */
    private path: string;

    /**
     * Contains the request method of the EVA service
     * @type {EXMLHttpRequestMethod}
     */
    private method: EXMLHttpRequestMethod;

    /**
     * Constructor for the EVAService class
     * @param  {string} name    The name of the EVA service to call
     * @return {void}           This method returns nothing
     */
    constructor( name: string, path?: string, method: EXMLHttpRequestMethod = EXMLHttpRequestMethod.POST )
    {
        this.name   = name || "GenericService";
        this.path   = path;
        this.method = method;
    }

    /**
     * Convenience method to get the current session ID
     * @return {string} The current sessionID
     */
    public getSessionID(): string
    {
        return SettingsManager.getSessionId();
    }

    /**
     * Performs the actual call to the backend
     * @param  {REQ}        requestData The data to be sent as the body for this call
     * @return {Promise}                Return a promise that will resolve with the call response
     */
    public call( requestData?: REQ, disableCache?: boolean, timeout?: number ): Promise<IEVAServiceResponse<REQ, RES>>
    {
        let serviceName = this.name;
        let callPromise: Promise<IEVAServiceResponse<REQ, RES>>;

        // Check for a cached service call response (unless explicitly disabled)
        // If a path has been defined we will only allow caching of GET requests
        // If it's a message style request we will only allow caching of GetXYZ and ListXYZ calls
        //
        let restGetCall: boolean    = ( this.path !== undefined && this.method === EXMLHttpRequestMethod.GET );
        let messageGetCall: boolean = ( this.path === undefined && serviceName.match(/^Get|^List/ ) !== null );

        if ( disableCache !== true && ( restGetCall || messageGetCall ) )
        {
            callPromise = serviceCache.get( serviceName, requestData, this.method, this.path );
            if ( callPromise )
            {
                // Return from cache
                //
                return callPromise;
            }
        }

        callPromise = <Promise<IEVAServiceResponse<REQ, RES>>> new Promise( ( resolve: any, reject: any ) =>
        {
            var headers: any = {};

            // Add user token if set
            //
            var userToken: string = SettingsManager.getAuthenticationToken();
            if ( typeof userToken !== "undefined" && userToken !== null )
            {
                headers.Authorization =  userToken;
            }

            // Add administrative headers
            //
            headers.Accept               = "application/json, text/javascript";
            headers.Authorization        = SettingsManager.getAuthenticationToken();
            headers[ "Accept-Language" ] = SettingsManager.getLanguage();
            headers[ "Content-Type" ]    = "application/json";
            headers.ClientVersion        = SettingsManager.getVersion();
            headers.ClientName           = SettingsManager.getClientName();

            // Prepare the request body
            // Make a copy of the request data or we mess up the service cache
            //
            var requestBody: any  = _.clone( requestData ) || {};

            // Add session ID if not provided
            //
            if ( !requestBody.SessionID )
            {
                requestBody.SessionID = SettingsManager.getSessionId();
            }

            var callURL = SettingsManager.getEndPointURL() + ( this.path ||  "/message/" + serviceName );
            var xhr     = new XHR();

            // Override default timeout if requested
            //
            if ( _.isNumber( timeout ) )
            {
                xhr.setTimeout( Math.max( 0, timeout ) );
            }

            // console.log( "[XHR] Performing call", callURL, requestBody, this.method );

            xhr.call( {
                method:         this.method,
                url:            callURL,
                contentType:    "application/json",
                headers:        headers,
                data:           requestBody
            } )
            .then( ( xhrResponse: IXMLHttpRequestResponse ) =>
            {
                // console.log( "[Service:" + serviceName + "] Service call completed. Status code:", xhrResponse.status );

                // Build the response structure ensure all types are cast correctly
                //
                var serviceResponse: IEVAServiceResponse<REQ, RES> = {
                    status:     xhrResponse.status,
                    request:    requestData,
                    response:   <RES>xhrResponse.response,
                    xhr:        xhrResponse.xhr
                };

                // Check the message response and determine if this is a success or failure
                // Using the xhrResponse here because it is cast as 'any'
                //
                if ( xhrResponse.response.Exception )
                {
                    reject( serviceResponse );
                }
                else
                {
                    resolve( serviceResponse );
                }
            } )
            .catch( function( error: any )
            {
                // console.error( "[Service:" + serviceName + "] Service call error", error.status );
                reject( error );
            } );
        } );

        // If the serviceName starts with one of the following we will need to purge the cache
        //
        // * Create
        // * Store
        // * Add
        // * Modify
        // * Cancel
        // * Delete
        // * Detach
        // * Attach
        //
        if ( serviceName.match( /^Create|^Store|^Add|^Modify|^Cancel|^Delete|^Detach|^Attach/ ) !== null )
        {
            serviceCache.flush();
        }
        else
        {
            serviceCache.add( serviceName, requestData, this.method, this.path, callPromise );
        }

        return callPromise;
    }
}
