import { serviceCache } from "./service-cache";

/**
 * The settings module provides the configured locations and authentication
 * details for the EVA back-end. All other EVA content and data classes rely on it.
 * This module is user internally in the client and exposes more methods then
 * the globally exposed settings namespace on the EVA Client
 *
 * NOTE: SettingsManager is the internal name. Externally it is simply exposed as 'settings'
 *
 * @preferred
 */

 /* tslint:disable */
 function uuid(a?: any): string
 {
     return a?(0|Math.random()*16).toString(16):(""+1e7+-1e3+-4e3+-8e3+-1e11).replace(/1|0/g,uuid);
 }
 /* tslint:enable */

/**
 * This declares the known settings for the EVA SDK
 */
export interface IEvaSettings {
    userAuthenticationToken?: string;
    siteAuthenticationToken?: string;
    applicationId?:           number;
    sessionId?:               string;
    endPointURL:              string;
    language:                 string;
    version:                  string;
    clientName:               string;
}

/**
 * Internal settings for all EVA module classes
 * @type {IEvaSettings}
 */
var settings: IEvaSettings = {
    endPointURL: "https://eva.newblack.io",
    language:    "nl-NL",
    version:     "0.1.0",
    clientName:  "eva-sdk-js",
    sessionId:   uuid()
};


/**
 * The settings manager provides methods to get and set EVA client options
 * @type {namespace}
 */
export namespace SettingsManager
{
    "use strict";

    /**
     * Generates a new session identifier for all future service calls
     * @return {string} The current session identifier
     */
    export function generateSessionID(): string {
        // Flush the service cache when starting a new session
        //
        serviceCache.flush();

        return settings.sessionId = uuid();
    }

    /**
     * Changes the currently configured site authentication token.
     * @return {string} An authorisation token
     */
    export function setSiteAuthenticationToken( token: string ): string {
        // Flush the service cache when changing token
        //
        serviceCache.flush();

        return settings.siteAuthenticationToken = token;
    }

    /**
     * Changes the currently configured user authentication token. This token
     * will be used in favor of the site authentication token.
     * @return {string} An authorisation token
     */
    export function setUserAuthenticationToken( token: string ): string {
        // Flush the service cache when changing token
        //
        serviceCache.flush();

        return settings.userAuthenticationToken = token;
    }

    /**
     * Return the currently configured authentication token
     * @return {string} An authorisation token
     */
    export function getAuthenticationToken(): string {
        return settings.userAuthenticationToken ? settings.userAuthenticationToken : settings.siteAuthenticationToken;
    }

    /**
     * Changes the currently configured session identifier
     * @return {string} A session identifier
     */
    export function setSessionId( sessionId: string ): string {
        // Flush the service cache when session
        //
        serviceCache.flush();

        return settings.sessionId = sessionId;
    }

    /**
     * Return the currently configured session identifier
     * @return {string} A session identifier
     */
    export function getSessionId(): string {
        return settings.sessionId;
    }

    /**
     * Return the currently configured end point URL
     * @return {string} An end point URL
     */
    export function getEndPointURL(): string {
        return settings.endPointURL;
    }

    /**
     * Changes the currently configured end point URL
     * @return {string} An end point URL
     */
    export function setEndPointURL( URL: string ): string {
        // Flush the service cache when changing end point
        //
        serviceCache.flush();

        return settings.endPointURL = URL;
    }

    /**
     * Return the currently configured application identifier
     * @return {number} An application identifier
     */
    export function getApplicationId(): number {
        return settings.applicationId;
    }

    /**
     * Changes the currently configured application identifier
     * @return {number} An application identifier
     */
    export function setApplicationId( applicationId: number ): number {
        return settings.applicationId = applicationId;
    }

    /**
     * Returns the currently configured client language
     * @return {string} Language code (ex. nl-NL)
     */
    export function getLanguage(): string {
        return settings.language;
    }

    /**
     * Changes the currently configured client language
     * @return {string} Language code (ex. nl-NL)
     */
    export function setLanguage( languageCode: string ): string {
        // Flush the service cache when changing language
        //
        serviceCache.flush();

        return settings.language = languageCode;
    }

    /**
     * Returns the current client version number which is in semver format
     * @return {string} Version number (ex. 0.1.0)
     */
    export function getVersion(): string {
        return settings.version;
    }

    /**
     * Returns the current client name
     * @return {string} Client name
     */
    export function getClientName(): string {
        return settings.clientName;
    }
}
