/**
 * This module provides access to XMLHttpRequest based service calling utilities
 *
 * @preferred
 */

import * as _      from "lodash";

/**
 * The default XMLHttpRequest transport provider
 * @type {any}
 */
var transportProvider: any;

/**
 * For node.js or alternative platforms you can use setTransportProvider to point to an XHR alternative
 * @param  {any}    alternativeXMLHttpRequest   Platform specific or alternative XMLHttpRequest implementation
 * @return {void}                               This method does not return anything
 */
export function setTransportProvider( alternativeXMLHttpRequest: any )
{
    "use strict";
    // console.log( "[XHR] Alternative provider set" );
    transportProvider = alternativeXMLHttpRequest;
}

/**
 * Create a new instance of an XMLHttpRequest object
 * @return {any} The XMLHttpRequest instance
 */
export function createTransport()
{
    "use strict";
    if ( transportProvider )
    {
        // console.log( "[XHR] Creating user provided transport" );
        return new transportProvider();
    }
    else if ( typeof XMLHttpRequest !== "undefined" )
    {
        // console.log( "[XHR] Creating default transport" );
        return new XMLHttpRequest();
    }
    else
    {
        throw "No XMLHttpRequest provider available";
    }
}

/**
 * Possible XMLHttpRequest call methods
 * @type {enum}
 */
export enum EXMLHttpRequestMethod {GET, POST, PUT, DELETE, PATCH}

/**
 * These are all the parameters that can be provided for an XMLHttpRequest call
 * @type {interface}
 */
export interface IXMLHttpRequestParameters {
    method:         EXMLHttpRequestMethod;
    url:            string;
    contentType?:   string;
    headers?:       any;
    data?:          any;
    username?:      string;
    password?:      string;
}

/**
 * Any response is returned along with its HTTP responseStatus code
 * @type {interface}
 */
export interface IXMLHttpRequestResponse {
    status:         number;
    response:       any;
    xhr:            XMLHttpRequest;
}

/**
 * Provides a wrapper for an XMLHttpRequest call
 * @type {class}
 */
export class XHR
{
    /**
     * The timeout value for an XMLHttpRequest call
     * @type {number}
     */
    private timeout: number = 30000;

    /**
     * Private fallback timer to ensure a timeout always occurs
     * @type {any}
     */
    private timer: any;

    /**
     * The XMLHttpRequest instance used for the call
     * @type {any}
     */
    private xhr: any;

    public appendURL( url: string, parameters: any ): string
    {
        var parameterString: string = "";

        switch ( typeof parameters )
        {
            case "string":
            case "number":
            {
                parameterString = parameters;
            } break;

            case "boolean":
            {
                parameterString = parameters ? "true" : "false";
            } break;

            case "object":
            {
                var parameterList: Array<any> = [];

                if ( _.isArray( parameters ) )
                {
                    parameterList = parameters;
                }
                else
                {
                    _.each( parameters, function( value, key )
                    {
                        if ( value !== undefined ) {
                            parameterList.push( `${key}=${value}` );
                        }
                    } );
                }

                parameterString = parameterList.join( "&" );
            } break;

            default:
            {
                // NOOP
            }
        }


        // Don't append anything when an empty parameterString resulted.
        // Otherwise, if a question mark (?) is already present in the URL then append as an extra
        // parameter with an ampersand (&) instead.
        //
        if ( parameterString !== "" )
        {
            url += ( /\?/.test( url ) ? "&" : "?" ) + parameterString;
        }

        return url;
    }

    /**
     * Sets the XMLHttpRequest timeout to another value then the default 30 seconds
     * @param  {number} timeout The new timeout value in milliseconds
     * @return {number}         A timeout in milliseconds
     */
    public setTimeout( timeout: number )
    {
        return this.timeout = timeout;
    }

    /**
     * Return the current XMLHttpRequest timeout value
     * @return {number} A timeout in milliseconds
     */
    public getTimeout()
    {
        return this.timeout;
    }

    /**
     * Performs an XMLHttpRequest call
     * @param  {IXMLHttpRequestParameters}  params  The call parameters
     * @return {Promise<IXMLHttpRequestResponse>} A promise that will resolve or reject depending on the call response
     */
    public call( params: IXMLHttpRequestParameters ): Promise<IXMLHttpRequestResponse>
    {
        return <Promise<IXMLHttpRequestResponse>> new Promise( ( resolve: any, reject: any ) =>
        {
            // Create the XMLHttpRequest instance
            //
            this.xhr = createTransport();

            // Start the fallback timeout timer with a half second extra buffer
            //
            this.timer = setTimeout( () =>
            {
                // Create an error response
                //
                reject( {
                    status:     408,
                    response:   "Request timed out"
                } );
            }, this.timeout + 500 );

            // Setup the onreadystatechange listener which listens to the actual response
            //
            this.xhr.onreadystatechange = () =>
            {
                // console.log( "[XHR] readystate change", this.xhr.readyState );

                if ( this.xhr.readyState === 4 )
                {
                    // Stop our fallback timer
                    //
                    clearTimeout( this.timer );

                    // You are not allowed to access the status before readystate 4
                    //
                    var responseStatus: number = parseInt( this.xhr.status, 10 );
                    var responseHeaders: string = this.xhr.getAllResponseHeaders();

                    // Check if the response status is a success
                    //
                    if ( ( responseStatus >= 200 && responseStatus < 300 ) || responseStatus === 1223 )
                    {
                        // Internet Explorer mangles the 204 no content status code hence the 1223 check
                        //
                        if ( responseStatus === 1223 )
                        {
                            responseStatus = 204;
                        }

                        // Create the success response
                        //
                        var response: string = this.xhr.responseText;
                        if ( params.contentType === "application/json" )
                        {
                            // Try to JSON parse the response before returning it
                            //
                            try {
                                resolve( {
                                    status:   responseStatus,
                                    response: JSON.parse( response ),
                                    xhr:      this.xhr
                                } );
                            } catch ( jsonParseException )
                            {
                                if ( console && typeof console.warn === "function" )
                                {
                                    console.warn( "[XHR] Error parsing JSON response", jsonParseException );
                                }
                                resolve( {
                                    status:   responseStatus,
                                    response: response,
                                    xhr:      this.xhr
                                } );
                            }
                        }
                        else
                        {
                            resolve( {
                                status:   responseStatus,
                                response: response,
                                xhr:      this.xhr
                            } );
                        }
                    }
                    else
                    {
                        // Create an error response
                        //
                        var errorResponse: string   = this.xhr.responseText;
                        var statusText: string      = this.xhr.statusText;


                        if ( params.contentType === "application/json" )
                        {
                            // Try to JSON parse the error response before returning it
                            //
                            try {
                                reject( {
                                    status:             responseStatus,
                                    statusText:         statusText,
                                    response:           JSON.parse( errorResponse ),
                                    responseHeaders:    responseHeaders,
                                    xhr:                this.xhr
                                } );
                            } catch ( jsonParseException )
                            {
                                // console.warn( "[XHR] Error parsing JSON error response", jsonParseException );
                                reject( {
                                    status:             responseStatus,
                                    statusText:         statusText,
                                    response:           errorResponse,
                                    responseHeaders:    responseHeaders,
                                    xhr:                this.xhr,
                                    exception:          jsonParseException
                                } );
                            }
                        }
                        else
                        {
                            reject( {
                                status:             responseStatus,
                                statusText:         statusText,
                                response:           errorResponse,
                                responseHeaders:    responseHeaders,
                                xhr:                this.xhr
                            } );
                        }
                    }
                }
            };

            // Add the request data to the URL if needed
            // Remember that we need to do this before the @open call
            //
            if ( params.data && params.method === EXMLHttpRequestMethod.GET )
            {
                params.url  = this.appendURL( params.url, params.data );
                params.data = null;
            }

            // Open the XMLHttpRequest
            //
            // console.log( "[XHR] Opening call", EXMLHttpRequestMethod[ params.method ], params.url, true, params.username, params.password );
            this.xhr.open( EXMLHttpRequestMethod[ params.method ], params.url, true, params.username, params.password );

            // Set the timeout value
            // Do this afer .open because IE will throw a hissy fit if you don't
            //
            this.xhr.timeout = this.timeout;

            // Prepare the call data
            //
            var callData: string;
            var dataType: string = typeof params.data;
            if ( dataType === "string" )
            {
                // Change new lines in the request body to match protocol specs
                //
                callData = params.data.replace( /\r?\n/g, "\r\n" );
            }
            else if ( !!params.data && ( dataType === "object" || dataType === "function" ) )
            {
                // Stringify objects before transport
                //
                callData = JSON.stringify( params.data ).replace( /\r?\n/g, "\r\n" );
            }

            // Set all the request headers
            //
            dataType = typeof params.headers;
            if ( !!params.headers && ( dataType === "object" || dataType === "function" ) )
            {
                for ( var name in params.headers)
                {
                    if ( params.headers.hasOwnProperty( name ) && typeof params.headers[ name ] === "string" )
                    {
                        this.xhr.setRequestHeader( name, params.headers[ name ] );
                    }
                }
            }

            // Check for missing ContentType header
            //
            if ( !params.headers[ "Content-Type" ] && params.contentType )
            {
                this.xhr.setRequestHeader( "Content-Type", params.contentType );
            }

            // Let's try to perform the call and catch any exceptions to ensure
            // we can always at least reject the promise
            //
            try
            {
                // console.log( "[XHR] Sending data", callData );
                this.xhr.send( callData );
            } catch ( xhrError )
            {
                if ( console && typeof console.error === "function" )
                {
                    console.error( "[XHR] Error during request", xhrError );
                }

                reject( {
                    status:     500,
                    response:   xhrError
                } );
            }
        } );
    }
}
